@@ -1,5 +1,5 @@
-(* $Id: gwb2ged.ml,v 4.7 2002/11/08 18:54:55 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: gwb2ged.ml,v 4.12 2004/12/14 09:30:09 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -394,7 +394,7 @@ value ged_ind_attr_str base oc per =
 ;
 
 value ged_famc base (per_sel, fam_sel) oc asc =
-  match asc.parents with
+  match parents asc with
   [ Some ifam ->
       if fam_sel ifam then
         fprintf oc "1 FAMC @F%d@\n" (Adef.int_of_ifam ifam + 1)
@@ -522,7 +522,7 @@ value ged_comment base oc fam =
 ;
 
 value has_personal_infos base per asc =
-  if asc.parents <> None then True
+  if parents asc <> None then True
   else if sou base per.first_name <> "?" then True
   else if sou base per.surname <> "?" then True
   else if per.birth <> Adef.codate_None then True
@@ -562,13 +562,13 @@ value ged_fam_record base ((per_sel, fam_sel) as sel) oc i =
     fprintf oc "0 @F%d@ FAM\n" (i + 1);
     ged_marriage base oc fam;
     ged_divorce base oc fam;
-    if has_personal_infos base (poi base cpl.father) (aoi base cpl.father) &&
-       per_sel cpl.father then
-      fprintf oc "1 HUSB @I%d@\n" (Adef.int_of_iper cpl.father + 1)
+    if has_personal_infos base (poi base (father cpl)) (aoi base (father cpl)) &&
+       per_sel (father cpl) then
+      fprintf oc "1 HUSB @I%d@\n" (Adef.int_of_iper (father cpl) + 1)
     else ();
-    if has_personal_infos base (poi base cpl.mother) (aoi base cpl.mother) &&
-       per_sel cpl.mother then
-      fprintf oc "1 WIFE @I%d@\n" (Adef.int_of_iper cpl.mother + 1)
+    if has_personal_infos base (poi base (mother cpl)) (aoi base (mother cpl)) &&
+       per_sel (mother cpl) then
+      fprintf oc "1 WIFE @I%d@\n" (Adef.int_of_iper (mother cpl) + 1)
     else ();
     Array.iter (ged_child base sel oc) des.children;
     ged_fsource base oc fam;
@@ -605,17 +605,21 @@ value gwb2ged base ifile ofile anc desc mem =
   in
   do {
     if not mem then
+(*
       let _ = base.data.persons.array () in
+*)
       let _ = base.data.ascends.array () in
       let _ = base.data.unions.array () in
       let _ = base.data.couples.array () in
+(*
       let _ = base.data.families.array () in
+*)
       let _ = base.data.descends.array () in ()
     else ();
     let oc = if ofile = "" then stdout else open_out ofile in
     let ((per_sel, fam_sel) as sel) =
       Select.functions base anc desc surnames.val None no_spouses_parents.val
-        censor.val with_siblings.val
+        censor.val with_siblings.val (-1)
     in
     ged_header base oc ifile ofile;
     flush oc;
@@ -738,6 +742,7 @@ value anonfun s =
 value main () =
   do {
     Argl.parse speclist anonfun errmsg;
+    Secure.set_base_dir (Filename.dirname ifile.val);
     let anc =
       if anc_1st.val <> "" then
         if anc_2nd.val = "" then do {
