@@ -1,6 +1,6 @@
 (* camlp4r ../src/pa_lock.cmo *)
-(* $Id: gwtp.ml,v 4.17 2002/02/14 10:19:37 ddr Exp $ *)
-(* (c) Copyright 2002 INRIA *)
+(* $Id: gwtp.ml,v 4.24 2004/12/14 09:53:16 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Printf;
 
@@ -9,6 +9,7 @@ value gwtp_dst = ref (Filename.concat ".." "gwtp_dst");
 value gwtp_log = ref "";
 value gwtp_etc = ref "";
 value gw_site = ref "";
+value no_upload = ref False;
 value token_tmout = ref 900.0;
 
 value filename_basename str =
@@ -147,6 +148,87 @@ value template_fname env fname =
   List.fold_right Filename.concat [gwtp_etc.val; "lang"] (fname ^ ".txt")
 ;
 
+value lindex s c =
+  pos 0 where rec pos i =
+    if i == String.length s then None
+    else if s.[i] == c then Some i
+    else pos (i + 1)
+;
+
+value input_lexicon lang =
+  let t = Hashtbl.create 501 in
+  try
+    let ic =
+      open_in
+        (List.fold_right Filename.concat [gwtp_etc.val; "lang"]
+           "lexicon.txt")
+    in
+    let derived_lang =
+      match lindex lang '-' with
+      [ Some i -> String.sub lang 0 i
+      | _ -> "" ]
+    in
+    try
+      do {
+        try
+          while True do {
+            let k =
+              find_key (input_line ic) where rec find_key line =
+                if String.length line < 4 then find_key (input_line ic)
+                else if String.sub line 0 4 <> "    " then
+                  find_key (input_line ic)
+                else line
+            in
+            let k = String.sub k 4 (String.length k - 4) in
+            let rec loop line =
+              match lindex line ':' with
+              [ Some i ->
+                  let line_lang = String.sub line 0 i in
+                  do {
+                    if line_lang = lang ||
+                       line_lang = derived_lang && not (Hashtbl.mem t k) then
+                      let v =
+                        if i + 1 = String.length line then ""
+                        else
+                          String.sub line (i + 2) (String.length line - i - 2)
+                      in
+                      Hashtbl.add t k v
+                    else ();
+                    loop (input_line ic)
+                  }
+              | None -> () ]
+            in
+            loop (input_line ic)
+          }
+        with
+        [ End_of_file -> () ];
+        close_in ic;
+        t
+      }
+    with e ->
+      do { close_in ic; raise e }
+  with
+  [ Sys_error _ -> t ]
+;
+
+value unfreeze_lexicon =
+  let lexicon = ref None in
+  fun lang ->
+    match lexicon.val with
+    [ Some lex -> lex
+    | None ->
+        let lex = input_lexicon lang in
+        do {
+          lexicon.val := Some lex;
+          lex
+        } ]
+;
+
+value transl lang w =
+  let lexicon = unfreeze_lexicon lang in
+  try Hashtbl.find lexicon w with [ Not_found -> "[" ^ w ^ "]" ]
+;
+
 value copy_template genv (varenv, filenv) env fname =
   let lang =
     match HttpEnv.getenv genv "lang" with
@@ -176,20 +258,26 @@ value copy_template genv (varenv, filenv) env fname =
                 let v = get_variable ic in
                 try print_string (quote_escaped (List.assoc v filenv)) with
                 [ Not_found -> () ]
+            | 'L' ->
+                let v = get_variable ic in
+                let lang_def = transl lang " !languages" in
+                print_string (Translate.language_name v lang_def)
             | c ->
                 try print_string (List.assoc c env) with
                 [ Not_found -> do { print_char '%'; print_char c; } ] ]
         | '[' ->
             let s =
               let c = input_char ic in
+              let s =
+                loop 0 (if c = '\n' then input_char ic else c)
+                where rec loop len c =
+                  if c = ']' then Buff.get len
+                  else loop (Buff.store len c) (input_char ic)
+              in
               if c = '\n' then
-                let s =
-                  loop 0 (input_char ic) where rec loop len c =
-                    if c = ']' then Buff.get len
-                    else loop (Buff.store len c) (input_char ic)
-                in
-                Translate.inline lang '%' (macro env) s
-              else "[" ^ String.make 1 c
+                let (s, alt) = Translate.inline lang '%' (macro env) s in
+                if alt then "[" ^ s ^ "]" else s
+              else transl lang s
             in
             print_string s
         | c -> print_char c ]
@@ -582,6 +670,7 @@ value make_temp env b =
     printf "persons: %d\n" base.Def.data.Def.persons.Def.len;
     printf "families: %d\n\n" base.Def.data.Def.families.Def.len;
     flush stdout;
+    Secure.set_base_dir (Filename.dirname bdir);
     Iobase.output bdir base;
     flush stdout;
   }
@@ -604,6 +693,10 @@ value copy_temp b =
       if Sys.file_exists old_forum then
         sys_copy old_forum (Filename.concat bdir "forum")
       else ();
+      let old_wiznotes = Filename.concat dir_old_gwb "wiznotes" in
+      if Sys.file_exists old_wiznotes then
+        Sys.rename old_wiznotes (Filename.concat bdir "wiznotes")
+      else ()
     }
     else ();
     Sys.rename bdir dir_gwb;
@@ -1033,7 +1126,10 @@ value gwtp_main str env b tok =
 <ul>
 " b b;
     if config_exists then do {
-      if Sys.file_exists (Filename.concat gwtp_etc.val "ged2gwb") then do {
+      if no_upload.val then ()
+      else if
+        Sys.file_exists (Filename.concat gwtp_etc.val "ged2gwb")
+      then do {
         printf "<li>Upload from\n<ul>\n";
         printf "<li><a href=\"%s?m=UPL;b=%s;t=%s\">database files</a>\n"
           gwtp_comm b tok;
@@ -1142,17 +1238,21 @@ value gwtp () =
     match HttpEnv.getenv env "m" with
     [ Some "LOGIN" -> gwtp_check_login from str env gwtp_main
     | Some "MAIN" -> gwtp_logged from str env gwtp_main
-    | Some "UPL" -> gwtp_logged from str env gwtp_upload
     | Some "DNL" -> gwtp_logged from str env gwtp_download
-    | Some "UPG" -> gwtp_logged from str env gwtp_upload_gedcom
     | Some "CNF" -> gwtp_logged from str env gwtp_config
-    | Some "SEND" -> gwtp_logged from str env gwtp_send
-    | Some "SEND_GEDCOM" -> gwtp_logged from str env gwtp_send_gedcom
     | Some "RECV" -> gwtp_logged from str env gwtp_receive
     | Some "SCNF" -> gwtp_logged from str env gwtp_setconf
     | Some "LOG" -> gwtp_logged from str env gwtp_print_log
     | Some "ACCW" -> gwtp_logged from str env (gwtp_print_accesses True)
     | Some "ACCF" -> gwtp_logged from str env (gwtp_print_accesses False)
+    | Some "UPL" when not no_upload.val ->
+        gwtp_logged from str env gwtp_upload
+    | Some "UPG" when not no_upload.val ->
+        gwtp_logged from str env gwtp_upload_gedcom
+    | Some "SEND" when not no_upload.val ->
+        gwtp_logged from str env gwtp_send
+    | Some "SEND_GEDCOM" when not no_upload.val ->
+        gwtp_logged from str env gwtp_send_gedcom
     | Some _ -> gwtp_invalid_request str env
     | None -> gwtp_login str env ];
     flush stdout;
@@ -1174,6 +1274,7 @@ value speclist =
     gwtp_tmp.val);
    ("-site", Arg.String (fun x -> gw_site.val := x),
     "<url>: site (if any) where databases are accomodated");
+   ("-noup", Arg.Set no_upload, "no upload");
    ("-tmout", Arg.Float (fun x -> token_tmout.val := x),
     "<sec>: tokens time out; default = " ^
     string_of_float token_tmout.val ^ " sec") ]
