@@ -1,5 +1,5 @@
 (* camlp4r *)
-(* $Id: setup.ml,v 4.42 2002/11/14 16:54:29 ddr Exp $ *)
+(* $Id: setup.ml,v 4.53 2004/12/14 15:03:05 ddr Exp $ *)
 
 open Printf;
 
@@ -119,19 +119,21 @@ value abs_setup_dir () =
 
 value trailer conf =
   do {
-    Wserver.wprint "<p>\n";
+    Wserver.wprint "<p />\n";
     if conf.comm = "" then ()
     else
       Wserver.wprint "
-<img src=\"file://%s/images/gwlogo.gif\"
- width=64 height=72 align=right>
-<br>
-" (slashify (abs_setup_dir ()));
+<img src=\"images/gwlogo.png\" width=\"64\" height=\"72\" align=\"right\" alt=\"logo\" />
+<br />
+";
     Wserver.wprint "
-<hr><font size=-1><em>(c) Copyright 2002 INRIA -
+<hr><font size=\"-1\"><em>Copyright (c) 1998-2005 INRIA -
 GeneWeb %s</em></font>" Version.txt;
-    Wserver.wprint "<br>";
-    Wserver.wprint "</body>\n"
+    Wserver.wprint "<br />";
+    
+    (* finish the html page *)
+    Wserver.wprint "</body>\n";
+    Wserver.wprint "</html>\n";
   }
 ;
 
@@ -249,10 +251,10 @@ value parameters =
           | None ->
               if s = "none" then loop comm env
               else if s = "on" then loop (comm ^ " -" ^ k) env
-              else if s.[0] = '_' then loop (comm ^ " -" ^ k ^ s) env
+              else if s.[0] = '_' then loop (comm ^ " -" ^ k ^ stringify s) env
               else if s.[String.length s - 1] = '_' then
                 loop (comm ^ " -" ^ s ^ k) env
-              else loop (comm ^ " -" ^ k ^ " " ^ s) env ]
+              else loop (comm ^ " -" ^ k ^ " " ^ stringify s) env ]
     | [] -> comm ]
 ;
 
@@ -320,7 +322,6 @@ value macro conf =
   | 'a' -> strip_spaces (s_getenv conf.env "anon")
   | 'c' -> stringify (Filename.concat setup_dir.val conf.comm)
   | 'd' -> conf.comm
-  | 'f' -> slashify (abs_setup_dir ())
   | 'i' -> strip_spaces (s_getenv conf.env "i")
   | 'l' -> conf.lang
   | 'm' -> server_string conf
@@ -415,7 +416,9 @@ value rec copy_from_stream conf print strm =
           match Stream.peek strm with
           [ Some '\n' ->
               let s = parse_upto ']' strm in
-              print (Translate.inline conf.lang '%' (macro conf) s)
+              let (s, alt) = Translate.inline conf.lang '%' (macro conf) s in
+              let s = if alt then "[" ^ s ^ "]" else s in
+              print s
           | _ ->
               let s =
                 loop 0 where rec loop len =
@@ -576,7 +579,12 @@ and print_selector conf print =
       List.map
         (fun x ->
            let d =
-             if x = ".." then Filename.dirname sel
+             if x = ".." then
+	       ifdef WIN95 then
+	         if sel.[String.length sel - 1] <> '\\' then
+		   Filename.dirname sel ^ "\\"
+		 else Filename.dirname sel
+	       else Filename.dirname sel
              else Filename.concat sel x
            in
            let x = if is_directory d then Filename.concat x "" else x in
@@ -695,7 +703,7 @@ value error conf str =
 value exec_f comm =
   let s = comm ^ " > " ^ "comm.log" in
   do {
-    eprintf "$ cd %s\n" (Sys.getcwd ());
+    eprintf "$ cd \"%s\"\n" (Sys.getcwd ());
     flush stderr;
     eprintf "$ %s\n" s;
     flush stderr;
@@ -1015,7 +1023,7 @@ value recover_2 conf =
   let rc =
     try
       do {
-        eprintf "$ cd %s\n" init_dir;
+        eprintf "$ cd \"%s\"\n" init_dir;
         flush stderr;
         Sys.chdir init_dir;
         let c =
@@ -1031,7 +1039,7 @@ value recover_2 conf =
   in
   let rc =
     if rc = 0 then do {
-      eprintf "$ cd %s\n" dir;
+      eprintf "$ cd \"%s\"\n" dir;
       flush stderr;
       Sys.chdir dir;
       let c =
@@ -1095,7 +1103,7 @@ value cleanup_1 conf =
   in
   let in_base_dir = in_base ^ ".gwb" in
   do {
-    eprintf "$ cd %s\n" (Sys.getcwd ());
+    eprintf "$ cd \"%s\"\n" (Sys.getcwd ());
     flush stderr;
     let c =
       Filename.concat setup_dir.val "gwu" ^ " " ^ in_base ^ " -o tmp.gw"
@@ -1213,7 +1221,7 @@ value merge_1 conf =
   let bases = selected conf.env in
   let dir = Sys.getcwd () in
   do {
-    eprintf "$ cd %s\n" dir;
+    eprintf "$ cd \"%s\"\n" dir;
     flush stderr;
     Sys.chdir dir;
     let rc =
@@ -1447,6 +1455,76 @@ value consang conf ok_file =
   }
 ;
 
+value separate_slashed_filename s =
+  loop 0 where rec loop i =
+    match try Some (String.index_from s i '/') with [ Not_found -> None ] with
+    [ Some j ->
+        if j > i then [String.sub s i (j - i) :: loop (j + 1)]
+        else loop (j + 1)
+    | None ->
+        if i >= String.length s then []
+        else [String.sub s i (String.length s - i)] ]
+;
+
+value start_with s x =
+  let slen = String.length s in
+  let xlen = String.length x in
+  slen >= xlen && String.sub s 0 xlen = x
+;
+
+value end_with s x =
+  let slen = String.length s in
+  let xlen = String.length x in
+  slen >= xlen && String.sub s (slen - xlen) xlen = x
+;
+
+value print_typed_file conf typ fname =
+  let ic_opt =
+    try Some (open_in_bin fname) with
+    [ Sys_error _ -> None ]
+  in
+  match ic_opt with
+  [ Some ic ->
+      do {
+        Wserver.http "";
+        Wserver.wprint "Content-type: %s" typ;
+        nl ();
+        Wserver.wprint "Content-length: %d" (in_channel_length ic);
+        nl (); nl ();
+	try
+          while True do {
+            let c = input_char ic in
+            Wserver.wprint "%c" c
+          }
+        with [ End_of_file -> () ];
+        close_in ic;
+      }
+  | None ->
+      let title _ = Wserver.wprint "Error" in
+      do {
+        header conf title;
+        Wserver.wprint "<ul><li>\n";
+        Wserver.wprint "Cannot access file \"%s\".\n" fname;
+        Wserver.wprint "</ul>\n";
+        trailer conf;
+        raise Exit
+      } ]
+;
+
+value raw_file conf s =
+  let fname =
+    List.fold_left Filename.concat setup_dir.val
+      (separate_slashed_filename s)
+  in
+  let typ =
+    if end_with s ".png" then "image/png"
+    else if end_with s ".jpg" then "image/jpeg"
+    else if end_with s ".gif" then "image/gif"
+    else "text/html"
+  in
+  print_typed_file conf typ fname
+;
+
 value has_gwb_directories dh =
   try
     let rec loop () =
@@ -1496,7 +1574,9 @@ value setup_comm_ok conf =
   | "gwf_1" -> gwf_1 conf
   | "gwd" -> gwd conf
   | "gwd_1" -> gwd_1 conf
-  | x -> error conf ("bad command: \"" ^ x ^ "\"") ]
+  | x ->
+      if start_with x "doc/" || start_with x "images/" then raw_file conf x
+      else error conf ("bad command: \"" ^ x ^ "\"") ]
 ;
 
 value setup_comm conf comm =
@@ -1728,6 +1808,10 @@ value intro () =
     default_lang.val := default_setup_lang;
     let (gwd_lang, setup_lang) =
       if daemon.val then
+        let setup_lang =
+          if String.length lang_param.val < 2 then default_setup_lang
+          else lang_param.val
+        in
         ifdef UNIX then do {
           printf "To start, open location http://localhost:%d/\n"
             port.val;
@@ -1737,7 +1821,7 @@ value intro () =
             null_reopen [Unix.O_WRONLY] Unix.stdout
           }
           else exit 0;
-          (default_gwd_lang, default_setup_lang)
+          (default_gwd_lang, setup_lang)
         }
         else (default_gwd_lang, default_setup_lang)
       else do {
