@@ -1,11 +1,12 @@
 (* camlp4r ./def.syn.cmo ./pa_html.cmo *)
-(* $Id: ascend.ml,v 4.28 2002/12/26 14:26:18 ddr Exp $ *)
-(* Copyright (c) 2002 INRIA *)
+(* $Id: ascend.ml,v 4.39 2004/12/14 09:30:10 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
 open Gutil;
 open Util;
+open Printf;
 
 value limit_by_list conf =
   match p_getint conf.base_env "max_anc_level" with
@@ -20,8 +21,10 @@ value limit_by_tree conf =
 ;
 
 value max_ancestor_level conf base ip =
+(*
   let _ = base.data.ascends.array () in
   let _ = base.data.couples.array () in
+*)
   let x = ref 0 in
   let mark = Array.create base.data.persons.len False in
   let rec loop level ip =
@@ -29,10 +32,10 @@ value max_ancestor_level conf base ip =
     else do {
       mark.(Adef.int_of_iper ip) := True;
       x.val := max x.val level;
-      match (aget conf base ip).parents with
+      match parents (aget conf base ip) with
       [ Some ifam ->
           let cpl = coi base ifam in
-          do { loop (succ level) cpl.father; loop (succ level) cpl.mother }
+          do { loop (succ level) (father cpl); loop (succ level) (mother cpl) }
       | _ -> () ]
     }
   in
@@ -47,7 +50,7 @@ value text_to conf =
   | 3 -> transl conf "to the grandparents"
   | 4 -> transl conf "to the great-grandparents"
   | i ->
-      Printf.sprintf (ftransl conf "to the %s generation")
+      sprintf (ftransl conf "to the %s generation")
         (transl_nth conf "nth (generation)" i) ]
 ;
 
@@ -59,7 +62,7 @@ value text_level conf =
   | 3 -> transl conf "the grandparents"
   | 4 -> transl conf "the great-grandparents"
   | i ->
-      Printf.sprintf (ftransl conf "the %s generation")
+      sprintf (ftransl conf "the %s generation")
         (transl_nth conf "nth (generation)" i) ]
 ;
 
@@ -116,6 +119,19 @@ value print_choice conf base p effective_level =
           Wserver.wprint
             "- %s <input type=checkbox name=image value=on><br>\n"
             (capitale (transl_nth conf "image/images" 1));
+          Wserver.wprint
+            "- %s <input name=bd size=1 maxlength=2 value=0><br>\n"
+            (capitale (transl conf "border"));
+          Wserver.wprint "\
+<table><tr><td>-&nbsp;%s</td>
+<td><input type=radio name=color value=\"\" checked></td>\n"
+              (capitale (transl conf "color"));
+          List.iter
+            (fun c ->
+               Wserver.wprint "\
+<td bgcolor=%s><input type=radio name=color value=%s></td>\n" c c)
+            ["FFC0C0"; "FFFFC0"; "C0FFC0"; "C0FFFF"; "C0C0FF"; "FFC0FF"];
+          Wserver.wprint "</tr></table>\n";
         end;
         tag "td valign=top" begin
           Wserver.wprint "<input type=radio name=t value=L> %s\n"
@@ -189,9 +205,8 @@ value display_ancestor_menu conf base p =
            do {
              conf.henv := List.remove_assoc "iz" conf.henv;
              let reference _ _ _ s =
-               "<a href=\"" ^ commd conf ^ "iz=" ^
-                 string_of_int (Adef.int_of_iper p.cle_index) ^ "\">" ^ s ^
-                 "</a>"
+               sprintf "<a href=\"%siz=%d\">%s</a>" (commd conf)
+                 (Adef.int_of_iper p.cle_index) s
              in
              Wserver.wprint "%s"
                (gen_person_title_text reference std_access conf base p)
@@ -215,11 +230,11 @@ value display_ancestors_upto conf base max_level p =
   let rec loop level ip =
     if level < max_level then
       let x = aget conf base ip in
-      match x.parents with
+      match parents x with
       [ Some ifam ->
           let cpl = coi base ifam in
-          let pere = pget conf base cpl.father in
-          let mere = pget conf base cpl.mother in
+          let pere = pget conf base (father cpl) in
+          let mere = pget conf base (mother cpl) in
           let know_fath = know base pere in
           let know_moth = know base mere in
           if know_fath || know_moth then
@@ -228,14 +243,14 @@ value display_ancestors_upto conf base max_level p =
                 Wserver.wprint "<li type=square> ";
                 display_ancestor conf base pere;
                 Wserver.wprint "\n";
-                loop (succ level) cpl.father
+                loop (succ level) (father cpl)
               }
               else ();
               if know_moth then do {
                 Wserver.wprint "<li type=circle> ";
                 display_ancestor conf base mere;
                 Wserver.wprint "\n";
-                loop (succ level) cpl.mother
+                loop (succ level) (mother cpl)
               }
               else ();
             end
@@ -279,11 +294,11 @@ value next_generation conf base mark gpl =
              let n_fath = Num.twice n in
              let n_moth = Num.inc n_fath 1 in
              let a = aget conf base ip in
-             match a.parents with
+             match parents a with
              [ Some ifam ->
                  let cpl = coi base ifam in
-                 [GP_person n_fath cpl.father (Some ifam);
-                  GP_person n_moth cpl.mother (Some ifam) :: gpl]
+                 [GP_person n_fath (father cpl) (Some ifam);
+                  GP_person n_moth (mother cpl) (Some ifam) :: gpl]
              | None -> [GP_missing n ip :: gpl] ]
          | GP_interv None -> [gp :: gpl]
          | GP_interv (Some (n1, n2, x)) ->
@@ -489,7 +504,7 @@ value get_link all_gp ip =
 
 value print_persons_parents conf base all_gp p =
   let a = aget conf base p.cle_index in
-  match a.parents with
+  match parents a with
   [ Some ifam ->
       let cpl = coi base ifam in
       let string ip =
@@ -505,8 +520,8 @@ value print_persons_parents conf base all_gp p =
       Wserver.wprint ", %s\n"
         (transl_a_of_gr_eq_gen_lev conf
            (transl_nth conf "son/daughter/child" (index_of_sex p.sex))
-           (string cpl.father ^ "\n" ^ transl_nth conf "and" 0 ^ "\n" ^
-              string cpl.mother))
+           (string (father cpl) ^ "\n" ^ transl_nth conf "and" 0 ^ "\n" ^
+              string (mother cpl)))
   | None -> () ]
 ;
 
@@ -599,8 +614,8 @@ value print_family_long conf base ws wn all_gp ifam nth moth_nb =
   let cpl = coi base ifam in
   let des = doi base ifam in
   let auth =
-    authorized_age conf base (pget conf base cpl.father) &&
-    authorized_age conf base (pget conf base cpl.mother)
+    authorized_age conf base (pget conf base (father cpl)) &&
+    authorized_age conf base (pget conf base (mother cpl))
   in
   do {
     Wserver.wprint "<p>\n... ";
@@ -627,7 +642,7 @@ value print_family_long conf base ws wn all_gp ifam nth moth_nb =
           let n = get_link all_gp ipc in
           Wserver.wprint "<li>\n";
           stag "strong" begin
-            if pc.surname = (pget conf base cpl.father).surname then
+            if pc.surname = (pget conf base (father cpl)).surname then
               Wserver.wprint "%s"
                 (referenced_person_text_without_surname conf base pc)
             else Wserver.wprint "\n%s" (referenced_person_text conf base pc);
@@ -672,7 +687,7 @@ value print_other_families conf base wn all_gp excl_ifam moth_nb =
     }
   in
   let cpl = coi base excl_ifam in
-  do { print cpl.father (Num.sub moth_nb Num.one); print cpl.mother moth_nb }
+  do { print (father cpl) (Num.sub moth_nb Num.one); print (mother cpl) moth_nb }
 ;
 
 value print_generation_person_long conf base ws wn all_gp last_gen gp =
@@ -688,8 +703,8 @@ value print_generation_person_long conf base ws wn all_gp last_gen gp =
               let fam = foi base ifam in
               let cpl = coi base ifam in
               let auth =
-                authorized_age conf base (pget conf base cpl.father) &&
-                authorized_age conf base (pget conf base cpl.mother)
+                authorized_age conf base (pget conf base (father cpl)) &&
+                authorized_age conf base (pget conf base (mother cpl))
               in
               Wserver.wprint "... ";
               Wserver.wprint (relation_txt conf Male fam)
@@ -713,12 +728,12 @@ value print_generation_person_long conf base ws wn all_gp last_gen gp =
         end;
         print_person_long_info conf base (authorized_age conf base p) None p;
         Wserver.wprint ".\n";
-        match (aget conf base ip).parents with
+        match parents (aget conf base ip) with
         [ Some ifam ->
             let cpl = coi base ifam in
             let prec =
               if last_gen then
-                (get_link all_gp cpl.father, get_link all_gp cpl.mother)
+                (get_link all_gp (father cpl), get_link all_gp (mother cpl))
               else
                 let n1 = Num.twice n in
                 (Some n1, Some (Num.inc n1 1))
@@ -870,7 +885,7 @@ value print_notes_for_other_families conf base all_gp excl_ifam moth_nb =
     }
   in
   let cpl = coi base excl_ifam in
-  do { print cpl.father (Num.sub moth_nb Num.one); print cpl.mother moth_nb }
+  do { print (father cpl) (Num.sub moth_nb Num.one); print (mother cpl) moth_nb }
 ;
 
 value print_notes conf base all_gp ws =
@@ -1127,8 +1142,8 @@ value print_generation_missing_persons conf base title sp_incl gp =
           let cpl = coi base u.family.(0) in
           let (parent_name_index, conj) =
             match p.sex with
-            [ Male -> (0, cpl.mother)
-            | _ -> (1, cpl.father) ]
+            [ Male -> (0, (mother cpl))
+            | _ -> (1, (father cpl)) ]
           in
           Wserver.wprint "%s"
             (transl_a_of_b conf
@@ -1286,8 +1301,8 @@ value add_missing conf base spouses_included list =
           let cpl = coi base u.family.(0) in
           let (a, p) =
             match p.sex with
-            [ Male -> (A_husband_of, pget conf base cpl.mother)
-            | _ -> (A_wife_of, pget conf base cpl.father) ]
+            [ Male -> (A_husband_of, pget conf base (mother cpl))
+            | _ -> (A_wife_of, pget conf base (father cpl)) ]
           in
           [(a, p) :: list]
         else [(A_person, p) :: list]
@@ -1303,8 +1318,8 @@ value add_missing conf base spouses_included list =
           let cpl = coi base u.family.(0) in
           let (a, p) =
             match p.sex with
-            [ Male -> (A_surname_of_husband_of n, pget conf base cpl.mother)
-            | _ -> (A_surname_of_wife_of n, pget conf base cpl.father) ]
+            [ Male -> (A_surname_of_husband_of n, pget conf base (mother cpl))
+            | _ -> (A_surname_of_wife_of n, pget conf base (father cpl)) ]
           in
           if p_surname base p = "?" then list else [(a, p) :: list]
         else [(A_parents_of, p) :: list]
@@ -1559,13 +1574,14 @@ value print_missing_ancestors_alphabetically conf base v spouses_included p =
   }
 ;
 
-value tree_reference gv conf base p s =
+value tree_reference gv bd color conf base p s =
   if conf.cancel_links || is_hidden p then s
   else
     let im = p_getenv conf.env "image" = Some "on" in
-    "<a href=\"" ^ commd conf ^ "m=A;t=T;v=" ^ string_of_int gv ^ ";" ^
-      acces conf base p ^ (if im then ";image=on" else "") ^ "\">" ^ s ^
-      "</a>"
+    sprintf "<a href=\"%sm=A;t=T;v=%d;%s%s%s%s\">%s</a>"
+      (commd conf) gv (acces conf base p) (if im then ";image=on" else "")
+      (if bd > 0 then ";bd=" ^ string_of_int bd else "")
+      (if color <> "" then ";color=" ^ color else "") s
 ;
 
 type pos = [ Left | Right | Center | Alone ];
@@ -1634,21 +1650,32 @@ value rec enrich_tree lst =
     p: person *)
 value print_tree_with_table conf base gv p =
   let gv = min (limit_by_tree conf) gv in
+  let bd = match p_getint conf.env "bd" with [ Some x -> x | None -> 0 ] in
+  let color =
+    match Util.p_getenv conf.env "color" with
+    [ None -> ""
+    | Some x -> x ]
+  in
+  let td_prop =
+    match Util.p_getenv conf.env "td" with
+    [ Some x -> " " ^ x
+    | _ -> if color = "" then "" else " bgcolor=" ^ color ]
+  in
   let next_gen pol =
     List.fold_right
       (fun po list ->
          match po with
          [ Empty -> [Empty :: list]
          | Cell p _ _ _ ->
-             match (aget conf base p.cle_index).parents with
+             match parents (aget conf base p.cle_index) with
              [ Some ifam ->
                  let cpl = coi base ifam in
                  let fath =
-                   let p = pget conf base cpl.father in
+                   let p = pget conf base (father cpl) in
                    if know base p then Some p else None
                  in
                  let moth =
-                   let p = pget conf base cpl.mother in
+                   let p = pget conf base (mother cpl) in
                    if know base p then Some p else None
                  in
                  match (fath, moth) with
@@ -1687,7 +1714,7 @@ value print_tree_with_table conf base gv p =
         let txt =
           match po with
           [ Empty -> "&nbsp;"
-          | Cell p _ _ _ -> tree_reference gv conf base p "|" ]
+          | Cell p _ _ _ -> tree_reference gv bd color conf base p "|" ]
         in
         Wserver.wprint "%s" txt;
       end;
@@ -1704,12 +1731,18 @@ value print_tree_with_table conf base gv p =
           | Cell p _ _ _ ->
               let txt = person_title_text conf base p in
               let txt = down_reference p txt in
-              txt ^ Date.short_dates_text conf base p ]
+              let txt = txt ^ Date.short_dates_text conf base p in
+              let txt =
+                if bd > 0 || td_prop <> "" then
+                  sprintf
+                    "<table border=%d><tr>\
+                     <td align=center%s>%s</td></tr></table>"
+                    bd td_prop txt
+                else txt
+              in
+              txt ^ Dag.image_txt conf base p ]
         in
         Wserver.wprint "%s" txt;
-        match po with
-        [ Empty -> ()
-        | Cell p _ _ _ -> Wserver.wprint "%s" (Dag.image_txt conf base p) ];
       end;
       Wserver.wprint "\n"
     }
@@ -1734,7 +1767,8 @@ value print_tree_with_table conf base gv p =
         stag "td" "%s" (align po) begin
           let txt =
             match po with
-            [ Cell _ Right _ _ | Cell _ Center _ _ -> "<hr noshade size=1>"
+            [ Cell _ Right _ _ | Cell _ Center _ _ ->
+                "<hr noshade size=1 width=\"100%\">"
             | _ -> "&nbsp;" ]
           in
           Wserver.wprint "%s" txt;
@@ -1747,9 +1781,9 @@ value print_tree_with_table conf base gv p =
           match po with
           [ Empty -> "&nbsp;"
           | Cell _ Left _ _ ->
-              "<hr noshade size=1 width=\"50%%\" align=right>"
+              "<hr noshade size=1 width=\"50%\" align=right>"
           | Cell _ Right _ _ ->
-              "<hr noshade size=1 width=\"50%%\" align=left>"
+              "<hr noshade size=1 width=\"50%\" align=left>"
           | Cell _ Alone _ _ -> "|"
           | Cell _ Center _ _ -> "<hr noshade size=1>" ]
         in
@@ -1817,11 +1851,11 @@ value print_tree conf base v p =
         let set = Dag.Pset.add ip set in
         if lev <= 1 then set
         else
-          match (aget conf base ip).parents with
+          match parents (aget conf base ip) with
           [ Some ifam ->
               let cpl = coi base ifam in
-              let set = loop set (lev - 1) cpl.mother in
-              loop set (lev - 1) cpl.father
+              let set = loop set (lev - 1) (mother cpl) in
+              loop set (lev - 1) (father cpl)
           | None -> set ]
     in
     let d = Dag.make_dag conf base (Dag.Pset.elements set) in
@@ -1863,10 +1897,10 @@ value print_horizontally conf base max_level p =
   let rec loop level s1 s2 s3 ip =
     if level >= max_level then ()
     else do {
-      match (aget conf base ip).parents with
+      match parents (aget conf base ip) with
       [ Some ifam ->
           loop (level + 1) (s1 ^ suff1) (s1 ^ suff2) (s1 ^ suff3)
-            (coi base ifam).father
+            (father (coi base ifam))
       | None -> () ];
       Wserver.wprint "<tt>%s</tt>" s2;
       let p = pget conf base ip in
@@ -1874,10 +1908,10 @@ value print_horizontally conf base max_level p =
       Wserver.wprint "%s%s<br>\n"
         (ref conf base p (no_spaces (person_text conf base p)))
         (no_spaces (Date.short_dates_text conf base p));
-      match (aget conf base ip).parents with
+      match parents (aget conf base ip) with
       [ Some ifam ->
           loop (level + 1) (s3 ^ suff3) (s3 ^ suff2) (s3 ^ suff1)
-            (coi base ifam).mother
+            (mother (coi base ifam))
       | None -> () ]
     }
   in
@@ -1900,10 +1934,10 @@ value print_male_female_line male conf base v p =
       let list = [ip :: list] in
       if lev <= 1 then list
       else
-        match (aget conf base ip).parents with
+        match parents (aget conf base ip) with
         [ Some ifam ->
             let cpl = coi base ifam in
-            loop list (lev - 1) (if male then cpl.father else cpl.mother)
+            loop list (lev - 1) (if male then (father cpl) else (mother cpl))
         | None -> list ]
   in
   let title _ =
@@ -2033,11 +2067,11 @@ value build_surnames_list conf base v p =
     else do {
       mark.(Adef.int_of_iper p.cle_index) :=
         mark.(Adef.int_of_iper p.cle_index) - 1;
-      match (aget conf base p.cle_index).parents with
+      match parents (aget conf base p.cle_index) with
       [ Some ifam ->
           let cpl = coi base ifam in
-          let fath = pget conf base cpl.father in
-          let moth = pget conf base cpl.mother in
+          let fath = pget conf base (father cpl) in
+          let moth = pget conf base (mother cpl) in
           do {
             if surn <> fath.surname && surn <> moth.surname then
               add_surname sosa p surn dp
