@@ -1,6 +1,6 @@
 (* camlp4r ./def.syn.cmo ./pa_html.cmo *)
-(* $Id: birthDeath.ml,v 4.8 2002/10/26 01:22:42 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: birthDeath.ml,v 4.14 2004/12/14 09:30:10 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -74,12 +74,13 @@ value select conf base get_date find_oldest =
   loop Q.empty 0 0
 ;
 
-value select_family conf base get_date =
+value select_family conf base get_date find_oldest =
   let module QF =
     Pqueue.Make
       (struct
          type t = (Def.family * Def.dmy * Def.calendar);
-         value leq (_, x, _) (_, y, _) = before_date y x;
+         value leq (_, x, _) (_, y, _) =
+	   if find_oldest then before_date x y else before_date y x;
        end)
   in
   let n =
@@ -98,17 +99,16 @@ value select_family conf base get_date =
       in
       loop [] q
     else
-      let p = base.data.families.get i in
+      let fam = base.data.families.get i in
       let (q, len) =
-        if Gutil.is_deleted_family p then (q, len)
-        else if p.relation == Married then
-          match get_date p with
+        if Gutil.is_deleted_family fam then (q, len)
+        else
+          match get_date fam with
           [ Some (Dgreg d cal) ->
-              let e = (p, d, cal) in
+              let e = (fam, d, cal) in
               if len < n then (QF.add e q, len + 1)
               else (snd (QF.take (QF.add e q)), len)
           | _ -> (q, len) ]
-        else (q, len)
       in
       loop q len (i + 1)
   in
@@ -305,13 +305,7 @@ value print_longest_lived conf base =
   }
 ;
 
-value print_marriage conf base =
-  let (list, len) =
-    select_family conf base (fun fam -> Adef.od_of_codate fam.marriage)
-  in
-  let title _ =
-    Wserver.wprint (fcapitale (ftransl conf "the latest %d marriages")) len
-  in
+value print_marr_or_eng conf base title list len =
   do {
     header conf title;
     print_link_to_welcome conf True;
@@ -340,12 +334,14 @@ value print_marriage conf base =
              Wserver.wprint "<li>\n";
              Wserver.wprint "<strong>\n";
              Wserver.wprint "\n%s"
-               (referenced_person_text conf base (pget conf base cpl.father));
+               (referenced_person_text conf base
+                  (pget conf base (father cpl)));
              Wserver.wprint "</strong>\n";
              Wserver.wprint "%s" (transl_nth conf "and" 0);
              Wserver.wprint "<strong>\n";
              Wserver.wprint "\n%s"
-               (referenced_person_text conf base (pget conf base cpl.mother));
+               (referenced_person_text conf base
+                  (pget conf base (mother cpl)));
              Wserver.wprint "</strong>,\n";
              if future then
                Wserver.wprint "<em>%s</em>.\n"
@@ -355,7 +351,8 @@ value print_marriage conf base =
                  (match fam.relation with
                   [ NotMarried -> transl_nth conf "relation/relations" 0
                   | Married | NoSexesCheck -> transl conf "married"
-                  | Engaged -> transl conf "engaged" ])
+                  | Engaged -> transl conf "engaged"
+		  | NoMention -> "" ])
                  (Date.string_of_ondate conf (Dgreg d cal));
              (month_txt, future)
            })
@@ -366,6 +363,44 @@ value print_marriage conf base =
   }
 ;
 
+value print_marriage conf base =
+  let (list, len) =
+    select_family conf base
+      (fun fam ->
+         if fam.relation == Married then Adef.od_of_codate fam.marriage
+         else None)
+      False
+  in
+  let title _ =
+    Wserver.wprint (fcapitale (ftransl conf "the latest %d marriages")) len
+  in
+  print_marr_or_eng conf base title list len
+;
+
+value print_oldest_engagements conf base =
+  let (list, len) =
+    select_family conf base
+      (fun fam ->
+         if fam.relation == Engaged then
+           let cpl = coi base fam.fam_index in
+           let husb = poi base (father cpl) in
+           let wife = poi base (mother cpl) in
+           match (husb.death, wife.death) with
+           [ (NotDead | DontKnowIfDead, NotDead | DontKnowIfDead) ->
+               Adef.od_of_codate fam.marriage
+           | _ -> None ]
+         else None)
+      True
+  in
+  let title _ =
+    Wserver.wprint
+     (fcapitale
+       (ftransl conf "the %d oldest couples perhaps still alive and engaged"))
+     len
+  in
+  print_marr_or_eng conf base title list len
+;
+
 value print_statistics conf base =
   let title _ = Wserver.wprint "%s" (capitale (transl conf "statistics")) in
   let n =
@@ -386,6 +421,11 @@ value print_statistics conf base =
         Wserver.wprint "<li><a href=\"%sm=LM;k=%d\">" (commd conf) n;
         Wserver.wprint (ftransl conf "the latest %d marriages") n;
         Wserver.wprint "</a>\n";
+        Wserver.wprint "<li><a href=\"%sm=OE;k=%d\">" (commd conf) n;
+        Wserver.wprint
+          (ftransl conf
+             "the %d oldest couples perhaps still alive and engaged") n;
+        Wserver.wprint "</a>\n";
         Wserver.wprint "<li><a href=\"%sm=OA;k=%d;lim=0\">" (commd conf) n;
         Wserver.wprint (ftransl conf "the %d oldest perhaps still alive") n;
         Wserver.wprint "</a>\n";
