@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: birthday.ml,v 4.9 2002/11/18 12:36:26 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: birthday.ml,v 4.11 2004/12/14 09:30:10 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Config;
@@ -276,8 +276,8 @@ value print_marriage conf base month =
         [ Some (Dgreg {day = d; month = m; year = y; prec = Sure} _)
           when d <> 0 && m <> 0 ->
             let cpl = base.data.couples.get i in
-            let father = pget conf base cpl.father in
-            let mother = pget conf base cpl.mother in
+            let father = pget conf base (father cpl) in
+            let mother = pget conf base (mother cpl) in
             if m == month &&
                authorized_age conf base father && not (is_hidden father) &&
                authorized_age conf base mother && not (is_hidden mother) then
@@ -302,11 +302,11 @@ value print_marriage conf base month =
                    html_li conf;
                    Wserver.wprint "%s"
                      (referenced_person_title_text conf base
-                        (pget conf base fam.father));
+                        (pget conf base (father fam)));
                    Wserver.wprint "\n%s\n" (transl_nth conf "and" 0);
                    Wserver.wprint "\n%s"
                      (referenced_person_title_text conf base
-                        (pget conf base fam.mother));
+                        (pget conf base (mother fam)));
                    Wserver.wprint ", <em>%s %d</em>\n"
                      (transl conf "in (year)") year;
                  })
@@ -329,11 +329,11 @@ value print_anniversaries_of_marriage conf base y list =
            html_li conf;
            Wserver.wprint "\n%s"
              (referenced_person_title_text conf base
-                (pget conf base fam.father));
+                (pget conf base (father fam)));
            Wserver.wprint "\n%s\n" (transl_nth conf "and" 0);
            Wserver.wprint "\n%s"
              (referenced_person_title_text conf base
-                (pget conf base fam.mother));
+                (pget conf base (mother fam)));
            Wserver.wprint ", <em>%s %d\n(" (transl conf "in (year)") year;
            Wserver.wprint (ftransl conf "%d years ago")
              (conf.today.year - year);
@@ -531,13 +531,13 @@ value print_menu_dead conf base =
 
 value match_mar_dates conf base cpl d1 d2 =
   if d1.day == d2.day && d1.month == d2.month then
-    authorized_age conf base (pget conf base cpl.father) &&
-    authorized_age conf base (pget conf base cpl.mother)
+    authorized_age conf base (pget conf base (father cpl)) &&
+    authorized_age conf base (pget conf base (mother cpl))
   else if
     d1.day == 29 && d1.month == 2 && d2.day == 1 && d2.month = 3 &&
     not (leap_year d2.year) then
-    authorized_age conf base (pget conf base cpl.father) &&
-    authorized_age conf base (pget conf base cpl.mother)
+    authorized_age conf base (pget conf base (father cpl)) &&
+    authorized_age conf base (pget conf base (mother cpl))
   else False
 ;
 
@@ -570,8 +570,8 @@ value print_menu_marriage conf base =
             in
             let cpl = base.data.couples.get i in
             if conf.use_restrict then
-              let father = pget conf base cpl.father in
-              let mother = pget conf base cpl.mother in
+              let father = pget conf base (father cpl) in
+              let mother = pget conf base (mother cpl) in
               if not (is_hidden father) && not (is_hidden mother) then
                 update_list cpl
               else ()
