@@ -1,20 +1,23 @@
-(* $Id: config.mli,v 4.8 2002/10/22 13:18:23 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: config.mli,v 4.12 2004/12/14 09:30:11 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 
 type config =
-  { wizard : bool;
+  { from : string;
+    wizard : bool;
     friend : bool;
     just_friend_wizard : bool;
     user : string;
     passwd : string;
+    username : string;
     cgi : bool;
     command : string;
     indep_command : string;
     highlight : string;
     lang : string;
     default_lang : string;
+    multi_parents : bool;
     can_send_image : bool;
     public_if_titles : bool;
     public_if_no_date : bool;
