@@ -1,5 +1,5 @@
-(* $Id: consangAll.ml,v 4.4 2002/03/11 19:02:55 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: consangAll.ml,v 4.11 2004/12/14 09:30:11 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -12,7 +12,7 @@ value rec clear_descend_consang base mark ifam =
     (fun ip ->
        if not mark.(Adef.int_of_iper ip) then do {
          let a = aoi base ip in
-         a.consang := no_consang;
+         set_consang a no_consang;
          mark.(Adef.int_of_iper ip) := True;
          let u = uoi base ip in
          Array.iter (clear_descend_consang base mark) u.family
@@ -36,26 +36,46 @@ value progr_bar_cnt =
   progr_bar_size * progr_bar_draw_rep * progr_bar_draw_len
 ;
 
+value start_progr_bar () =
+  do {
+    for i = 1 to progr_bar_size do { Printf.eprintf "%c" progr_bar_empty };
+    Printf.eprintf "\013"
+  }
+;
+
+value run_progr_bar cnt max_cnt =
+  do {
+    let already_disp = cnt * progr_bar_size / max_cnt in
+    let to_disp = (cnt + 1) * progr_bar_size / max_cnt in
+    for i = already_disp + 1 to to_disp do {
+      Printf.eprintf "%c" progr_bar_full
+    };
+    let already_disp = cnt * progr_bar_cnt / max_cnt in
+    let to_disp = (cnt + 1) * progr_bar_cnt / max_cnt in
+    if cnt = max_cnt - 1 then Printf.eprintf " \008"
+    else if to_disp > already_disp then
+      let k = to_disp mod progr_bar_draw_len in
+      let k = if k < 0 then progr_bar_draw_len + k else k in
+      Printf.eprintf "%c\008" progr_bar_draw.[k]
+    else ();
+    flush stderr;
+  }
+;
+
+value finish_progr_bar () =
+  do {
+    Printf.eprintf "\n";
+    flush stderr;
+  }
+;
+
 value trace quiet cnt max_cnt =
   do {
-    if quiet then do {
-      let x = max_cnt - cnt in
-      let already_disp = x * progr_bar_size / max_cnt in
-      let to_disp = (x + 1) * progr_bar_size / max_cnt in
-      for i = already_disp + 1 to to_disp do {
-        Printf.eprintf "%c" progr_bar_full
-      };
-      let already_disp = x * progr_bar_cnt / max_cnt in
-      let to_disp = (x + 1) * progr_bar_cnt / max_cnt in
-      if cnt = 1 then Printf.eprintf " \008"
-      else if to_disp > already_disp then
-        let k = to_disp mod progr_bar_draw_len in
-        let k = if k < 0 then progr_bar_draw_len + k else k in
-        Printf.eprintf "%c\008" progr_bar_draw.[k]
-      else ()
+    if quiet then run_progr_bar (max_cnt - cnt) max_cnt
+    else do {
+      Printf.eprintf "%6d\008\008\008\008\008\008" cnt;
+      flush stderr;
     }
-    else Printf.eprintf "%6d\008\008\008\008\008\008" cnt;
-    flush stderr;
   }
 ;
 
@@ -73,33 +93,26 @@ value compute base from_scratch quiet =
       match base.func.patched_ascends () with
       [ [] -> ()
       | list ->
-          let _ = base.data.families.array () in
-          do {
-            List.iter
-              (fun ip ->
-                 let u = uoi base ip in
-                 Array.iter (clear_descend_consang base mark) u.family)
-              list;
-            base.data.families.clear_array ()
-          } ]
+          List.iter
+            (fun ip ->
+               let u = uoi base ip in
+               Array.iter (clear_descend_consang base mark) u.family)
+            list ]
     else ();
     for i = 0 to base.data.ascends.len - 1 do {
       let a = base.data.ascends.get i in
-      if from_scratch then a.consang := no_consang
+      if from_scratch then set_consang a no_consang
       else
-        match a.parents with
-        [ Some ifam -> consang_tab.(Adef.int_of_ifam ifam) := a.consang
+        match parents a with
+        [ Some ifam -> consang_tab.(Adef.int_of_ifam ifam) := consang a
         | None -> () ];
-      if a.consang == no_consang then incr cnt else ()
+      if consang a == no_consang then incr cnt else ()
     };
     let max_cnt = cnt.val in
     let most = ref None in
     Printf.eprintf "To do: %d persons\n" max_cnt;
     if max_cnt = 0 then ()
-    else if quiet then do {
-      for i = 1 to progr_bar_size do { Printf.eprintf "%c" progr_bar_empty };
-      Printf.eprintf "\013"
-    }
+    else if quiet then start_progr_bar ()
     else Printf.eprintf "Computing consanguinity...";
     flush stderr;
     let running = ref True in
@@ -107,26 +120,28 @@ value compute base from_scratch quiet =
       running.val := False;
       for i = 0 to base.data.ascends.len - 1 do {
         let a = base.data.ascends.get i in
-        if a.consang == no_consang then
-          match a.parents with
+        if consang a == no_consang then
+          match parents a with
           [ Some ifam ->
               let pconsang = consang_tab.(Adef.int_of_ifam ifam) in
               if pconsang == no_consang then
                 let cpl = coi base ifam in
-                let fath = aoi base cpl.father in
-                let moth = aoi base cpl.mother in
-                if fath.consang != no_consang &&
-                   moth.consang != no_consang then
+                let fath = aoi base (father cpl) in
+                let moth = aoi base (mother cpl) in
+                if consang fath != no_consang &&
+                   consang moth != no_consang then
                    do {
-                  let consang = relationship base tab cpl.father cpl.mother in
+                  let consang =
+                    relationship base tab (father cpl) (mother cpl)
+                  in
                   trace quiet cnt.val max_cnt;
                   decr cnt;
-                  a.consang := Adef.fix_of_float consang;
-                  consang_tab.(Adef.int_of_ifam ifam) := a.consang;
+                  set_consang a (Adef.fix_of_float consang);
+                  consang_tab.(Adef.int_of_ifam ifam) := Gutil.consang a;
                   if not quiet then
                     let better =
                       match most.val with
-                      [ Some m -> a.consang > m.consang
+                      [ Some m -> Gutil.consang a > Gutil.consang m
                       | None -> True ]
                     in
                     if better then do {
@@ -140,19 +155,19 @@ value compute base from_scratch quiet =
                 }
                 else running.val := True
               else do {
-                trace quiet cnt.val max_cnt; decr cnt; a.consang := pconsang
+                trace quiet cnt.val max_cnt; decr cnt; set_consang a pconsang
               }
           | None ->
               do {
                 trace quiet cnt.val max_cnt;
                 decr cnt;
-                a.consang := Adef.fix_of_float 0.0
+                set_consang a (Adef.fix_of_float 0.0)
               } ]
         else ()
       }
     };
     if max_cnt = 0 then ()
-    else if quiet then Printf.eprintf "\n"
+    else if quiet then finish_progr_bar ()
     else Printf.eprintf " done   \n";
     flush stderr;
   }
