@@ -1,5 +1,5 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: dag.ml,v 4.10 2002/10/14 04:23:12 ddr Exp $ *)
+(* $Id: dag.ml,v 4.17 2004/12/13 16:05:16 ddr Exp $ *)
 
 open Dag2html;
 open Def;
@@ -78,11 +78,11 @@ value make_dag conf base list =
     Array.map
       (fun ip ->
          let pare =
-           match (aget conf base ip).parents with
+           match parents (aget conf base ip) with
            [ Some ifam ->
                let c = coi base ifam in
-               let l = try [M.find c.mother map] with [ Not_found -> [] ] in
-               try [M.find c.father map :: l] with [ Not_found -> l ]
+               let l = try [M.find (mother c) map] with [ Not_found -> [] ] in
+               try [M.find (father c) map :: l] with [ Not_found -> l ]
            | None -> [] ]
          in
          let chil =
@@ -108,13 +108,15 @@ value image_normal_txt conf base p fname width height =
   let s = Unix.stat fname in
   let b = acces conf base p in
   let k = default_image_name base p in
-  sprintf "<a href=\"%sm=IM;%s;k=/%s\">" (commd conf) b k ^
+  let r =
     sprintf "\
-<img src=\"%sm=IM;d=%d;%s;k=/%s\" width=%d height=%d border=0 alt=\"%s\">"
+<img src=\"%sm=IM;d=%d;%s;k=/%s\" width=%d%s border=0 alt=\"%s\">"
       (commd conf)
       (int_of_float (mod_float s.Unix.st_mtime (float_of_int max_int))) b k
-      width height image_txt ^
-    "</a>"
+      width (if height = 0 then "" else " height=" ^ string_of_int height) image_txt
+  in
+  if conf.cancel_links then r
+  else sprintf "<a href=\"%sm=IM;%s;k=/%s\">" (commd conf) b k ^ r ^ "</a>"
 ;
 
 value image_url_txt conf base url height =
@@ -140,6 +142,9 @@ value image_txt conf base p =
       [ Some (True, f, Some (wid, hei)) ->
           "<br>\n<center><table border=0><tr align=left><td>\n" ^
             image_normal_txt conf base p f wid hei ^ "</table></center>\n"
+      | Some (True, f, None) ->
+          "<br>\n<center><table border=0><tr align=left><td>\n" ^
+            image_normal_txt conf base p f 100 0 ^ "</table></center>\n"
       | Some (False, url, Some (wid, hei)) ->
           "<br>\n<center><table border=0><tr align=left><td>\n" ^
             image_url_txt_with_size conf base url wid hei ^
@@ -182,7 +187,7 @@ value print_table conf hts =
               match align with
               [ LeftA -> Wserver.wprint " width=\"50%%\" align=left"
               | RightA -> Wserver.wprint " width=\"50%%\" align=right"
-              | _ -> () ];
+              | _ -> Wserver.wprint " width=\"100%%\"" ];
               Wserver.wprint ">"
             } ];
         Wserver.wprint "</td>\n"
@@ -719,15 +724,15 @@ value make_tree_hts
                 (fun list id ->
                    match d.dag.(int_of_idag id).valu with
                    [ Left cip ->
-                       match (aget conf base cip).parents with
+                       match parents (aget conf base cip) with
                        [ Some ifam ->
                            let cpl = coi base ifam in
-                           if ip == cpl.father then
-                             if List.mem_assoc cpl.mother list then list
-                             else [(cpl.mother, Some ifam) :: list]
-                           else if ip == cpl.mother then
-                             if List.mem_assoc cpl.father list then list
-                             else [(cpl.father, Some ifam) :: list]
+                           if ip == (father cpl) then
+                             if List.mem_assoc (mother cpl) list then list
+                             else [((mother cpl), Some ifam) :: list]
+                           else if ip == (mother cpl) then
+                             if List.mem_assoc (father cpl) list then list
+                             else [((father cpl), Some ifam) :: list]
                            else list
                        | None -> list ]
                    | Right _ -> list ])
@@ -764,7 +769,10 @@ value make_tree_hts
   let td =
     match Util.p_getenv conf.env "td" with
     [ Some x -> " " ^ x
-    | _ -> "" ]
+    | _ ->
+        match Util.p_getenv conf.env "color" with
+	[ None | Some "" -> ""
+        | Some x -> " bgcolor=" ^ x ] ]
   in
   let indi_txt n =
     let (bd, td) =
@@ -774,7 +782,7 @@ value make_tree_hts
     in
     if bd > 0 || td <> "" then
       sprintf
-        "<table border=%d><tr align-left><td align=center%s>%s</table>" bd td
+        "<table border=%d><tr align=left><td align=center%s>%s</table>" bd td
         (indi_txt n)
     else indi_txt n
   in
@@ -806,8 +814,8 @@ value print_slices_menu conf base hts_opt =
         (fun (k, v) ->
            if k = "slices" then ()
            else
-             Wserver.wprint "<input type=hidden name=%s value=%s>\n"
-               (code_varenv k) (code_varenv v))
+             Wserver.wprint "<input type=hidden name=\"%s\" value=\"%s\">\n"
+               (decode_varenv k) (decode_varenv v))
         conf.env;
       tag "table" begin
         tag "tr" "align=left" begin
