@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: date.ml,v 4.16 2002/12/26 14:26:19 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: date.ml,v 4.21 2004/12/14 09:30:11 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Util;
@@ -87,7 +87,7 @@ value hebrew_month conf m =
 ;
 
 value code_french_year conf y =
-  transl_nth conf "year/month/day" 0 ^ " " ^
+  transl_nth conf "year/month/day" 3 ^ " " ^
     (if y >= 1 && y < 4000 then roman_of_arabian y else string_of_int y)
 ;
 
@@ -475,35 +475,53 @@ value french_month_name conf n = capitale (nominative (french_month conf n));
 value hebrew_month_name conf n = capitale (nominative (hebrew_month conf n));
 
 value print_year date var =
-  tag "td" begin
-    Wserver.wprint "<input type=submit name=y%s1 value=\"&lt;\">" var;
-    Wserver.wprint "<input name=y%s size=5 maxlength=5 value=%d>" var
-      date.year;
-    Wserver.wprint "<input type=submit name=y%s2 value=\"&gt;\">\n" var;
-  end
+  do {
+    tag "td" begin
+      Wserver.wprint "<input type=submit name=y%s1 value=\" &lt; \">" var;
+    end;
+    tag "td" begin
+      Wserver.wprint "<input name=y%s size=5 maxlength=5 value=%d>" var
+        date.year;
+    end;
+    tag "td" begin
+      Wserver.wprint "<input type=submit name=y%s2 value=\" &gt; \">\n" var;
+    end
+  }
 ;
 
 value print_month conf date month_name n_months var =
-  tag "td" "align=center" begin
-    Wserver.wprint "<input type=submit name=m%s1 value=\"&lt;\">" var;
-    stag "select" "name=m%s" var begin
-      for i = 1 to n_months do {
-        Wserver.wprint "<option value=%d%s> %s\n" i
-          (if date.month = i then " selected" else "")
-          (month_name conf (i - 1))
-      };
+  do {
+    tag "td" begin
+      Wserver.wprint "<input type=submit name=m%s1 value=\" &lt; \">" var;
     end;
-    Wserver.wprint "<input type=submit name=m%s2 value=\"&gt;\">" var;
-  end
+    tag "td" "align=center" begin
+      tag "select" "name=m%s" var begin
+        for i = 1 to n_months do {
+          Wserver.wprint "<option value=%d%s> %s\n" i
+            (if date.month = i then " selected" else "")
+            (month_name conf (i - 1))
+        };
+      end;
+    end;
+    tag "td" begin
+      Wserver.wprint "<input type=submit name=m%s2 value=\" &gt; \">" var;
+    end
+  }
 ;
 
 value print_day date var =
-  tag "td" begin
-    Wserver.wprint "<input type=submit name=d%s1 value=\"&lt;\">" var;
-    Wserver.wprint "<input name=d%s size=2 maxlength=2 value=%d>" var
-      date.day;
-    Wserver.wprint "<input type=submit name=d%s2 value=\"&gt;\">\n" var;
-  end
+  do {
+    tag "td" begin
+      Wserver.wprint "<input type=submit name=d%s1 value=\" &lt; \">" var;
+    end;
+    tag "td" begin
+      Wserver.wprint "<input name=d%s size=2 maxlength=2 value=%d>" var
+        date.day;
+    end;
+    tag "td" begin
+      Wserver.wprint "<input type=submit name=d%s2 value=\" &gt; \">\n" var;
+    end
+  }
 ;
 
 value print_some_calendar conf order date n month_name n_months var =
@@ -526,7 +544,7 @@ value print_some_calendar conf order date n month_name n_months var =
         print_day date var;
       };
       tag "td" begin
-        Wserver.wprint "<input type=submit name=t%s value=\"=\">\n" var;
+        Wserver.wprint "<input type=submit name=t%s value=\" = \">\n" var;
       end;
     end;
   }
@@ -538,13 +556,13 @@ value print_calendar_head conf order =
     Wserver.wprint "\n";
     if order = "ddmmyy" then
       for i = 2 downto 0 do {
-        tag "th align=center" begin
+        tag "th" "align=center colspan=3" begin
           Wserver.wprint "%s" (capitale (transl_nth conf "year/month/day" i));
         end
       }
     else
       for i = 0 to 2 do {
-        tag "th align=center" begin
+        tag "th" "align=center" begin
           Wserver.wprint "%s" (capitale (transl_nth conf "year/month/day" i));
         end
       };
@@ -623,39 +641,67 @@ value print_calendar conf base =
   do {
     header conf title;
     print_link_to_welcome conf True;
-    Wserver.wprint "- %s -"
-      (capitale (nominative (transl_nth conf "(week day)" wday)));
-    if date = conf.today then
-      let (hh, mm, ss) = conf.time in
-      Wserver.wprint " <tt>%02d:%02d:%02d</tt>" hh mm ss
-    else ();
-    Wserver.wprint "\n";
-    html_p conf;
-    tag "form" "method=GET action=\"%s\"" conf.command begin
-      List.iter
-        (fun (k, v) ->
-           Wserver.wprint "<input type=hidden name=%s value=%s>\n" k
-             (quote_escaped (decode_varenv v)))
-        conf.henv;
-      Wserver.wprint "<input type=hidden name=m value=CAL>\n\n";
-      let order = transl conf " !dates order" in
-      tag "table" "border=1" begin
-        print_calendar_head conf order;
-        print_some_calendar conf order date 0 gregorian_month_name 12 "g";
-        print_some_calendar conf order (Calendar.julian_of_gregorian date) 1
-          julian_month_name 12 "j";
-        print_some_calendar conf order (Calendar.french_of_gregorian date) 2
-          french_month_name 13 "f";
-        print_some_calendar conf order (Calendar.hebrew_of_gregorian date) 3
-          hebrew_month_name 13 "h";
+    tag "table" "align=center" begin
+      stag "tbody" begin
+        stag "tr" begin
+          tag "td" "align=left" begin
+            Wserver.wprint "- %s -"
+              (capitale (nominative (transl_nth conf "(week day)" wday)));
+            if date = conf.today then
+              let (hh, mm, ss) = conf.time in
+              Wserver.wprint " <tt>%02d:%02d:%02d</tt>" hh mm ss
+            else ();
+            Wserver.wprint "\n";
+          end;
+        end;
+        stag "tr" begin
+          stag "td" "align=left" begin
+            Wserver.wprint "&nbsp;";
+          end;
+        end;
+        stag "tr" begin
+          tag "td" "align=center" begin
+            tag "form" "method=GET action=\"%s\"" conf.command begin
+              List.iter
+                (fun (k, v) ->
+                   Wserver.wprint "<input type=hidden name=%s value=%s>\n" k
+                     (quote_escaped (decode_varenv v)))
+                conf.henv;
+              Wserver.wprint "<input type=hidden name=m value=CAL>\n\n";
+              let order = transl conf " !dates order" in
+              tag "table" "border=1" begin
+                print_calendar_head conf order;
+                print_some_calendar conf order date 0 gregorian_month_name
+                  12 "g";
+                print_some_calendar conf order
+                  (Calendar.julian_of_gregorian date) 1 julian_month_name
+                  12 "j";
+                print_some_calendar conf order
+                  (Calendar.french_of_gregorian date) 2 french_month_name
+                  13 "f";
+                print_some_calendar conf order
+                  (Calendar.hebrew_of_gregorian date) 3 hebrew_month_name
+                  13 "h";
+              end;
+            end;
+          end;
+        end;
+        stag "tr" begin
+          stag "td" "align=left" begin
+            Wserver.wprint "&nbsp;";
+          end;
+        end;
+        stag "tr" begin
+          tag "td" "align=center" begin
+            Wserver.wprint "%s: " (capitale (transl conf "julian day"));
+            let jd = Calendar.sdn_of_gregorian date in
+            if jd < 0 then Wserver.wprint "%d" jd
+            else
+              Num.print (fun x -> Wserver.wprint "%s" x)
+                (transl conf "(thousand separator)") (Num.of_int jd);
+          end;
+        end;
       end;
-      Wserver.wprint "<br><p>%s: " (capitale (transl conf "julian day"));
-      let jd = Calendar.sdn_of_gregorian date in
-      if jd < 0 then Wserver.wprint "%d" jd
-      else
-        Num.print (fun x -> Wserver.wprint "%s" x)
-          (transl conf "(thousand separator)") (Num.of_int jd);
-      Wserver.wprint "\n";
     end;
     trailer conf;
   }
