@@ -1,5 +1,5 @@
-(* $Id: def.mli,v 4.4 2002/09/23 15:49:15 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: def.mli,v 4.11 2004/12/14 09:30:11 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 type choice 'a 'b = [ Left of 'a | Right of 'b ];
 
@@ -27,7 +27,9 @@ and precision = Adef.precision ==
   [ Sure | About | Maybe | Before | After | OrYear of int | YearInt of int ]
 ;
 
-type relation_kind = [ Married | NotMarried | Engaged | NoSexesCheck ];
+type relation_kind =
+  [ Married | NotMarried | Engaged | NoSexesCheck | NoMention ]
+;
 
 type divorce = [ NotDivorced | Divorced of codate | Separated ];
 
@@ -139,10 +141,7 @@ type gen_family 'person 'string =
     fam_index : mutable ifam }
 ;
 
-type gen_couple 'person =
-  { father : mutable 'person;
-    mother : mutable 'person }
-;
+type gen_couple 'person = Adef.gen_couple 'person;
 
 type gen_descend 'person =
   { children : mutable array 'person }
