@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: descend.ml,v 4.18 2002/11/18 12:36:27 ddr Exp $ *)
-(* Copyright (c) 2002 INRIA *)
+(* $Id: descend.ml,v 4.24 2004/12/14 09:30:11 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
@@ -126,6 +126,19 @@ value print_choice conf base p effective_level =
           Wserver.wprint
             "- %s <input type=checkbox name=image value=on><br>\n"
             (capitale (transl_nth conf "image/images" 1));
+          Wserver.wprint
+            "- %s <input name=bd size=1 maxlength=2 value=0><br>\n"
+            (capitale (transl conf "border"));
+          Wserver.wprint "\
+<table><tr><td>-&nbsp;%s</td>
+<td><input type=radio name=color value=\"\" checked></td>\n"
+              (capitale (transl conf "color"));
+          List.iter
+            (fun c ->
+               Wserver.wprint "\
+<td bgcolor=%s><input type=radio name=color value=%s></td>\n" c c)
+            ["FFC0C0"; "FFFFC0"; "C0FFC0"; "C0FFFF"; "C0C0FF"; "FFC0FF"];
+          Wserver.wprint "</tr></table>\n";
         end;
         tag "td" begin
           Wserver.wprint "<input type=radio name=t value=S> %s<br>\n"
@@ -171,10 +184,10 @@ value display_descendant_menu conf base p =
 
 value named_like_father conf base ip =
   let a = aget conf base ip in
-  match a.parents with
+  match parents a with
   [ Some ifam ->
       (pget conf base ip).surname =
-         (pget conf base (coi base ifam).father).surname
+         (pget conf base (father (coi base ifam))).surname
   | _ -> False ]
 ;
 
@@ -263,8 +276,11 @@ value display_descendants_upto conf base max_level p line =
   let levt = make_level_table conf base max_level p in
   let count = ref 0 in
   let always_surname =
-    try List.assoc "always_surname" conf.base_env = "yes" with
-    [ Not_found -> False ]
+    match p_getenv conf.env "alwsurn" with
+    [ Some x -> x = "yes"
+    | None ->
+        try List.assoc "always_surname" conf.base_env = "yes" with
+        [ Not_found -> False ] ]
   in
   let rec loop level p u =
     if level <= max_level then
@@ -491,7 +507,7 @@ value labelled conf base marks max_lev lev ip =
   let a = aget conf base ip in
   let u = uget conf base ip in
   Array.length u.family <> 0 &&
-  (match a.parents with
+  (match parents a with
    [ Some ifam ->
        let cpl = coi base ifam in
        List.exists
@@ -504,7 +520,7 @@ value labelled conf base marks max_lev lev ip =
                  Array.length u.family <> 0 &&
                  not (close_to_end conf base marks max_lev lev e))
               (Array.to_list el))
-         (Array.to_list (uget conf base cpl.father).family)
+         (Array.to_list (uget conf base (father cpl)).family)
    | _ -> False ])
 ;
 
@@ -1000,6 +1016,15 @@ value display_descendant_with_table conf base max_lev a =
 
 value make_tree_hts conf base gv p =
   let gv = min (limit_by_tree conf) gv in
+  let bd = match p_getint conf.env "bd" with [ Some x -> x | None -> 0 ] in
+  let td_prop =
+    match Util.p_getenv conf.env "td" with
+    [ Some x -> " " ^ x
+    | _ ->
+        match Util.p_getenv conf.env "color" with
+	[ None | Some "" -> ""
+        | Some x -> " bgcolor=" ^ x ] ]
+  in
   let rec nb_column n v u =
     if v == 0 then n + 1
     else if Array.length u.family = 0 then n + 1
@@ -1128,7 +1153,15 @@ value make_tree_hts conf base gv p =
           let txt =
             if auth then txt ^ Date.short_dates_text conf base p else txt
           in
-          (2 * ncol - 1, CenterA, TDstring (txt ^ Dag.image_txt conf base p))
+          let txt =
+            if bd > 0 || td_prop <> "" then
+              Printf.sprintf
+                "<table border=%d><tr><td align=center%s>%s</td></tr></table>"
+                bd td_prop txt
+            else txt
+          in
+          let txt = txt ^ Dag.image_txt conf base p in
+          (2 * ncol - 1, CenterA, TDstring txt)
       | None -> (1, LeftA, TDstring "&nbsp;") ]
     in
     [td :: tdl]
@@ -1164,6 +1197,13 @@ value make_tree_hts conf base gv p =
                    else "") ^
                   "&nbsp;" ^ txt ^ Dag.image_txt conf base sp
               in
+              let s =
+                if bd > 0 || td_prop <> "" then
+                  Printf.sprintf
+                    "<table border=%d><tr>\
+                     <td align=center%s>%s</td></tr></table>" bd td_prop s
+                else s
+              in
               (2 * ncol - 1, CenterA, TDstring s)
             in
             loop [td :: tdl] (i + 1)
