@@ -1,6 +1,6 @@
-(* camlp4r ./def.syn.cmo ./pa_html.cmo *)
-(* $Id: family.ml,v 4.30 2002/12/09 22:42:42 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* camlp4r ./def.syn.cmo ./pa_lock.cmo ./pa_html.cmo *)
+(* $Id: family.ml,v 4.45 2004/12/14 09:30:12 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -70,34 +70,7 @@ value person_selected conf base p =
 ;
 
 value compact_list conf base xl =
-  let pl =
-    Sort.list
-      (fun p1 p2 ->
-         match
-           (Adef.od_of_codate p1.birth, p1.death, Adef.od_of_codate p2.birth,
-            p2.death)
-         with
-         [ (Some d1, _, Some d2, _) -> d1 strictly_before d2
-         | (Some d1, _, _, Death _ d2) ->
-             d1 strictly_before Adef.date_of_cdate d2
-         | (_, Death _ d1, Some d2, _) ->
-             Adef.date_of_cdate d1 strictly_before d2
-         | (_, Death _ d1, _, Death _ d2) ->
-             Adef.date_of_cdate d1 strictly_before Adef.date_of_cdate d2
-         | (Some _, _, _, _) -> False
-         | (_, Death _ _, _, _) -> False
-         | (_, _, Some _, _) -> True
-         | (_, _, _, Death _ _) -> True
-         | _ ->
-             let c = alphabetic (p_surname base p1) (p_surname base p2) in
-             if c == 0 then
-               let c =
-                 alphabetic (p_first_name base p1) (p_first_name base p2)
-               in
-               if c == 0 then p1.occ > p2.occ else c > 0
-             else c > 0 ])
-      xl
-  in
+  let pl = sort_person_list base xl in
   let pl =
     List.fold_right
       (fun p pl ->
@@ -168,7 +141,7 @@ value name_with_roman_number str =
 ;
 
 value find_all conf base an =
-  let sosa_ref = Util.find_person_in_env conf base "z" in
+  let sosa_ref = Util.find_sosa_ref conf base in
   let sosa_nb = try Some (Num.of_string an) with [ Failure _ -> None ] in
   match (sosa_ref, sosa_nb) with
   [ (Some p, Some n) ->
@@ -306,14 +279,10 @@ value specify conf base n pl =
            | [h :: hl] ->
                let s =
                  List.fold_left
-                   (fun s h ->
-                      s ^ ", " ^ transl_nth conf "and" 0 ^ "\n" ^
-                        person_title_text conf base h)
+                   (fun s h -> s ^ ",\n" ^ person_title_text conf base h)
                    (person_title_text conf base h) hl
                in
-               Wserver.wprint ", <em>%s</em>\n"
-                 (transl_a_of_gr_eq_gen_lev conf
-                    (transl_nth conf "spouse" (index_of_sex p.sex)) s) ];
+               Wserver.wprint ", <em>&amp; %s</em>\n" s ]
          })
       ptll;
     Wserver.wprint "</ul>\n";
@@ -341,6 +310,12 @@ value set_senv conf vm vi =
     match p_getenv conf.env "cgl" with
     [ Some "on" -> conf.senv := conf.senv @ [("cgl", "on")]
     | _ -> () ];
+    match p_getenv conf.env "bd" with
+    [ None | Some ("0" | "") -> ()
+    | Some x -> conf.senv := conf.senv @ [("bd", x)] ];
+    match p_getenv conf.env "color" with
+    [ Some x -> conf.senv := conf.senv @ [("color", x)]
+    | _ -> () ];
   }
 ;
 
@@ -402,12 +377,12 @@ value family_m conf base =
   | Some "DAG" -> Dag.print conf base
   | Some "DEL_FAM" when conf.wizard -> UpdateFam.print_del conf base
   | Some "DEL_FAM_OK" when conf.wizard -> UpdateFamOk.print_del conf base
-  | Some "DEL_IND" when conf.wizard -> UpdateInd.print_del conf base
-  | Some "DEL_IND_OK" when conf.wizard -> UpdateIndOk.print_del conf base
   | Some "DEL_IMAGE" when conf.wizard && conf.can_send_image ->
       SendImage.print_del conf base
   | Some "DEL_IMAGE_OK" when conf.wizard && conf.can_send_image ->
       SendImage.print_del_ok conf base
+  | Some "DEL_IND" when conf.wizard -> UpdateInd.print_del conf base
+  | Some "DEL_IND_OK" when conf.wizard -> UpdateIndOk.print_del conf base
   | Some "DOC" -> Doc.print conf
   | Some "FORUM" -> Forum.print conf base
   | Some "FORUM_ADD" -> Forum.print_add conf base
@@ -418,6 +393,7 @@ value family_m conf base =
       [ Some f -> Srcfile.print conf base f
       | None -> Util.incorrect_request conf ]
   | Some "HIST" -> History.print conf base
+  | Some "IMH" -> Image.print_html conf base
   | Some "INV_FAM" when conf.wizard -> UpdateFam.print_inv conf base
   | Some "INV_FAM_OK" when conf.wizard -> UpdateFamOk.print_inv conf base
   | Some "KILL_ANC" when conf.wizard ->
@@ -430,6 +406,12 @@ value family_m conf base =
   | Some "LM" when conf.wizard || conf.friend ->
       BirthDeath.print_marriage conf base
   | Some "LEX" -> Srcfile.print_lexicon conf base
+  | Some "MOD_FAM" when conf.wizard -> UpdateFam.print_mod conf base
+  | Some "MOD_FAM_OK" when conf.wizard -> UpdateFamOk.print_mod conf base
+  | Some "MOD_IND" when conf.wizard -> UpdateInd.print_mod conf base
+  | Some "MOD_IND_OK" when conf.wizard -> UpdateIndOk.print_mod conf base
+  | Some "MOD_NOTES" when conf.wizard -> Notes.print_mod conf base
+  | Some "MOD_NOTES_OK" when conf.wizard -> Notes.print_mod_ok conf base
   | Some "MRG" when conf.wizard ->
       match find_person_in_env conf base "" with
       [ Some p -> Merge.print conf base p
@@ -442,12 +424,6 @@ value family_m conf base =
   | Some "MRG_IND_OK" when conf.wizard -> MergeIndOk.print_merge conf base
   | Some "MRG_MOD_IND_OK" when conf.wizard ->
       MergeIndOk.print_mod_merge conf base
-  | Some "MOD_FAM" when conf.wizard -> UpdateFam.print_mod conf base
-  | Some "MOD_FAM_OK" when conf.wizard -> UpdateFamOk.print_mod conf base
-  | Some "MOD_IND" when conf.wizard -> UpdateInd.print_mod conf base
-  | Some "MOD_IND_OK" when conf.wizard -> UpdateIndOk.print_mod conf base
-  | Some "MOD_NOTES" when conf.wizard -> Notes.print_mod conf base
-  | Some "MOD_NOTES_OK" when conf.wizard -> Notes.print_mod_ok conf base
   | Some "RLM" -> Relation.print_multi conf base
   | Some "N" ->
       match p_getenv conf.env "v" with
@@ -491,6 +467,8 @@ value family_m conf base =
   | Some "NOTES" -> Notes.print conf base
   | Some "OA" when conf.wizard || conf.friend ->
       BirthDeath.print_oldest_alive conf base
+  | Some "OE" when conf.wizard || conf.friend ->
+      BirthDeath.print_oldest_engagements conf base
   | Some "P" ->
       match p_getenv conf.env "v" with
       [ Some v -> Some.first_name_print conf base v
@@ -548,14 +526,15 @@ value print_no_index conf base =
 
 value special_vars =
   ["dsrc"; "em"; "ei"; "ep"; "en"; "eoc"; "escache"; "et"; "long"; "spouse";
-   "cgl"; "iz"; "nz"; "pz"; "ocz"; "templ"; "log_uid"; "log_pwd"; "log_cnl"]
+   "cgl"; "iz"; "nz"; "pz"; "ocz"; "templ"; "log_uid"; "log_pwd"; "log_cnl";
+   "alwsurn"]
 ;
 
 value only_special_env = List.for_all (fun (x, _) -> List.mem x special_vars);
 
 value extract_henv conf base =
   do {
-    match find_person_in_env conf base "z" with
+    match find_sosa_ref conf base with
     [ Some p ->
         let x =
           let first_name = p_first_name base p in
@@ -580,6 +559,9 @@ value extract_henv conf base =
         let v = escache_value conf in
         conf.henv := conf.henv @ [("escache", v)]
     | None -> () ];
+    match p_getenv conf.env "alwsurn" with
+    [ Some x -> conf.henv := conf.henv @ [("alwsurn", x)]
+    | _ -> () ];
   }
 ;
 
@@ -608,27 +590,119 @@ value log_count conf (log_file, log_oc, flush_log) r =
     | None -> () ]
 ;
 
-value family conf base log =
+value print_moved conf base s =
+  match Util.open_etc_file "moved" with
+  [ Some ic ->
+      let env = [('b', fun () -> conf.bname); ('t', fun () -> s)] in
+      do {
+        Util.html conf;
+        Util.nl ();
+        Util.copy_from_etc env conf.lang conf.indep_command ic;
+      }
+  | None ->
+      let title _ = Wserver.wprint "%s -&gt; %s" conf.bname s in
+      do {
+        Util.header_no_page_title conf title;
+        Wserver.wprint "The database %s has moved to:\n<dl><dt><dd>\n"
+          conf.bname;
+        stag "a" "href=\"%s\"" s begin Wserver.wprint "%s" s; end;
+        Wserver.wprint "\n</dd></dt></dl>\n";
+        Util.trailer conf;
+      } ]
+;
+
+value treat_request conf base log =
   do {
-    match (p_getenv conf.env "opt", p_getenv conf.env "m") with
-    [ (Some "no_index", _) -> print_no_index conf base
-    | (_, Some "IM") -> Image.print conf base
-    | _ ->
-        do {
-          set_owner conf;
-          extract_henv conf base;
-          make_senv conf base;
-          if only_special_env conf.env then do {
-            let r = Srcfile.incr_welcome_counter conf in
-            log_count conf log r;
-            Srcfile.print_start conf base;
-          }
-          else do {
-            let r = Srcfile.incr_request_counter conf in
-            log_count conf log r;
-            family_m conf base;
-          };
-        } ];
+    match p_getenv conf.base_env "moved" with
+    [ Some s ->
+        print_moved conf base s
+    | None ->
+        match (p_getenv conf.env "opt", p_getenv conf.env "m") with
+        [ (Some "no_index", _) -> print_no_index conf base
+        | (_, Some "IM") -> Image.print conf base
+        | _ ->
+    
+            do {
+              set_owner conf;
+              extract_henv conf base;
+              make_senv conf base;
+              if only_special_env conf.env then do {
+                let r = Srcfile.incr_welcome_counter conf in
+                log_count conf log r;
+                Srcfile.print_start conf base;
+              }
+              else do {
+                let r = Srcfile.incr_request_counter conf in
+                log_count conf log r;
+                family_m conf base;
+              };
+            } ] ];
     Wserver.wflush ();
   }
 ;
+
+value treat_request_on_possibly_locked_base conf bfile log =
+  match try Left (Iobase.input bfile) with e -> Right e with
+  [ Left base ->
+      do {
+        try treat_request conf base log with exc ->
+          do { base.func.cleanup (); raise exc };
+        base.func.cleanup ();
+      }
+  | Right e ->
+      let transl conf w =
+        try Hashtbl.find conf.lexicon w with [ Not_found -> "[" ^ w ^ "]" ]
+      in
+      let title _ =
+        Wserver.wprint "%s" (Util.capitale (transl conf "error"))
+      in
+      do {
+        Util.rheader conf title;
+        Wserver.wprint "<ul>";
+        Util.html_li conf;
+        Wserver.wprint "%s"
+          (Util.capitale (transl conf "cannot access base"));
+        Wserver.wprint " \"%s\".</ul>\n" conf.bname;
+        match e with
+        [ Sys_error _ -> ()
+        | _ ->
+            Wserver.wprint
+              "<em><font size=-1>Internal message: %s</font></em>\n"
+              (Printexc.to_string e) ];
+        Util.trailer conf;
+      } ]
+;
+
+value this_request_updates_database conf =
+  match p_getenv conf.env "m" with
+  [ Some ("FORUM_ADD_OK" | "FORUM_DEL") -> True
+  | Some x when conf.wizard ->
+      match x with
+      [ "ADD_FAM_OK" | "ADD_IND_OK" | "CHG_CHN_OK" | "DEL_FAM_OK" |
+        "DEL_IMAGE_OK" | "DEL_IND_OK" | "INV_FAM_OK" | "KILL_ANC" |
+	"MOD_FAM_OK" | "MOD_IND_OK" | "MRG_IND" | "MRG_MOD_FAM_OK" |
+	"MRG_MOD_IND_OK" | "SND_IMAGE_OK" -> True
+      | _ -> False ]
+  | _ -> False ]
+;
+
+value error_locked conf =
+  let title _ = Wserver.wprint "%s" (capitale (transl conf "error")) in
+  do {
+    rheader conf title;
+    Wserver.wprint
+      (fcapitale
+         (ftransl conf "the file is temporarily locked: please try again"));
+    Wserver.wprint ".\n";
+    trailer conf
+  }
+;
+
+value treat_request_on_base conf log =
+  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
+  if this_request_updates_database conf then
+    lock Iobase.lock_file bfile with
+    [ Accept -> treat_request_on_possibly_locked_base conf bfile log
+    | Refuse -> error_locked conf ]
+  else treat_request_on_possibly_locked_base conf bfile log
+;
