@@ -1,5 +1,5 @@
-(* $Id: gutil.ml,v 4.13 2002/11/19 12:43:52 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: gutil.ml,v 4.26 2004/12/14 09:30:12 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 
@@ -83,7 +83,7 @@ value decline case s =
       if i == ibeg then "" else decline_word case s ibeg i
     else
       match s.[i] with
-      [ ' ' | '<' as sep ->
+      [ ' ' | '<' | '/' as sep ->
           decline_word case s ibeg i ^ String.make 1 sep ^
             loop (i + 1) (i + 1)
       | '>' -> String.sub s ibeg (i + 1 - ibeg) ^ loop (i + 1) (i + 1)
@@ -108,7 +108,7 @@ value nb_days_in_month =
   fun m a ->
     if m == 2 && leap_year a then 29
     else if m >= 1 && m <= 12 then tb.(m - 1)
-    else 31
+    else 0
 ;
 
 value common_prec p1 p2 =
@@ -211,10 +211,30 @@ value designation base p =
   let nom = p_surname base p in prenom ^ "." ^ string_of_int p.occ ^ " " ^ nom
 ;
 
+value father = Adef.father;
+value mother = Adef.mother;
+value couple multi fath moth =
+  if not multi then Adef.couple fath moth else Adef.multi_couple fath moth
+;
+value parent multi parent =
+  if not multi then Adef.parent parent else Adef.multi_parent parent
+;
+value parent_array = Adef.parent_array;
+value set_father = Adef.set_father;
+value set_mother = Adef.set_mother;
+
+value parents asc = asc.parents;
+value consang asc = asc.consang;
+value no_ascend () = {parents = None; consang = Adef.fix (-1)};
+value set_parents asc v = asc.parents := v;
+value set_consang asc v = asc.consang := v;
+
 value spouse ip cpl =
-  if ip == cpl.father then cpl.mother
-  else if ip == cpl.mother then cpl.father
-  else invalid_arg "spouse"
+  if ip == father cpl then mother cpl
+(*
+  else if ip == mother cpl then father cpl
+*)
+  else father cpl
 ;
 
 value saints = ["saint"; "sainte"];
@@ -278,7 +298,7 @@ value person_misc_names base p =
         List.fold_left
           (fun list ifam ->
              let cpl = coi base ifam in
-             let husband = poi base cpl.father in
+             let husband = poi base (father cpl) in
              let husband_surname = p_surname base husband in
              let husband_surnames_aliases =
                List.map (sou base) husband.surnames_aliases
@@ -323,10 +343,10 @@ value person_misc_names base p =
         list p.titles
     in
     let list =
-      match (aoi base p.cle_index).parents with
+      match parents (aoi base p.cle_index) with
       [ Some ifam ->
           let cpl = coi base ifam in
-          let fath = poi base cpl.father in
+          let fath = poi base (father cpl) in
           let first_names =
             [first_name :: List.map (sou base) p.first_names_aliases]
           in
@@ -487,10 +507,10 @@ value check_noloop base error =
     match tab.(i) with
     [ NotVisited ->
         do {
-          match (base.data.ascends.get i).parents with
+          match parents (base.data.ascends.get i) with
           [ Some fam ->
-              let fath = (coi base fam).father in
-              let moth = (coi base fam).mother in
+              let fath = father (coi base fam) in
+              let moth = mother (coi base fam) in
               do {
                 tab.(i) := BeingVisited;
                 noloop (Adef.int_of_iper fath);
@@ -518,14 +538,12 @@ value check_noloop_for_person_list base error ipl =
     match tab.(i) with
     [ NotVisited ->
         do {
-          match (aoi base ip).parents with
+          match parents (aoi base ip) with
           [ Some ifam ->
               let cpl = coi base ifam in
               do {
                 tab.(i) := BeingVisited;
-                noloop cpl.father;
-                noloop cpl.mother;
-                ()
+                Array.iter noloop (parent_array cpl);
               }
           | None -> () ];
           tab.(i) := Visited;
@@ -788,8 +806,8 @@ value check_normal_marriage_date_for_someone base error warning fam ip =
 value check_normal_marriage_date base error warning fam =
   let cpl = coi base fam.fam_index in
   do {
-    check_normal_marriage_date_for_someone base error warning fam cpl.father;
-    check_normal_marriage_date_for_someone base error warning fam cpl.mother;
+    check_normal_marriage_date_for_someone base error warning fam (father cpl);
+    check_normal_marriage_date_for_someone base error warning fam (mother cpl);
   }
 ;
 
@@ -837,8 +855,8 @@ value sort_children base warning ifam des =
 
 value check_family base error warning fam cpl des =
   let ifam = fam.fam_index in
-  let fath = poi base cpl.father in
-  let moth = poi base cpl.mother in
+  let fath = poi base (father cpl) in
+  let moth = poi base (mother cpl) in
   do {
     match fath.sex with
     [ Male -> birth_before_death base warning fath
@@ -860,10 +878,10 @@ value check_family base error warning fam cpl des =
              birth_before_death base warning child;
              born_after_his_elder_sibling base error warning child np ifam
                des;
-             child_born_after_his_parent base error warning child cpl.father;
-             child_born_after_his_parent base error warning child cpl.mother;
-             child_born_before_mother_death base warning child cpl.mother;
-             possible_father base warning child cpl.father;
+             child_born_after_his_parent base error warning child (father cpl);
+             child_born_after_his_parent base error warning child (mother cpl);
+             child_born_before_mother_death base warning child (mother cpl);
+             possible_father base warning child (father cpl);
              child_has_sex warning child;
              match Adef.od_of_codate child.birth with
              [ Some d -> Some (child, d)
@@ -885,23 +903,6 @@ value check_person base error warning p =
 
 value is_deleted_family fam = fam.fam_index = Adef.ifam_of_int (-1);
 
-value check_base base error warning =
-  do {
-    for i = 0 to base.data.persons.len - 1 do {
-      let p = base.data.persons.get i in check_person base error warning p
-    };
-    for i = 0 to base.data.families.len - 1 do {
-      let fam = base.data.families.get i in
-      if is_deleted_family fam then ()
-      else
-        let cpl = base.data.couples.get i in
-        let des = base.data.descends.get i in
-        check_family base error warning fam cpl des
-    };
-    check_noloop base error;
-  }
-;
-
 value strip_all_trailing_spaces s =
   let b = Buffer.create (String.length s) in
   let len =
@@ -1049,32 +1050,12 @@ value map_family_ps fp fs fam =
    fam_index = fam.fam_index}
 ;
 
-value map_couple_p fp fam = {father = fp fam.father; mother = fp fam.mother};
+value map_couple_p multi_parents fp cpl =
+  parent multi_parents (Array.map fp (parent_array cpl))
+;
 
 value map_descend_p fp des = {children = Array.map fp des.children};
 
-(*
-value string_of_place p =
-  loop "" 0 where rec loop s i =
-    if i == Array.length p then s
-    else if s = "" then loop p.(i) (i + 1)
-    else s ^ "," ^ loop p.(i) (i + 1)
-;
-
-value place_of_string s =
-  loop [] 0 0 where rec loop list ibeg i =
-    if i == String.length s then
-      let list =
-        if i == ibeg then list else [String.sub s ibeg (i - ibeg) :: list]
-      in
-      Array.of_list (List.rev list)
-    else if s.[i] == ',' then
-      let list = [String.sub s ibeg (i - ibeg) :: list] in
-      loop list (i + 1) (i + 1)
-    else loop list ibeg (i + 1)
-;
-*)
-
 value arg_list_of_string line =
   loop [] 0 0 None where rec loop list i len quote =
     if i == String.length line then
@@ -1090,3 +1071,58 @@ value arg_list_of_string line =
       | (None, ('"' | ''' as c)) -> loop list (i + 1) 0 (Some c)
       | (None, c) -> loop list (i + 1) (Buff.store len c) None ]
 ;
+
+value sort_person_list base pl =
+  Sort.list
+    (fun p1 p2 ->
+       match
+         (Adef.od_of_codate p1.birth, p1.death, Adef.od_of_codate p2.birth,
+          p2.death)
+       with
+       [ (Some d1, _, Some d2, _) -> strictly_before d1 d2
+       | (Some d1, _, _, Death _ d2) ->
+           strictly_before d1 (Adef.date_of_cdate d2)
+       | (_, Death _ d1, Some d2, _) ->
+           strictly_before (Adef.date_of_cdate d1) d2
+       | (_, Death _ d1, _, Death _ d2) ->
+           strictly_before (Adef.date_of_cdate d1) (Adef.date_of_cdate d2)
+       | (Some _, _, _, _) -> False
+       | (_, Death _ _, _, _) -> False
+       | (_, _, Some _, _) -> True
+       | (_, _, _, Death _ _) -> True
+       | _ ->
+           let c = alphabetic (p_surname base p1) (p_surname base p2) in
+           if c == 0 then
+             let c =
+               alphabetic (p_first_name base p1) (p_first_name base p2)
+             in
+             if c == 0 then p1.occ > p2.occ else c > 0
+           else c > 0 ])
+    pl
+;
+
+value find_free_occ base f s i =
+  let first_name = nominative f in
+  let surname = nominative s in
+  let ipl = base.func.persons_of_name (f ^ " " ^ s) in
+  let first_name = Name.lower f in
+  let surname = Name.lower s in
+  let list_occ =
+    loop [] ipl where rec loop list =
+      fun
+      [ [ip :: ipl] ->
+          let p = poi base ip in
+          if not (List.mem p.occ list) &&
+             first_name = Name.lower (p_first_name base p) &&
+             surname = Name.lower (p_surname base p) then
+            loop [p.occ :: list] ipl
+          else loop list ipl
+      | [] -> list ]
+  in
+  let list_occ = List.sort compare list_occ in
+  loop 0 list_occ where rec loop cnt1 =
+    fun
+    [ [cnt2 :: list] ->
+        if cnt1 = cnt2 then loop (cnt1 + 1) list else cnt1
+    | [] -> cnt1 ]
+;
