@@ -1,5 +1,5 @@
-(* $Id: gutil.mli,v 4.9 2002/11/19 12:43:52 ddr Exp $ *)
-(* Copyright (c) 2002 INRIA *)
+(* $Id: gutil.mli,v 4.18 2004/12/14 09:30:12 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 
@@ -55,7 +55,7 @@ value map_person_ps :
 value map_family_ps :
   ('a -> 'c) -> ('b -> 'd) -> gen_family 'a 'b -> gen_family 'c 'd
 ;
-value map_couple_p : ('a -> 'b) -> gen_couple 'a -> gen_couple 'b;
+value map_couple_p : bool -> ('a -> 'b) -> gen_couple 'a -> gen_couple 'b;
 value map_descend_p : ('a -> 'b) -> gen_descend 'a -> gen_descend 'b;
 
 (* check base *)
@@ -98,10 +98,6 @@ value check_noloop_for_person_list :
   base -> (base_error -> unit) -> list iper -> unit
 ;
 
-value check_base :
-  base -> (base_error -> unit) -> (base_warning -> unit) -> unit
-;
-
 value strip_spaces : string -> string;
 value alphabetic : string -> string -> int;
 value initial : string -> int;
@@ -115,3 +111,22 @@ value list_iter_first : (bool -> 'a -> unit) -> list 'a -> unit;
 value surnames_pieces : string -> list string;
 
 value arg_list_of_string : string -> list string;
+
+value sort_person_list : base -> list person -> list person;
+
+value father : gen_couple 'a -> 'a;
+value mother : gen_couple 'a -> 'a;
+value couple : bool -> 'a -> 'a -> gen_couple 'a;
+value parent : bool -> array 'a -> gen_couple 'a;
+value parent_array : gen_couple 'a -> array 'a;
+
+value set_father : gen_couple 'a -> 'a -> unit;
+value set_mother : gen_couple 'a -> 'a -> unit;
+
+value parents : gen_ascend 'a -> option 'a;
+value consang : gen_ascend 'a -> Adef.fix;
+value no_ascend : unit -> gen_ascend 'a;
+value set_parents : gen_ascend 'a -> option 'a -> unit;
+value set_consang : gen_ascend 'a -> Adef.fix -> unit;
+
+value find_free_occ : base -> string -> string -> int -> int;
