@@ -1,5 +1,5 @@
-(* $Id: gwcomp.ml,v 4.5 2002/11/14 04:15:34 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: gwcomp.ml,v 4.13 2004/12/14 09:30:12 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -29,7 +29,7 @@ value copy_decode s i1 i2 =
   in
   let rec loop_copy t i j =
     if i >= i2 then t
-    else if i == i2 - 1 then do { t.[j] := s.[i]; t }
+    else if i == i2 - 1 && s.[i] <> '_' then do { t.[j] := s.[i]; t }
     else do {
       let (c, i) =
         match s.[i] with
@@ -490,6 +490,7 @@ value get_mar_date str =
               | _ -> Neuter ]
             in
             ((NoSexesCheck, decode_sex 0, decode_sex 1), l)
+        | ["#noment" :: l] -> ((NoMention, Male, Female), l)
         | _ -> ((Married, Male, Female), l) ]
       in
       let (place, l) = get_field "#mp" l in
@@ -750,7 +751,7 @@ value read_family ic fname =
           | Some (str, ["cbp" :: _]) -> failwith str
           | _ -> ("", line) ]
         in
-        let co = {father = cle_pere; mother = cle_mere} in
+        let co = couple False cle_pere cle_mere in
         let (comm, line) =
           match line with
           [ Some (str, ["comm" :: _]) ->
