@@ -1,6 +1,6 @@
 (* camlp4r pa_extend.cmo ./pa_html.cmo ./pa_lock.cmo *)
-(* $Id: gwd.ml,v 4.52 2002/12/30 18:40:04 ddr Exp $ *)
-(* Copyright (c) 2002 INRIA *)
+(* $Id: gwd.ml,v 4.69 2004/12/14 09:53:21 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
@@ -36,7 +36,7 @@ value trace_failed_passwd = ref False;
 
 value log_oc () =
   if log_file.val <> "" then
-    try open_out_gen log_flags 0o644 log_file.val with
+    try Secure.open_out_gen log_flags 0o644 log_file.val with
     [ Sys_error _ -> do { log_file.val := ""; stderr } ]
   else stderr
 ;
@@ -129,7 +129,7 @@ value log_passwd_failed passwd uauth oc tm from request base_file =
 ;
 
 value copy_file fname =
-  match try Some (open_in fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_in fname) with [ Sys_error _ -> None ] with
   [ Some ic ->
       do {
         try
@@ -149,7 +149,7 @@ value http answer =
 ;
 
 value refuse_log from cgi =
-  let oc = open_out_gen log_flags 0o644 "refuse_log" in
+  let oc = Secure.open_out_gen log_flags 0o644 "refuse_log" in
   do {
     let tm = Unix.localtime (Unix.time ()) in
     fprintf_date oc tm;
@@ -214,7 +214,7 @@ value input_lexicon lang =
   let t = Hashtbl.create 501 in
   try
     let ic =
-      open_in
+      Secure.open_in
         (Util.search_in_lang_path (Filename.concat "lang" "lexicon.txt"))
     in
     let derived_lang =
@@ -271,7 +271,7 @@ value alias_lang lang =
     let fname =
       Util.search_in_lang_path (Filename.concat "lang" "alias_lg.txt")
     in
-    match try Some (open_in fname) with [ Sys_error _ -> None ] with
+    match try Some (Secure.open_in fname) with [ Sys_error _ -> None ] with
     [ Some ic ->
         let lang =
           try
@@ -312,7 +312,7 @@ value strip_trailing_spaces s =
 
 value read_base_env cgi bname =
   let fname = Util.base_path [] (bname ^ ".gwf") in
-  match try Some (open_in fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_in fname) with [ Sys_error _ -> None ] with
   [ Some ic ->
       let env =
         loop [] where rec loop env =
@@ -410,41 +410,6 @@ value print_redirected conf from request new_addr =
   }
 ;
 
-value start_with_base conf bname =
-  let bfile = Util.base_path [] (bname ^ ".gwb") in
-  match try Left (Iobase.input bfile) with e -> Right e with
-  [ Left base ->
-      do {
-        try
-          Family.family conf base (log_file.val, log_oc, flush_log)
-        with exc ->
-          do { base.func.cleanup (); raise exc };
-        base.func.cleanup ();
-      }
-  | Right e ->
-      let transl conf w =
-        try Hashtbl.find conf.lexicon w with [ Not_found -> "[" ^ w ^ "]" ]
-      in
-      let title _ =
-        Wserver.wprint "%s" (Util.capitale (transl conf "error"))
-      in
-      do {
-        Util.rheader conf title;
-        Wserver.wprint "<ul>";
-        Util.html_li conf;
-        Wserver.wprint "%s"
-          (Util.capitale (transl conf "cannot access base"));
-        Wserver.wprint " \"%s\".</ul>\n" conf.bname;
-        match e with
-        [ Sys_error _ -> ()
-        | _ ->
-            Wserver.wprint
-              "<em><font size=-1>Internal message: %s</font></em>\n"
-              (Printexc.to_string e) ];
-        Util.trailer conf;
-      } ]
-;
-
 value propose_base conf =
   let title _ = Wserver.wprint "Base" in
   do {
@@ -486,40 +451,58 @@ value unauth_server conf passwd =
           "" conf.env
     in
     Wserver.wprint "\
-<head>
+<html><head>
 <title>%s access failed for database %s</title>
 </head>
 " typ conf.bname;
     Wserver.wprint "<body><h1>%s access failed for database %s</h1>"
       typ conf.bname;
     Wserver.wprint "Return to <a href=\"%s\">welcome page</a>\n" url;
-    Wserver.wprint "</body>\n";
+    Wserver.wprint "</body></html>\n";
   }
 ;
 
 value match_auth_file auth_file uauth =
-  if auth_file = "" then False
+  if auth_file = "" then None
   else
     let auth_file = Util.base_path [] auth_file in
-    match try Some (open_in auth_file) with [ Sys_error _ -> None ] with
+    match try Some (Secure.open_in auth_file) with [ Sys_error _ -> None ] with
     [ Some ic ->
         try
           let rec loop () =
-            let sauth = input_line ic in
-            let sauth =
+            let line = input_line ic in
+            let sauth = line in
+            let (sauth, i) =
               try
                 let i = String.index sauth ':' in
-                let i = String.index_from sauth (i + 1) ':' in
-                String.sub sauth 0 i
+                let i =
+                  try String.index_from sauth (i + 1) ':' with
+                  [ Not_found -> String.length sauth ]
+                in
+                (String.sub sauth 0 i, i)
               with
-              [ Not_found -> sauth ]
+              [ Not_found -> (uauth ^ "a" (* <> auth *), String.length sauth) ]
             in
-            if uauth = sauth then do { close_in ic; True } else loop ()
+            if uauth = sauth then
+              do {
+                close_in ic;
+                let username =
+                  if i = String.length line then ""
+                  else
+                    try
+                      let j = String.index_from line (i + 1) ':' in
+                      String.sub line (i + 1) (j - i - 1)
+                    with
+                    [ Not_found -> "" ]
+                in
+                Some username
+              }
+            else loop ()
           in
           loop ()
         with
-        [ End_of_file -> do { close_in ic; False } ]
-    | None -> False ]
+        [ End_of_file -> do { close_in ic; None } ]
+    | None -> None ]
 ;
 
 value match_simple_passwd sauth uauth =
@@ -533,7 +516,7 @@ value match_simple_passwd sauth uauth =
 ;
 
 value match_auth passwd auth_file uauth =
-  if passwd <> "" && match_simple_passwd passwd uauth then True
+  if passwd <> "" && match_simple_passwd passwd uauth then Some ""
   else match_auth_file auth_file uauth
 ;
 
@@ -547,7 +530,7 @@ value compatible_tokens check_from (addr1, base1_pw1) (addr2, base2_pw2) =
 
 value get_actlog check_from utm from_addr base_password =
   let fname = Srcfile.adm_file "actlog" in
-  match try Some (open_in fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_in fname) with [ Sys_error _ -> None ] with
   [ Some ic ->
       let tmout = float_of_int login_timeout.val in
       let rec loop changed r list =
@@ -593,7 +576,7 @@ value get_actlog check_from utm from_addr base_password =
 
 value set_actlog list =
   let fname = Srcfile.adm_file "actlog" in
-  match try Some (open_out fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_out fname) with [ Sys_error _ -> None ] with
   [ Some oc ->
       do {
         List.iter
@@ -774,7 +757,17 @@ value make_conf cgi from_addr (addr, request) script_name contents env =
         in
         let access_type =
           match passwd with
-          [ "" | "w" | "f" -> ATnone
+          [ "" ->
+             if not cgi then ATnone
+             else
+               let mode = try Sys.getenv "GW_MODE" with [ Not_found -> "" ] in
+               let r_user = try Sys.getenv "REMOTE_USER" with [ Not_found -> "" ] in
+               match (mode, r_user) with
+               [ (_, "") -> ATnone
+               | ("F", u) -> ATfriend u
+               | ("W", u) -> ATwizard u
+               | _ -> ATnone ]                  
+          | "w" | "f" -> ATnone
           | _ -> get_token True utm from_addr base_passwd ]
         in
         (passwd, env, access_type)
@@ -842,35 +835,40 @@ value make_conf cgi from_addr (addr, request) script_name contents env =
         Base64.decode (String.sub auth i (String.length auth - i))
     in
     let uauth = if passwd = "w" || passwd = "f" then passwd1 else passwd in
-    let (ok, wizard, friend) =
+    let (ok, wizard, friend, username) =
       match access_type with
-      [ ATwizard user -> (True, True, False)
-      | ATfriend user -> (True, False, True)
-      | ATnormal -> (True, False, False)
+      [ ATwizard user -> (True, True, False, "")
+      | ATfriend user -> (True, False, True, "")
+      | ATnormal -> (True, False, False, "")
       | ATnone | ATset ->
           if not cgi && (passwd = "w" || passwd = "f") then
             if passwd = "w" then
               if wizard_passwd = "" && wizard_passwd_file = "" then
-                (True, True, friend_passwd = "")
-              else if match_auth wizard_passwd wizard_passwd_file uauth then
-                (True, True, False)
-              else (False, False, False)
+                (True, True, friend_passwd = "", "")
+              else
+                match match_auth wizard_passwd wizard_passwd_file uauth with
+                [ Some username -> (True, True, False, username)
+                | None -> (False, False, False, "") ]
             else if passwd = "f" then
               if friend_passwd = "" && friend_passwd_file = "" then
-                (True, False, True)
-              else if match_auth friend_passwd friend_passwd_file uauth then
-                (True, False, True)
-              else (False, False, False)
+                (True, False, True, "")
+              else
+                match match_auth friend_passwd friend_passwd_file uauth with
+                [ Some username -> (True, False, True, username)
+                | None -> (False, False, False, "") ]
             else assert False
           else if wizard_passwd = "" && wizard_passwd_file = "" then
-            (True, True, friend_passwd = "")
-          else if match_auth wizard_passwd wizard_passwd_file uauth then
-            (True, True, False)
-          else if friend_passwd = "" && friend_passwd_file = "" then
-            (True, False, True)
-          else if match_auth friend_passwd friend_passwd_file uauth then
-            (True, False, True)
-          else (True, False, False) ]
+            (True, True, friend_passwd = "", "")
+          else
+             match match_auth wizard_passwd wizard_passwd_file uauth with
+             [ Some username -> (True, True, False, username)
+             | _ ->
+                  if friend_passwd = "" && friend_passwd_file = "" then
+                    (True, False, True, "")
+                  else
+                  match match_auth friend_passwd friend_passwd_file uauth with
+                    [ Some username -> (True, False, True, username)
+                  | None -> (True, False, False, "") ] ] ]
     in
     let user =
       match lindex uauth ':' with
@@ -911,9 +909,11 @@ value make_conf cgi from_addr (addr, request) script_name contents env =
       | _ -> False ]
     in
     let conf =
-      {wizard = wizard && not wizard_just_friend;
+      {from = from_addr;
+       wizard = wizard && not wizard_just_friend;
        friend = friend || wizard_just_friend && wizard;
-       just_friend_wizard = wizard && wizard_just_friend; user = user;
+       just_friend_wizard = wizard && wizard_just_friend;
+       user = user; username = username;
        passwd = passwd1; cgi = cgi; command = command;
        indep_command = (if cgi then command else "geneweb") ^ "?";
        highlight =
@@ -921,6 +921,9 @@ value make_conf cgi from_addr (addr, request) script_name contents env =
          [ Not_found -> green_color ];
        lang = if lang = "" then default_lang else lang;
        default_lang = default_lang;
+       multi_parents =
+         try List.assoc "multi_parents" base_env = "yes" with
+         [ Not_found -> False ];
        can_send_image =
          try List.assoc "can_send_image" base_env <> "no" with
          [ Not_found -> True ];
@@ -1027,7 +1030,9 @@ value auth_err request auth_file =
   else
     let auth = Wserver.extract_param "authorization: " '\r' request in
     if auth <> "" then
-      match try Some (open_in auth_file) with [ Sys_error _ -> None ] with
+      match
+        try Some (Secure.open_in auth_file) with [ Sys_error _ -> None ]
+      with
       [ Some ic ->
           let auth =
             let i = String.length "Basic " in
@@ -1075,61 +1080,62 @@ value conf_and_connection cgi from (addr, request) script_name contents env =
         else if cgi then (True, "")
         else auth_err request conf.auth_file
       in
-      match (cgi, auth_err, passwd_err) with
-      [ (True, True, _) ->
-          if is_robot from then Robot.robot_error cgi from 0 0
-          else no_access conf
-      | (_, True, _) ->
-          if is_robot from then Robot.robot_error cgi from 0 0
-          else
-            let auth_type =
-              let x =
-                try List.assoc "auth_file" conf.base_env with
-                [ Not_found -> "" ]
+      let mode = Util.p_getenv conf.env "m" in
+      do {
+        if mode <> Some "IM" then
+          let contents =
+            if List.mem_assoc "log_pwd" env then "..." else contents
+          in
+          log_and_robot_check conf auth from request script_name contents
+        else ();
+        match (cgi, auth_err, passwd_err) with
+        [ (True, True, _) ->
+            if is_robot from then Robot.robot_error cgi from 0 0
+            else no_access conf
+        | (_, True, _) ->
+            if is_robot from then Robot.robot_error cgi from 0 0
+            else
+              let auth_type =
+                let x =
+                  try List.assoc "auth_file" conf.base_env with
+                  [ Not_found -> "" ]
+                in
+                if x = "" then "GeneWeb service" else "database " ^ conf.bname
               in
-              if x = "" then "GeneWeb service" else "database " ^ conf.bname
-            in
-            refuse_auth conf from auth auth_type
-      | (_, _, Some (passwd, uauth)) ->
-          if is_robot from then Robot.robot_error cgi from 0 0
-          else do {
-            let tm = Unix.time () in
-            lock_wait Srcfile.adm_file "gwd.lck" with
-            [ Accept ->
-                let oc = log_oc () in
-                do {
-                  log_passwd_failed passwd uauth oc tm from request
-                    conf.bname;
-                  flush_log oc;
-                }
-            | Refuse -> () ];
-            unauth_server conf passwd;
-          }
-      | _ ->
-          let mode = Util.p_getenv conf.env "m" in
-          do {
-            if mode <> Some "IM" then
-              let contents =
-                if List.mem_assoc "log_pwd" env then "..." else contents
-              in
-              log_and_robot_check conf auth from request script_name contents
-            else ();
+              refuse_auth conf from auth auth_type
+        | (_, _, Some (passwd, uauth)) ->
+            if is_robot from then Robot.robot_error cgi from 0 0
+            else do {
+              let tm = Unix.time () in
+              lock_wait Srcfile.adm_file "gwd.lck" with
+              [ Accept ->
+                  let oc = log_oc () in
+                  do {
+                    log_passwd_failed passwd uauth oc tm from request
+                      conf.bname;
+                    flush_log oc;
+                  }
+              | Refuse -> () ];
+              unauth_server conf passwd;
+            }
+        | _ ->
             match mode with
             [ Some "DOC" -> Doc.print conf
             | _ ->
-                if conf.bname = "" then general_welcome conf
-                else
-                  match
-                    try Some (List.assoc "renamed" conf.base_env) with
-                    [ Not_found -> None ]
-                  with
-                  [ Some n when n <> "" -> print_renamed conf n
-                  | _ ->
-                      do {
-                        start_with_base conf conf.bname;
-                        if sleep > 0 then Unix.sleep sleep else ();
-                      } ] ];
-          } ] ]
+               if conf.bname = "" then general_welcome conf
+               else
+                 match
+                   try Some (List.assoc "renamed" conf.base_env) with
+                   [ Not_found -> None ]
+                 with
+                 [ Some n when n <> "" -> print_renamed conf n
+                 | _ ->
+                     do {
+                       Family.treat_request_on_base conf
+                         (log_file.val, log_oc, flush_log);
+                       if sleep > 0 then Unix.sleep sleep else ();
+                     } ] ] ]
+      } ]
 ;
 
 value chop_extension name =
@@ -1324,7 +1330,7 @@ value geneweb_server () =
         | None -> try Unix.gethostname () with _ -> "computer" ]
       in
       eprintf "GeneWeb %s - " Version.txt;
-      eprintf "Copyright (c) 2003 INRIA\n";
+      eprintf "Copyright (c) 1998-2005 INRIA\n";
       if not daemon.val then do {
         eprintf "Possible addresses:";
         eprintf "
@@ -1486,13 +1492,11 @@ value main () =
       " [options] where options are:"
     in
     let speclist =
-      [("-hd",
-        Arg.String (fun x -> Util.lang_path.val := [x :: Util.lang_path.val]),
+      [("-hd", Arg.String Util.add_lang_path,
         "<dir>\n       Directory where the directory lang is installed.");
-       ("-dd",
-        Arg.String (fun x -> Util.doc_path.val := [x :: Util.doc_path.val]),
+       ("-dd", Arg.String Util.add_doc_path,
         "<dir>\n       Directory where the documentation is installed.");
-       ("-bd", Arg.String (fun x -> Util.base_dir.val := x),
+       ("-bd", Arg.String Util.set_base_dir,
         "<dir>\n       Directory where the databases are installed.");
        ("-wd", Arg.String make_cnt_dir, "\
 <dir>
@@ -1533,6 +1537,9 @@ value main () =
        ("-robot_xcl", Arg.String robot_exclude_arg, "\
 <cnt>,<sec>
        Exclude connections when more than <cnt> requests in <sec> seconds.");
+       ("-min_disp_req", Arg.Int (fun x -> Robot.min_disp_req.val := x),
+        "\n       Minimum number of requests in robot trace (default: " ^
+        string_of_int Robot.min_disp_req.val ^ ")");
        ("-login_tmout", Arg.Int (fun x -> login_timeout.val := x), "\
 <sec>
        Login timeout for entries with passwords in CGI mode (default " ^ string_of_int login_timeout.val ^ "\
@@ -1587,11 +1594,11 @@ s)"); ("-redirect", Arg.String (fun x -> redirected_addr.val := Some x), "\
       in
       Util.images_url.val := "file://" ^ slashify abs_dir
     else ();
-    if Util.doc_path.val = [] then
-      Util.doc_path.val :=
-        List.map (fun d -> Filename.concat d "doc") Util.lang_path.val
+    if Secure.doc_path () = [] then
+      List.iter (fun d -> Util.add_doc_path (Filename.concat d "doc"))
+        (List.rev (Secure.lang_path ()))
     else ();
-    if Util.cnt_dir.val = "" then Util.cnt_dir.val := Util.base_dir.val
+    if Util.cnt_dir.val = "" then Util.cnt_dir.val := Secure.base_dir ()
     else ();
     let (query, cgi) =
       try (Sys.getenv "QUERY_STRING", True) with
