@@ -1,5 +1,5 @@
-(* $Id: gwu.ml,v 4.13 2002/10/22 13:18:24 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: gwu.ml,v 4.26 2004/12/14 09:30:12 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -19,7 +19,7 @@ value print_date_dmy oc d =
     | Before -> fprintf oc "<"
     | After -> fprintf oc ">"
     | _ -> () ];
-    if d.day == 0 && d.month == 0 then fprintf oc "%s" (soy d.year)
+    if (*d.day == 0 &&*) d.month == 0 then fprintf oc "%s" (soy d.year)
     else if d.day == 0 then fprintf oc "%d/%s" d.month (soy d.year)
     else fprintf oc "%d/%d/%s" d.day d.month (soy d.year);
     match d.prec with
@@ -82,7 +82,8 @@ value gen_correct_string no_colon s =
   let s = strip_spaces s in
   loop 0 0 where rec loop i len =
     if i == String.length s then Buff.get len
-    else if len == 0 && not (starting_char s) then loop i (Buff.store len '_')
+    else if len == 0 && not (starting_char s) then
+      loop i (Buff.store len '_')
     else
       match s.[i] with
       [ ' ' | '\n' | '\t' ->
@@ -97,7 +98,10 @@ value gen_correct_string no_colon s =
           loop (i + 1) (Buff.store len c) ]
 ;
 
-value s_correct_string = gen_correct_string False;
+value s_correct_string s =
+  let s = gen_correct_string False s in
+  if s = "" then "_" else s
+;
 value correct_string base is = s_correct_string (sou base is);
 value correct_string_no_colon base is = gen_correct_string True (sou base is);
 
@@ -254,7 +258,7 @@ value print_infos oc base is_child csrc cbp p =
 value print_parent oc base mark fam_sel fam p =
   let a = aoi base p.cle_index in
   let has_printed_parents =
-    match a.parents with
+    match parents a with
     [ Some ifam -> fam_sel ifam
     | None -> False ]
   in
@@ -346,7 +350,7 @@ value print_witness oc base mark p notes_pl_p =
     fprintf oc "%s %s%s" (correct_string base p.surname)
       (correct_string base p.first_name)
       (if p.occ = 0 then "" else "." ^ string_of_int p.occ);
-    if Array.length u.family = 0 && a.parents = None &&
+    if Array.length u.family = 0 && parents a = None &&
        not mark.(Adef.int_of_iper p.cle_index)
     then do {
       mark.(Adef.int_of_iper p.cle_index) := True;
@@ -378,7 +382,8 @@ value print_family oc base mark (per_sel, fam_sel) fam_done notes_pl_p m =
           | Female -> 'f'
           | Neuter -> '?' ]
         in
-        fprintf oc " #nsck %c%c" (c m.m_fath) (c m.m_moth) ];
+        fprintf oc " #nsck %c%c" (c m.m_fath) (c m.m_moth)
+    | NoMention -> fprintf oc " #noment" ];
     print_if_no_empty oc base "#mp" fam.marriage_place;
     print_if_no_empty oc base "#ms" fam.marriage_src;
     match fam.divorce with
@@ -444,12 +449,12 @@ value get_persons_with_notes base m list =
   let fath = m.m_fath in
   let moth = m.m_moth in
   let list =
-    match (sou base fath.notes, (aoi base fath.cle_index).parents) with
+    match (sou base fath.notes, parents (aoi base fath.cle_index)) with
     [ ("", _) | (_, Some _) -> list
     | _ -> [fath :: list] ]
   in
   let list =
-    match (sou base moth.notes, (aoi base moth.cle_index).parents) with
+    match (sou base moth.notes, parents (aoi base moth.cle_index)) with
     [ ("", _) | (_, Some _) -> list
     | _ -> [moth :: list] ]
   in
@@ -498,13 +503,13 @@ value print_notes oc base ml per_sel pl =
 ;
 
 value is_isolated base p =
-  match (aoi base p.cle_index).parents with
+  match parents (aoi base p.cle_index) with
   [ Some _ -> False
   | None -> Array.length (uoi base p.cle_index).family = 0 ]
 ;
 
 value is_definition_for_parent base p =
-  match (aoi base p.cle_index).parents with
+  match parents (aoi base p.cle_index) with
   [ Some _ -> False
   | None -> True ]
 ;
@@ -516,10 +521,10 @@ value get_isolated_related base m list =
     else if is_isolated base p then
       match p.rparents with
       [ [{r_fath = Some x} :: _] when x = p_relation.cle_index ->
-          [(p, True) :: list]
+          list @ [(p, True)]
       | [{r_fath = None; r_moth = Some x} :: _]
         when x = p_relation.cle_index ->
-          [(p, True) :: list]
+          list @ [(p, True)]
       | _ -> list ]
     else list
   in
@@ -545,12 +550,12 @@ value get_persons_with_relations base m list =
   let fath = m.m_fath in
   let moth = m.m_moth in
   let list =
-    match (fath.rparents, (aoi base fath.cle_index).parents) with
+    match (fath.rparents, parents (aoi base fath.cle_index)) with
     [ ([], _) | (_, Some _) -> list
     | _ -> [(fath, False) :: list] ]
   in
   let list =
-    match (moth.rparents, (aoi base moth.cle_index).parents) with
+    match (moth.rparents, parents (aoi base moth.cle_index)) with
     [ ([], _) | (_, Some _) -> list
     | _ -> [(moth, False) :: list] ]
   in
@@ -558,7 +563,7 @@ value get_persons_with_relations base m list =
     List.fold_right
       (fun ip list ->
          let p = poi base ip in
-         match (p.rparents, (aoi base p.cle_index).parents) with
+         match (p.rparents, parents (aoi base p.cle_index)) with
          [ ([], _) | (_, Some _) -> list
          | ([{r_fath = Some x} :: _], _) when x <> m.m_fath.cle_index -> list
          | _ -> [(p, False) :: list] ])
@@ -579,7 +584,7 @@ value print_relation_parent oc base mark defined_p p =
     fprintf oc "%s %s%s" (correct_string base p.surname)
       (correct_string base p.first_name)
       (if p.occ = 0 then "" else "." ^ string_of_int p.occ);
-    if Array.length u.family = 0 && a.parents = None &&
+    if Array.length u.family = 0 && parents a = None &&
        not mark.(Adef.int_of_iper p.cle_index)
     then do {
       mark.(Adef.int_of_iper p.cle_index) := True;
@@ -722,7 +727,7 @@ value rec filter f =
 ;
 
 value connected_families base fam_sel fam cpl =
-  loop [fam.fam_index] [] [cpl.father] where rec loop ifaml ipl_scanned =
+  loop [fam.fam_index] [] [(father cpl)] where rec loop ifaml ipl_scanned =
     fun
     [ [ip :: ipl] ->
         if List.memq ip ipl_scanned then loop ifaml ipl_scanned ipl
@@ -735,7 +740,7 @@ value connected_families base fam_sel fam cpl =
             List.fold_right
               (fun ifam ipl ->
                  let cpl = coi base ifam in
-                 [cpl.father; cpl.mother :: ipl])
+                 [(father cpl); (mother cpl) :: ipl])
               ifaml1 ipl
           in
           loop ifaml [ip :: ipl_scanned] ipl
@@ -763,11 +768,11 @@ type separate =
 ;
 
 value rec find_ancestors base surn p list =
-  match (aoi base p.cle_index).parents with
+  match parents (aoi base p.cle_index) with
   [ Some ifam ->
       let cpl = coi base ifam in
-      let fath = poi base cpl.father in
-      let moth = poi base cpl.mother in
+      let fath = poi base (father cpl) in
+      let moth = poi base (mother cpl) in
       if fath.surname <> surn && moth.surname <> surn then [p :: list]
       else
         let list =
@@ -840,21 +845,21 @@ value scan_connex_component base test_action len ifam =
       Array.fold_left
         (fun len ifam1 ->
            if ifam1 = ifam then len else test_action loop len ifam1)
-        len (uoi base cpl.father).family
+        len (uoi base (father cpl)).family
     in
     let len =
       Array.fold_left
         (fun len ifam1 ->
            if ifam1 = ifam then len else test_action loop len ifam1)
-        len (uoi base cpl.mother).family
+        len (uoi base (mother cpl)).family
     in
     let len =
-      match (aoi base cpl.father).parents with
+      match parents (aoi base (father cpl)) with
       [ Some ifam -> test_action loop len ifam
       | _ -> len ]
     in
     let len =
-      match (aoi base cpl.mother).parents with
+      match parents (aoi base (mother cpl)) with
       [ Some ifam -> test_action loop len ifam
       | _ -> len ]
     in
@@ -898,8 +903,8 @@ value mark_one_connex_component base mark ifam =
   else do {
     eprintf "%s: group of size %d not included\n" origin_file len;
     let cpl = coi base ifam in
-    eprintf "    %s + %s\n" (designation base (poi base cpl.father))
-      (designation base (poi base cpl.mother));
+    eprintf "    %s + %s\n" (designation base (poi base (father cpl)))
+      (designation base (poi base (mother cpl)));
     flush stderr;
     set_mark Scanned
   }
@@ -950,6 +955,7 @@ value no_spouses_parents = ref False;
 value no_notes = ref False;
 value censor = ref 0;
 value with_siblings = ref False;
+value maxlev = ref (-1);
 
 value gwu base out_dir out_oc src_oc_list anc desc ancdesc =
   let to_separate = separate base in
@@ -970,7 +976,7 @@ value gwu base out_dir out_oc src_oc_list anc desc ancdesc =
   in
   let ((per_sel, fam_sel) as sel) =
     Select.functions base anc desc surnames.val ancdesc no_spouses_parents.val
-      censor.val with_siblings.val
+      censor.val with_siblings.val maxlev.val
   in
   let fam_done = Array.create base.data.families.len False in
   let mark = Array.create base.data.persons.len False in
@@ -1004,8 +1010,8 @@ value gwu base out_dir out_oc src_oc_list anc desc ancdesc =
                let cpl = coi base ifam in
                let des = doi base ifam in
                let m =
-                 {m_fam = fam; m_fath = poi base cpl.father;
-                  m_moth = poi base cpl.mother;
+                 {m_fam = fam; m_fath = poi base (father cpl);
+                  m_moth = poi base (mother cpl);
                   m_chil = Array.map (fun ip -> poi base ip) des.children}
                in
                if empty_family base m then do {
@@ -1097,6 +1103,8 @@ value speclist =
     "\"<1st_name>\" [num] \"<surname>\" : select ancestors with siblings");
    ("-s", Arg.String (fun x -> surnames.val := [x :: surnames.val]),
     "\"<surname>\" : select this surname (option usable several times)");
+   ("-maxlev", Arg.Int (fun i -> maxlev.val := i),
+    "\"<level>\" : maximum level of generations of descendants");
    ("-nsp", Arg.Set no_spouses_parents,
     ": no spouses' parents (for options -s and -d)");
    ("-nn", Arg.Set no_notes, ": no (database) notes");
@@ -1181,6 +1189,7 @@ value main () =
       exit 2
     }
     else ();
+    Secure.set_base_dir (Filename.dirname in_file.val);
     let anc =
       if anc_1st.val <> "" then
         if anc_2nd.val = "" then do {
@@ -1225,8 +1234,9 @@ value main () =
     let _ = base.data.ascends.array () in
     let _ = base.data.strings.array () in
     if not mem.val then
-      let _ = base.data.persons.array () in
+(*
       let _ = base.data.families.array () in
+*)
       let _ = base.data.couples.array () in
       let _ = base.data.unions.array () in
       let _ = base.data.descends.array () in
