@@ -1,6 +1,6 @@
 (* camlp4r *)
-(* $Id: image.ml,v 4.5 2002/07/15 08:34:24 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: image.ml,v 4.11 2004/12/14 09:30:12 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Util;
 open Config;
@@ -21,7 +21,7 @@ value content cgi t len fname =
 ;
 
 value print_image_type cgi fname itype =
-  match try Some (open_in_bin fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_in_bin fname) with [ Sys_error _ -> None ] with
   [ Some ic ->
       let buf = String.create 1024 in
       let len = in_channel_length ic in
@@ -81,3 +81,20 @@ value print conf base =
       [ Some p -> print_personal_image conf base p
       | _ -> incorrect_request conf ] ]
 ;
+
+value print_html conf base =
+  do {
+    Util.html1 conf;
+    Wserver.wprint "<html>\n<head>\n";
+    Wserver.wprint "  <title>%s</title>\n"
+      (Util.transl_nth conf "image/images" 0);
+    Wserver.wprint "</head>\n<body>\n";
+    Wserver.wprint "<img src=\"%s" (commd conf);
+    Gutil.list_iter_first
+      (fun first (k, v) ->
+	 let v = if k = "m" then "IM" else v in
+	 Wserver.wprint "%s%s=%s" (if first then "" else ";") k v)
+      conf.env;
+    Wserver.wprint "\">\n</body>\n</html>";
+  }
+;
