@@ -1,5 +1,5 @@
-(* $Id: iobase.ml,v 4.15 2002/03/15 19:18:51 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: iobase.ml,v 4.33 2004/12/14 09:30:12 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -51,6 +51,8 @@ value magic_gwb = "GnWb001y";
             surname (modulo length)
           - to the array of the corresponding string indexes
 
+    names.acc - direct accesses to arrays inside names.inx
+
     strings.inx - index for strings, surnames, first names
        length of the strings offset array : binary_int
        offset of surnames index           : binary_int
@@ -62,15 +64,39 @@ value magic_gwb = "GnWb001y";
          strings list array (length = string array length)
            - associating a string index
            - to the index of the next index holding the same hash value
+       -- the following table has been obsolete since version 4.10
+       -- it has been replaced by snames.inx/sname.dat which use
+       -- much less memory
        surnames index                     : value
          binary tree
           - associating the string index of a surname
           - to the corresponding list of persons holding this surname
+       -- the following table has been obsolete since version 4.10
+       -- it has been replaced by fnames.inx/fname.dat which use
+       -- much less memory
        first_names index                  : value
          binary tree
           - associating the string index of a first name
           - to the corresponding list of persons holding this first name
 
+    snames.inx - index for surnames
+       binary tree
+        - associating the string index of a surname
+        - to a pointer (int) to snames.dat
+
+    snames.dat - data associated with snames.inx
+      table of list of persons holding a surname
+
+    fnames.inx - index for first names
+       binary tree
+        - associating the string index of a first name
+        - to a pointer (int) to fnames.dat
+
+    fnames.dat - data associated with fnames.inx
+      table of list of persons holding a first name
+
+the corresponding list of persons holding this surname
+
     patches - patches
        When updated, none of the previous files are modified. Only this one
        is written and rewritten. It holds a record of type "patches", composed
@@ -90,17 +116,28 @@ value output_value_no_sharing oc v =
   Marshal.to_channel oc v [Marshal.No_sharing]
 ;
 
-value array_header_size arr = if Array.length arr < 8 then 1 else 5;
+value array_header_size arr = if arr.len < 8 then 1 else 5;
 
 value output_array_access oc arr pos =
   loop (pos + output_value_header_size + array_header_size arr) 0 where rec
-    loop pos i =
-    if i == Array.length arr then pos
+  loop pos i =
+    if i == arr.len then pos
     else do {
-      output_binary_int oc pos; loop (pos + Iovalue.size arr.(i)) (i + 1)
+      output_binary_int oc pos; loop (pos + Iovalue.size (arr.get i)) (i + 1)
     }
 ;
 
+value output_cache_array_no_sharing oc arr =
+  do {
+    for i = 1 to output_value_header_size + array_header_size arr do {
+      output_byte oc 0;
+    };
+    for i = 0 to arr.len - 1 do {
+      Iovalue.output oc (arr.get i);
+    };
+  }
+;
+
 (* Search index of a given string in file strings.inx *)
 
 value int_size = 4;
@@ -223,39 +260,62 @@ value rec list_remove_elemq x =
   | [] -> [] ]
 ;
 
-value persons_of_first_name_or_surname base_data strings params =
-  let (ic2, start_pos, proj, person_patches, tree_name) = params in
+(* compatibility with databases created with versions <= 4.09 *)
+(* should be removed after some time (when all databases will have
+   been rebuilt with version >= 4.10 *)
+value old_persons_of_first_name_or_surname base_data strings params =
+  let (ic2, start_pos, proj, person_patches, _, _, _) = params in
   let module IstrTree =
     Btree.Make
       (struct type t = istr; value compare = compare_istr_fun base_data; end)
   in
   let bt =
     let btr = ref None in
-    fun () ->
+    let completed = ref False in
+    let update_bt gistro bt =
+      do {
+        let bt = ref bt in
+        Hashtbl.iter
+          (fun i p ->
+             let istr = proj p in
+             if gistro <> None && gistro <> Some istr then ()
+             else
+               let ipera =
+                 try IstrTree.find istr bt.val with [ Not_found -> [] ]
+               in
+               if List.memq p.cle_index ipera then ()
+               else
+	         bt.val := IstrTree.add istr [p.cle_index :: ipera] bt.val)
+          person_patches;
+        if gistro = None then completed.val := True else ();
+        bt.val
+      }
+    in
+    fun gistro ->
       match btr.val with
-      [ Some bt -> bt
+      [ Some bt ->
+          if completed.val then bt
+          else
+            let bt = update_bt gistro bt in
+            do { btr.val := Some bt; bt }
       | None ->
           do {
             seek_in ic2 start_pos;
+(*
+let ab1 = Gc.allocated_bytes () in
+*)
             let bt : IstrTree.t (list iper) = input_value ic2 in
-            let bt =
-              do {
-                let bt = ref bt in
-                Hashtbl.iter
-                  (fun i p ->
-                     let istr = proj p in
-                     let ipera =
-                       try IstrTree.find istr bt.val with [ Not_found -> [] ]
-                     in
-                     if List.memq p.cle_index ipera then ()
-                     else
-                       bt.val := IstrTree.add istr [p.cle_index :: ipera]
-                         bt.val)
-                  person_patches;
-                bt.val
-              }
-            in
+(*
+let ab2 = Gc.allocated_bytes () in
+*)
+            let bt = update_bt gistro bt in
             btr.val := Some bt;
+(*
+Printf.eprintf "*** old database created by version <= 4.09\n"; flush stderr;
+Printf.eprintf "*** using index allocating here %.0f bytes\n"
+  (ab2 -. ab1);
+flush stderr;
+*)
             bt
           } ]
   in
@@ -275,45 +335,166 @@ value persons_of_first_name_or_surname base_data strings params =
     }
   in
   let find istr =
-    try check_patches istr (IstrTree.find istr (bt ())) with
+    try check_patches istr (IstrTree.find istr (bt (Some istr))) with
     [ Not_found -> [] ]
   in
   let cursor str =
     IstrTree.key_after
       (fun key -> compare_names str (strings.get (Adef.int_of_istr key)))
-      (bt ())
+      (bt None)
   in
-  let next key = IstrTree.next key (bt ()) in
+  let next key = IstrTree.next key (bt None) in
   {find = find; cursor = cursor; next = next}
 ;
 
+value new_persons_of_first_name_or_surname base_data strings params =
+  let (_, _, proj, person_patches, names_inx, names_dat, bname) = params in
+  let module IstrTree =
+    Btree.Make
+      (struct type t = istr; value compare = compare_istr_fun base_data; end)
+  in
+  let fname_dat = Filename.concat bname names_dat in
+  let bt =
+    let btr = ref None in
+    fun () ->
+      match btr.val with
+      [ Some bt -> bt
+      | None ->
+          do {
+            let fname_inx = Filename.concat bname names_inx in
+            let ic_inx = Secure.open_in_bin fname_inx in
+(*
+let ab1 = Gc.allocated_bytes () in
+*)
+            let bt : IstrTree.t int = input_value ic_inx in
+(*
+let ab2 = Gc.allocated_bytes () in
+Printf.eprintf "*** new database created by version >= 4.10\n";
+Printf.eprintf "*** using index '%s' allocating here only %.0f bytes\n"
+  names_inx (ab2 -. ab1);
+flush stderr;
+*)
+            close_in ic_inx;
+            btr.val := Some bt;
+            bt
+          } ]
+  in
+  let find istr =
+    let ipera =
+      try
+        let pos = IstrTree.find istr (bt ()) in
+        let ic_dat = Secure.open_in_bin fname_dat in
+        do {
+          seek_in ic_dat pos;
+          let len = input_binary_int ic_dat in
+          let rec read_loop ipera len =
+            if len = 0 then ipera
+            else
+              let iper = Adef.iper_of_int (input_binary_int ic_dat) in
+              read_loop [iper :: ipera] (len - 1)
+          in
+          let ipera = read_loop [] len in
+          close_in ic_dat;
+          ipera
+        }
+      with
+      [ Not_found -> [] ]
+    in
+    let ipera = ref ipera in
+    do {
+      Hashtbl.iter
+        (fun i p ->
+           let istr1 = proj p in
+           if istr1 <> istr then ()
+           else if List.memq p.cle_index ipera.val then ()
+           else ipera.val := [p.cle_index :: ipera.val])
+        person_patches;
+      ipera.val
+    }
+  in
+  let bt_patched =
+    let btr = ref None in
+    fun () ->
+      match btr.val with
+      [ Some bt -> bt
+      | None ->
+          let bt = ref (bt ()) in
+          do {
+            Hashtbl.iter
+              (fun i p ->
+                 let istr1 = proj p in
+                 try
+                   let _ = IstrTree.find istr1 bt.val in
+                   ()
+                 with
+                 [ Not_found -> bt.val := IstrTree.add istr1 0 bt.val ])
+              person_patches;
+            btr.val := Some bt.val;
+	    bt.val
+          } ]
+  in
+  let cursor str =
+    IstrTree.key_after
+      (fun key -> compare_names str (strings.get (Adef.int_of_istr key)))
+      (bt_patched ())
+  in
+  let next key = IstrTree.next key (bt_patched ()) in
+  {find = find; cursor = cursor; next = next}
+;
+
+value persons_of_first_name_or_surname base_data strings params =
+  let (_, _, _, _, names_inx, _, bname) = params in
+  if Sys.file_exists (Filename.concat bname names_inx) then
+    new_persons_of_first_name_or_surname base_data strings params
+  else
+    old_persons_of_first_name_or_surname base_data strings params
+;
+
 (* Search index for a given name in file names.inx *)
 
 type name_index_data = array (array iper);
+value table_size = 0x3fff;
 
 value persons_of_name bname patches =
   let t = ref None in
   fun s ->
     let s = Name.crush_lower s in
-    let a =
-      match t.val with
-      [ Some a -> a
-      | None ->
-          let ic_inx = open_in_bin (Filename.concat bname "names.inx") in
-          do {
-            seek_in ic_inx int_size;
-            let a : name_index_data = input_value ic_inx in
-            close_in ic_inx;
-            t.val := Some a;
-            a
-          } ]
-    in
     let i = Hashtbl.hash s in
+    let ai =
+      let ic_inx = Secure.open_in_bin (Filename.concat bname "names.inx") in
+      let ai =        
+        let i = i mod table_size in
+        let fname_inx_acc = Filename.concat bname "names.acc" in
+        if Sys.file_exists fname_inx_acc then
+          let ic_inx_acc = Secure.open_in_bin fname_inx_acc in
+          do {
+            seek_in ic_inx_acc (Iovalue.sizeof_long * i);
+            let pos = input_binary_int ic_inx_acc in
+            close_in ic_inx_acc;
+            seek_in ic_inx pos;
+            (Iovalue.input ic_inx : array iper)
+          }
+        else (* compatibility *)
+          let a =
+            match t.val with
+            [ Some a -> a
+            | None ->
+                do {
+                  seek_in ic_inx int_size;
+                  let a : name_index_data = input_value ic_inx in
+                  t.val := Some a;
+                  a
+                } ]
+          in
+          a.(i)
+      in
+      do { close_in ic_inx; ai }          
+    in
     try
       let l = Hashtbl.find patches i in
-      l @ Array.to_list a.(i mod Array.length a)
+      l @ Array.to_list ai
     with
-    [ Not_found -> Array.to_list a.(i mod Array.length a) ]
+    [ Not_found -> Array.to_list ai ]
 ;
 
 type strings_of_fsname = array (array istr);
@@ -322,22 +503,38 @@ value strings_of_fsname bname strings (_, person_patches) =
   let t = ref None in
   fun s ->
     let s = Name.crush_lower s in
-    let a =
-      match t.val with
-      [ Some a -> a
-      | None ->
-          let ic_inx = open_in_bin (Filename.concat bname "names.inx") in
-          let pos = input_binary_int ic_inx in
-          do {
-            seek_in ic_inx pos;
-            let a : strings_of_fsname = input_value ic_inx in
-            close_in ic_inx;
-            t.val := Some a;
-            a
-          } ]
-    in
     let i = Hashtbl.hash s in
-    let r = a.(i mod Array.length a) in
+    let r =
+      let ic_inx = Secure.open_in_bin (Filename.concat bname "names.inx") in
+      let ai =
+        let i = i mod table_size in
+        let fname_inx_acc = Filename.concat bname "names.acc" in
+        if Sys.file_exists fname_inx_acc then
+          let ic_inx_acc = Secure.open_in_bin fname_inx_acc in
+          do {
+            seek_in ic_inx_acc (Iovalue.sizeof_long * (table_size + i));
+            let pos = input_binary_int ic_inx_acc in
+            close_in ic_inx_acc;
+            seek_in ic_inx pos;
+            (Iovalue.input ic_inx : array istr)
+          }
+        else (* compatibility *)
+          let a =
+            match t.val with
+            [ Some a -> a
+            | None ->
+                let pos = input_binary_int ic_inx in
+                do {
+                  seek_in ic_inx pos;
+                  let a : strings_of_fsname = input_value ic_inx in
+                  t.val := Some a;
+                  a
+                } ]
+          in
+          a.(i)
+        in
+        do { close_in ic_inx; ai }
+    in
     let l = ref (Array.to_list r) in
     do {
       Hashtbl.iter
@@ -362,6 +559,7 @@ value strings_of_fsname bname strings (_, person_patches) =
       l.val
     }
 ;
+(**)
 
 value lock_file bname =
   let bname =
@@ -382,7 +580,7 @@ value make_visible_cache bname persons =
   let r = { v_write = fun []; v_get = fun [] } in
   let read_or_create_visible () =
     let visible =
-      match try Some (open_in fname) with [ Sys_error _ -> None ] with
+      match try Some (Secure.open_in fname) with [ Sys_error _ -> None ] with
       [ Some ic ->
           do {
             ifdef UNIX then
@@ -404,7 +602,7 @@ value make_visible_cache bname persons =
     match visible_ref.val with
     [ Some visible ->
         try do {
-          let oc = open_out fname;
+          let oc = Secure.open_out fname;
           ifdef UNIX then
             if verbose.val then do {
               Printf.eprintf "*** write restrict file\n";
@@ -600,6 +798,9 @@ value make_cache ic ic_acc shift array_pos (plenr, patches) len name =
     [ Some x -> x
     | None ->
         do {
+          if name = "persons" then failwith "bug: access to person array"
+          else if name = "families" then failwith "bug: access to family array"
+	  else ();
           ifdef UNIX then
             if verbose.val then do {
               Printf.eprintf "*** read %s%s\n" name
@@ -643,7 +844,9 @@ value make_cache ic ic_acc shift array_pos (plenr, patches) len name =
 value input_patches bname =
   let patches =
     match
-      try Some (open_in_bin (Filename.concat bname "patches")) with _ -> None
+      try
+        Some (Secure.open_in_bin (Filename.concat bname "patches"))
+      with _ -> None
     with
     [ Some ic -> let p = input_value ic in do { close_in ic; p }
     | None ->
@@ -711,7 +914,7 @@ value input bname =
   in
   let patches = input_patches bname in
   let ic =
-    let ic = open_in_bin (Filename.concat bname "base") in
+    let ic = Secure.open_in_bin (Filename.concat bname "base") in
     do { check_magic ic; ic }
   in
   let persons_len = input_binary_int ic in
@@ -725,8 +928,8 @@ value input bname =
   let descends_array_pos = input_binary_int ic in
   let strings_array_pos = input_binary_int ic in
   let norigin_file = input_value ic in
-  let ic_acc = open_in_bin (Filename.concat bname "base.acc") in
-  let ic2 = open_in_bin (Filename.concat bname "strings.inx") in
+  let ic_acc = Secure.open_in_bin (Filename.concat bname "base.acc") in
+  let ic2 = Secure.open_in_bin (Filename.concat bname "strings.inx") in
   let ic2_string_start_pos = 3 * int_size in
   let ic2_string_hash_len = input_binary_int ic2 in
   let ic2_surname_start_pos = input_binary_int ic2 in
@@ -771,18 +974,20 @@ value input bname =
   in
   let cleanup () = cleanup_ref.val () in
   let commit_patches () =
+    let tmp_fname = Filename.concat bname "1patches" in
     let fname = Filename.concat bname "patches" in
     do {
-      try Sys.remove (fname ^ "~") with [ Sys_error _ -> () ];
-      try Sys.rename fname (fname ^ "~") with [ Sys_error _ -> () ];
       let oc9 =
-        try open_out_bin fname with
+        try Secure.open_out_bin tmp_fname with
         [ Sys_error _ ->
             raise (Adef.Request_failure "the database is not writable") ]
       in
       let patches = patches_of_patches_ht patches in
+      remove_file (fname ^ "~");
       output_value_no_sharing oc9 (patches : Old.patches);
       close_out oc9;
+      try Sys.rename fname (fname ^ "~") with [ Sys_error _ -> () ];
+      try Sys.rename tmp_fname fname with [ Sys_error _ -> () ];
     }
   in
   let patched_ascends () =
@@ -861,7 +1066,7 @@ value input bname =
   in
   let read_notes mlen =
     match
-      try Some (open_in (Filename.concat bname "notes")) with
+      try Some (Secure.open_in (Filename.concat bname "notes")) with
       [ Sys_error _ -> None ]
     with
     [ Some ic ->
@@ -885,7 +1090,7 @@ value input bname =
       try Sys.rename fname (fname ^ "~") with _ -> ();
       if s = "" then ()
       else do {
-        let oc = open_out fname in output_string oc s; close_out oc; ()
+        let oc = Secure.open_out fname in output_string oc s; close_out oc; ()
       }
     }
   in
@@ -905,11 +1110,11 @@ value input bname =
      persons_of_surname =
        persons_of_first_name_or_surname base_data strings
          (ic2, ic2_surname_start_pos, fun p -> p.surname, snd patches.h_person,
-          "surname");
+          "snames.inx", "snames.dat", bname);
      persons_of_first_name =
        persons_of_first_name_or_surname base_data strings
          (ic2, ic2_first_name_start_pos, fun p -> p.first_name,
-          snd patches.h_person, "first_name");
+          snd patches.h_person, "fnames.inx", "fnames.dat", bname);
      patch_person = patch_person; patch_ascend = patch_ascend;
      patch_union = patch_union; patch_family = patch_family;
      patch_couple = patch_couple; patch_descend = patch_descend;
@@ -932,7 +1137,10 @@ value rec prime_after n = if is_prime n then n else prime_after (n + 1);
 value output_strings_hash oc2 base =
   let strings_array = base.data.strings.array () in
   let taba =
-    Array.create (prime_after (max 2 (10 * Array.length strings_array))) (-1)
+    Array.create
+      (min Sys.max_array_length
+	 (prime_after (max 2 (10 * Array.length strings_array))))
+      (-1)
   in
   let tabl = Array.create (Array.length strings_array) (-1) in
   do {
@@ -955,7 +1163,7 @@ value output_strings_hash oc2 base =
   }
 ;
 
-value output_surname_index oc2 base =
+value output_surname_index oc2 base tmp_snames_inx tmp_snames_dat =
   let module IstrTree =
     Btree.Make
       (struct type t = istr; value compare = compare_istr_fun base.data; end)
@@ -967,11 +1175,33 @@ value output_surname_index oc2 base =
       let a = try IstrTree.find p.surname bt.val with [ Not_found -> [] ] in
       bt.val := IstrTree.add p.surname [p.cle_index :: a] bt.val
     };
+    (* obsolete table: saved by compatibility with GeneWeb versions <= 4.09,
+       i.e. the created database can be still read by these versions but this
+       table will not be used in versions >= 4.10 *)
     output_value_no_sharing oc2 (bt.val : IstrTree.t (list iper));
+    (* new table created from version >= 4.10 *)
+    let oc_sn_dat = Secure.open_out_bin tmp_snames_dat in
+    let bt2 =
+      IstrTree.map
+        (fun ipl ->
+           let i = pos_out oc_sn_dat in
+           do {
+             output_binary_int oc_sn_dat (List.length ipl);
+             List.iter
+               (fun ip -> output_binary_int oc_sn_dat (Adef.int_of_iper ip))
+               ipl;
+             i
+           })
+        bt.val
+    in
+    close_out oc_sn_dat;
+    let oc_sn_inx = Secure.open_out_bin tmp_snames_inx in
+    output_value_no_sharing oc_sn_inx (bt2 : IstrTree.t int);
+    close_out oc_sn_inx;
   }
 ;
 
-value output_first_name_index oc2 base =
+value output_first_name_index oc2 base tmp_fnames_inx tmp_fnames_dat =
   let module IstrTree =
     Btree.Make
       (struct type t = istr; value compare = compare_istr_fun base.data; end)
@@ -980,21 +1210,40 @@ value output_first_name_index oc2 base =
   do {
     for i = 0 to base.data.persons.len - 1 do {
       let p = base.data.persons.get i in
-      let a =
-        try IstrTree.find p.first_name bt.val with [ Not_found -> [] ]
-      in
+      let a = try IstrTree.find p.first_name bt.val with [ Not_found -> [] ] in
       bt.val := IstrTree.add p.first_name [p.cle_index :: a] bt.val
     };
+    (* obsolete table: saved by compatibility with GeneWeb versions <= 4.09,
+       i.e. the created database can be still read by these versions but this
+       table will not be used in versions >= 4.10 *)
     output_value_no_sharing oc2 (bt.val : IstrTree.t (list iper));
+    (* new table created from version >= 4.10 *)
+    let oc_fn_dat = Secure.open_out_bin tmp_fnames_dat in
+    let bt2 =
+      IstrTree.map
+        (fun ipl ->
+           let i = pos_out oc_fn_dat in
+           do {
+             output_binary_int oc_fn_dat (List.length ipl);
+             List.iter
+               (fun ip -> output_binary_int oc_fn_dat (Adef.int_of_iper ip))
+               ipl;
+             i
+           })
+        bt.val
+    in
+    close_out oc_fn_dat;
+    let oc_fn_inx = Secure.open_out_bin tmp_fnames_inx in
+    output_value_no_sharing oc_fn_inx (bt2 : IstrTree.t int);
+    close_out oc_fn_inx;
   }
 ;
 
-value table_size = 0x3fff;
 value make_name_index base =
   let t = Array.create table_size [| |] in
-  let a = base.data.persons.array () in
   let add_name key valu =
-    let i = Hashtbl.hash (Name.crush (Name.abbrev key)) mod Array.length t in
+    let key = Name.crush (Name.abbrev key) in
+    let i = Hashtbl.hash key mod Array.length t in
     if array_memq valu t.(i) then ()
     else t.(i) := Array.append [| valu |] t.(i)
   in
@@ -1004,7 +1253,7 @@ value make_name_index base =
     | [n :: nl] -> do { add_name n ip; add_names ip nl } ]
   in
   do {
-    for i = 0 to Array.length a - 1 do {
+    for i = 0 to base.data.persons.len - 1 do {
       let p = base.data.persons.get i in
       let first_name = p_first_name base p in
       let surname = p_surname base p in
@@ -1020,21 +1269,44 @@ value make_name_index base =
   }
 ;
 
-value create_name_index oc_inx base =
+value count_error computed found =
+  do {
+    Printf.eprintf "Count error. Computed %d. Found %d.\n" computed found;
+    flush stderr;
+    exit 2
+  }
+;
+
+value cache_of tab =
+  let c =
+    {array = fun _ -> tab; get = fun []; len = Array.length tab;
+     clear_array = fun _ -> ()}
+  in
+  do { c.get := fun i -> (c.array ()).(i); c }
+;
+
+value create_name_index oc_inx oc_inx_acc base =
   let ni = make_name_index base in
-  output_value_no_sharing oc_inx (ni : name_index_data)
+  let bpos = pos_out oc_inx in
+  do {
+    output_value_no_sharing oc_inx (ni : name_index_data);
+    let epos = output_array_access oc_inx_acc (cache_of ni) bpos in
+    if epos <> pos_out oc_inx then count_error epos (pos_out oc_inx)
+    else ()
+  }
 ;
 
 value add_name t key valu =
-  let i = Hashtbl.hash (Name.crush_lower key) mod Array.length t in
-  if array_memq valu t.(i) then () else t.(i) := Array.append [| valu |] t.(i)
+  let key = Name.crush_lower key in
+  let i = Hashtbl.hash key mod Array.length t in
+  if array_memq valu t.(i) then ()
+  else t.(i) := Array.append [| valu |] t.(i)
 ;
 
 value make_strings_of_fsname base =
   let t = Array.create table_size [| |] in
-  let a = base.data.persons.array () in
   do {
-    for i = 0 to Array.length a - 1 do {
+    for i = 0 to base.data.persons.len - 1 do {
       let p = base.data.persons.get i in
       let first_name = p_first_name base p in
       let surname = p_surname base p in
@@ -1051,16 +1323,14 @@ value make_strings_of_fsname base =
   }
 ;
 
-value create_strings_of_fsname oc_inx base =
+value create_strings_of_fsname oc_inx oc_inx_acc base =
   let t = make_strings_of_fsname base in
-  output_value_no_sharing oc_inx (t : strings_of_fsname)
-;
-
-value count_error computed found =
+  let bpos = pos_out oc_inx in
   do {
-    Printf.eprintf "Count error. Computed %d. Found %d.\n" computed found;
-    flush stderr;
-    exit 2
+    output_value_no_sharing oc_inx (t : strings_of_fsname);
+    let epos = output_array_access oc_inx_acc (cache_of t) bpos in
+    if epos <> pos_out oc_inx then count_error epos (pos_out oc_inx)
+    else ()
   }
 ;
 
@@ -1069,10 +1339,10 @@ value just_copy bname what oc oc_acc =
     Printf.eprintf "*** copying %s\n" what;
     flush stderr;
     let ic =
-      let ic = open_in_bin (Filename.concat bname "base") in
+      let ic = Secure.open_in_bin (Filename.concat bname "base") in
       do { check_magic ic; ic }
     in
-    let ic_acc = open_in_bin (Filename.concat bname "base.acc") in
+    let ic_acc = Secure.open_in_bin (Filename.concat bname "base.acc") in
     let persons_len = input_binary_int ic in
     let families_len = input_binary_int ic in
     let strings_len = input_binary_int ic in
@@ -1140,27 +1410,39 @@ value gen_output no_patches bname base =
   in
   do {
     try Unix.mkdir bname 0o755 with _ -> ();
-    let tmp_fname = Filename.concat bname "1base" in
-    let tmp_fname_acc = Filename.concat bname "1base.acc" in
-    let tmp_fname_inx = Filename.concat bname "1names.inx" in
-    let tmp_fname_gw2 = Filename.concat bname "1strings.inx" in
-    let tmp_fname_not = Filename.concat bname "1notes" in
+    let tmp_base = Filename.concat bname "1base" in
+    let tmp_base_acc = Filename.concat bname "1base.acc" in
+    let tmp_names_inx = Filename.concat bname "1names.inx" in
+    let tmp_names_acc = Filename.concat bname "1names.acc" in
+    let tmp_snames_inx = Filename.concat bname "1snames.inx" in
+    let tmp_snames_dat = Filename.concat bname "1snames.dat" in
+    let tmp_fnames_inx = Filename.concat bname "1fnames.inx" in
+    let tmp_fnames_dat = Filename.concat bname "1fnames.dat" in
+    let tmp_strings_inx = Filename.concat bname "1strings.inx" in
+    let tmp_notes = Filename.concat bname "1notes" in
     if not no_patches then
+(*
       let _ = base.data.persons.array () in
+*)
       let _ = base.data.ascends.array () in
       let _ = base.data.unions.array () in
+(*
       let _ = base.data.families.array () in
+*)
       let _ = base.data.couples.array () in
       let _ = base.data.descends.array () in
       let _ = base.data.strings.array () in ()
     else ();
-    base.func.cleanup ();
-    let oc = open_out_bin tmp_fname in
-    let oc_acc = open_out_bin tmp_fname_acc in
-    let output_array arr =
+    let oc = Secure.open_out_bin tmp_base in
+    let oc_acc = Secure.open_out_bin tmp_base_acc in
+    let output_array arrname arr =
       let bpos = pos_out oc in
       do {
-        output_value_no_sharing oc arr;
+        Printf.eprintf "*** saving %s array\n" arrname;
+        flush stderr;
+        match try Some (arr.array ()) with [ Failure _ -> None ] with
+        [ Some a -> output_value_no_sharing oc a
+	| None -> output_cache_array_no_sharing oc arr ];
         let epos = output_array_access oc_acc arr bpos in
         if epos <> pos_out oc then count_error epos (pos_out oc) else ()
       }
@@ -1181,25 +1463,25 @@ value gen_output no_patches bname base =
         output_binary_int oc 0;
         output_value_no_sharing oc base.data.bnotes.norigin_file;
         let persons_array_pos = pos_out oc in
-        if not no_patches then output_array (base.data.persons.array ())
+        if not no_patches then output_array "persons" base.data.persons
         else just_copy bname "persons" oc oc_acc;
         let ascends_array_pos = pos_out oc in
         if not no_patches then () else trace "saving ascends";
-        output_array (base.data.ascends.array ());
+        output_array "ascends" base.data.ascends;
         let unions_array_pos = pos_out oc in
-        if not no_patches then output_array (base.data.unions.array ())
+        if not no_patches then output_array "unions" base.data.unions
         else just_copy bname "unions" oc oc_acc;
         let families_array_pos = pos_out oc in
-        if not no_patches then output_array (base.data.families.array ())
+        if not no_patches then output_array "families" base.data.families
         else just_copy bname "families" oc oc_acc;
         let couples_array_pos = pos_out oc in
-        if not no_patches then output_array (base.data.couples.array ())
+        if not no_patches then output_array "couples" base.data.couples
         else just_copy bname "couples" oc oc_acc;
         let descends_array_pos = pos_out oc in
-        if not no_patches then output_array (base.data.descends.array ())
+        if not no_patches then output_array "descends" base.data.descends
         else just_copy bname "descends" oc oc_acc;
         let strings_array_pos = pos_out oc in
-        if not no_patches then output_array (base.data.strings.array ())
+        if not no_patches then output_array "strings" base.data.strings
         else just_copy bname "strings" oc oc_acc;
         seek_out oc array_start_indexes;
         output_binary_int oc persons_array_pos;
@@ -1214,13 +1496,14 @@ value gen_output no_patches bname base =
         close_out oc;
         close_out oc_acc;
         if not no_patches then
-          let oc_inx = open_out_bin tmp_fname_inx in
-          let oc2 = open_out_bin tmp_fname_gw2 in
+          let oc_inx = Secure.open_out_bin tmp_names_inx in
+          let oc_inx_acc = Secure.open_out_bin tmp_names_acc in
+          let oc2 = Secure.open_out_bin tmp_strings_inx in
           try
             do {
               trace "create name index";
               output_binary_int oc_inx 0;
-              create_name_index oc_inx base;
+              create_name_index oc_inx oc_inx_acc base;
               base.data.ascends.clear_array ();
               base.data.unions.clear_array ();
               base.data.couples.clear_array ();
@@ -1228,10 +1511,11 @@ value gen_output no_patches bname base =
               else ();
               let surname_or_first_name_pos = pos_out oc_inx in
               trace "create strings of fsname";
-              create_strings_of_fsname oc_inx base;
+              create_strings_of_fsname oc_inx oc_inx_acc base;
               seek_out oc_inx 0;
               output_binary_int oc_inx surname_or_first_name_pos;
               close_out oc_inx;
+              close_out oc_inx_acc;
               if save_mem.val then do { trace "compacting"; Gc.compact () }
               else ();
               trace "create string index";
@@ -1240,21 +1524,21 @@ value gen_output no_patches bname base =
               else ();
               let surname_pos = pos_out oc2 in
               trace "create surname index";
-              output_surname_index oc2 base;
+              output_surname_index oc2 base tmp_snames_inx tmp_snames_dat;
               if save_mem.val then do {
                 trace "compacting"; Gc.compact ()
               }
               else ();
               let first_name_pos = pos_out oc2 in
               trace "create first name index";
-              output_first_name_index oc2 base;
+	      output_first_name_index oc2 base tmp_fnames_inx tmp_fnames_dat;
               seek_out oc2 int_size;
               output_binary_int oc2 surname_pos;
               output_binary_int oc2 first_name_pos;
               let s = base.data.bnotes.nread 0 in
               if s = "" then ()
               else do {
-                let oc_not = open_out tmp_fname_not in
+                let oc_not = Secure.open_out tmp_notes in
                 output_string oc_not s;
                 close_out oc_not;
               };
@@ -1263,6 +1547,7 @@ value gen_output no_patches bname base =
           with e ->
             do {
               try close_out oc_inx with _ -> ();
+              try close_out oc_inx_acc with _ -> ();
               try close_out oc2 with _ -> ();
               raise e
             }
@@ -1273,26 +1558,39 @@ value gen_output no_patches bname base =
       do {
         try close_out oc with _ -> ();
         try close_out oc_acc with _ -> ();
-        remove_file tmp_fname;
-        remove_file tmp_fname_acc;
+        remove_file tmp_base;
+        remove_file tmp_base_acc;
         if not no_patches then do {
-          remove_file tmp_fname_inx; remove_file tmp_fname_gw2; ()
+          remove_file tmp_names_inx;
+          remove_file tmp_names_acc;
+          remove_file tmp_strings_inx;
         }
         else ();
         raise e
       };
+    base.func.cleanup ();
     remove_file (Filename.concat bname "base");
-    Sys.rename tmp_fname (Filename.concat bname "base");
+    Sys.rename tmp_base (Filename.concat bname "base");
     remove_file (Filename.concat bname "base.acc");
-    Sys.rename tmp_fname_acc (Filename.concat bname "base.acc");
+    Sys.rename tmp_base_acc (Filename.concat bname "base.acc");
     if not no_patches then do {
       remove_file (Filename.concat bname "names.inx");
-      Sys.rename tmp_fname_inx (Filename.concat bname "names.inx");
+      Sys.rename tmp_names_inx (Filename.concat bname "names.inx");
+      remove_file (Filename.concat bname "names.acc");
+      Sys.rename tmp_names_acc (Filename.concat bname "names.acc");
+      remove_file (Filename.concat bname "snames.dat");
+      Sys.rename tmp_snames_dat (Filename.concat bname "snames.dat");
+      remove_file (Filename.concat bname "snames.inx");
+      Sys.rename tmp_snames_inx (Filename.concat bname "snames.inx");
+      remove_file (Filename.concat bname "fnames.dat");
+      Sys.rename tmp_fnames_dat (Filename.concat bname "fnames.dat");
+      remove_file (Filename.concat bname "fnames.inx");
+      Sys.rename tmp_fnames_inx (Filename.concat bname "fnames.inx");
       remove_file (Filename.concat bname "strings.inx");
-      Sys.rename tmp_fname_gw2 (Filename.concat bname "strings.inx");
+      Sys.rename tmp_strings_inx (Filename.concat bname "strings.inx");
       remove_file (Filename.concat bname "notes");
-      if Sys.file_exists tmp_fname_not then
-        Sys.rename tmp_fname_not (Filename.concat bname "notes")
+      if Sys.file_exists tmp_notes then
+        Sys.rename tmp_notes (Filename.concat bname "notes")
       else ();
       remove_file (Filename.concat bname "patches");
       remove_file (Filename.concat bname "patches~");
