@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo ./pa_lock.cmo *)
-(* $Id: mergeInd.ml,v 4.21 2002/06/03 13:29:27 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: mergeInd.ml,v 4.26 2004/12/14 09:30:14 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
@@ -266,7 +266,7 @@ value propose_merge_ind conf base branches p1 p2 =
 value reparent_ind base p1 p2 =
   let a1 = aoi base p1.cle_index in
   let a2 = aoi base p2.cle_index in
-  match (a1.parents, a2.parents) with
+  match (parents a1, parents a2) with
   [ (None, Some ifam) ->
       let des = doi base ifam in
       do {
@@ -276,8 +276,8 @@ value reparent_ind base p1 p2 =
           else replace (i + 1)
         in
         replace 0;
-        a1.parents := Some ifam;
-        a1.consang := Adef.fix (-1);
+        set_parents a1 (Some ifam);
+        set_consang a1 (Adef.fix (-1));
         base.func.patch_ascend p1.cle_index a1;
         base.func.patch_descend ifam des;
       }
@@ -292,8 +292,8 @@ value effective_merge_ind conf base p1 p2 =
       for i = 0 to Array.length u2.family - 1 do {
         let ifam = u2.family.(i) in
         let cpl = coi base ifam in
-        if p2.cle_index = cpl.father then cpl.father := p1.cle_index
-        else if p2.cle_index = cpl.mother then cpl.mother := p1.cle_index
+        if p2.cle_index = (father cpl) then set_father cpl p1.cle_index
+        else if p2.cle_index = (mother cpl) then set_mother cpl p1.cle_index
         else assert False;
         base.func.patch_couple ifam cpl;
       };
@@ -401,7 +401,7 @@ value effective_merge_fam conf base fam1 fam2 p1 p2 =
     for i = 0 to Array.length des2.children - 1 do {
       let ip = des2.children.(i) in
       let a = aoi base ip in
-      a.parents := Some fam1.fam_index;
+      set_parents a (Some fam1.fam_index);
       base.func.patch_ascend ip a;
     };
     des2.children := [| |];
@@ -458,30 +458,30 @@ value rec try_merge conf base branches ip1 ip2 changes_done =
   let a2 = aoi base ip2 in
   let ok_so_far = True in
   let (ok_so_far, changes_done) =
-    match (a1.parents, a2.parents) with
+    match (parents a1, parents a2) with
     [ (Some ifam1, Some ifam2) when ifam1 <> ifam2 ->
         let branches = [(ip1, ip2) :: branches] in
         let cpl1 = coi base ifam1 in
         let cpl2 = coi base ifam2 in
         let (ok_so_far, changes_done) =
           if ok_so_far then
-            if cpl1.father = cpl2.father then (True, changes_done)
+            if (father cpl1) = (father cpl2) then (True, changes_done)
             else
-              try_merge conf base branches cpl1.father cpl2.father
+              try_merge conf base branches (father cpl1) (father cpl2)
                 changes_done
           else (False, changes_done)
         in
         let (ok_so_far, changes_done) =
           if ok_so_far then
-            if cpl1.mother = cpl2.mother then (True, changes_done)
+            if (mother cpl1) = (mother cpl2) then (True, changes_done)
             else
-              try_merge conf base branches cpl1.mother cpl2.mother
+              try_merge conf base branches (mother cpl1) (mother cpl2)
                 changes_done
           else (False, changes_done)
         in
         let (ok_so_far, changes_done) =
           if ok_so_far then
-            merge_fam conf base branches ifam1 ifam2 cpl1.father cpl1.mother
+            merge_fam conf base branches ifam1 ifam2 (father cpl1) (mother cpl1)
               changes_done
           else (False, changes_done)
         in
@@ -510,9 +510,9 @@ value is_ancestor base ip1 ip2 =
     else if ip = ip1 then True
     else do {
       visited.(Adef.int_of_iper ip) := True;
-      match (aoi base ip).parents with
+      match parents (aoi base ip) with
       [ Some ifam ->
-          let cpl = coi base ifam in loop cpl.father || loop cpl.mother
+          let cpl = coi base ifam in loop (father cpl) || loop (mother cpl)
       | None -> False ]
     }
   in
@@ -563,9 +563,11 @@ value print conf base =
       else if is_ancestor base p2.cle_index p1.cle_index then
         error_loop conf base p1
       else
+(*
         let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
         lock (Iobase.lock_file bfile) with
         [ Accept ->
+*)
             let (ok, changes_done) =
               try_merge conf base [] p1.cle_index p2.cle_index False
             in
@@ -580,7 +582,9 @@ value print conf base =
               }
               else ();
             }
+(*
         | Refuse -> Update.error_locked conf base ]
+*)
   | _ -> not_found_or_incorrect conf ]
 ;
 
@@ -588,12 +592,12 @@ value print conf base =
 
 value rec kill_ancestors conf base included_self p nb_ind nb_fam =
   do {
-    match (aoi base p.cle_index).parents with
+    match parents (aoi base p.cle_index) with
     [ Some ifam ->
         let cpl = coi base ifam in
         do {
-          kill_ancestors conf base True (poi base cpl.father) nb_ind nb_fam;
-          kill_ancestors conf base True (poi base cpl.mother) nb_ind nb_fam;
+          kill_ancestors conf base True (poi base (father cpl)) nb_ind nb_fam;
+          kill_ancestors conf base True (poi base (mother cpl)) nb_ind nb_fam;
           UpdateFamOk.effective_del conf base (foi base ifam);
           incr nb_fam;
         }
@@ -626,17 +630,14 @@ value print_kill_ancestors conf base =
       [ Some p ->
           let key = (sou base p.first_name, sou base p.surname, p.occ) in
           let bfile = Util.base_path [] (conf.bname ^ ".lck") in
-          lock (Iobase.lock_file bfile) with
-          [ Accept ->
-              let nb_ind = ref 0 in
-              let nb_fam = ref 0 in
-              do {
-                kill_ancestors conf base False p nb_ind nb_fam;
-                Util.commit_patches conf base;
-                History.record conf base key "ka";
-                print_killed conf base p nb_ind.val nb_fam.val;
-              }
-          | Refuse -> Update.error_locked conf base ]
+          let nb_ind = ref 0 in
+          let nb_fam = ref 0 in
+          do {
+            kill_ancestors conf base False p nb_ind nb_fam;
+            Util.commit_patches conf base;
+            History.record conf base key "ka";
+            print_killed conf base p nb_ind.val nb_fam.val;
+          }
       | None -> incorrect_request conf ]
   | _ -> incorrect_request conf ]
 ;
