@@ -2,14 +2,14 @@
 #cd (*
 exec ocaml camlp4r.cma $0
 *) ".";
-(* $Id: mk_missing_i18n.sh,v 4.9 2002/11/26 08:39:40 ddr Exp $ *)
+(* $Id: mk_missing_i18n.sh,v 4.12 2004/01/29 16:34:17 ddr Exp $ *)
 
 open Printf;
 
 value languages =
-  ["af"; "bg"; "ca"; "cs"; "da"; "de"; "en"; "es"; "eo"; "et"; "fi"; "fr";
-   "he"; "is"; "it"; "lv"; "nl"; "no"; "pl"; "pt"; "pt-br"; "ro"; "ru"; "sl";
-   "sv"; "zh"]
+  ["af"; "bg"; "br"; "ca"; "cs"; "da"; "de"; "en"; "eo"; "es"; "et"; "fi";
+   "fr"; "he"; "is"; "it"; "lv"; "nl"; "no"; "pl"; "pt"; "pt-br"; "ro"; "ru";
+   "sl"; "sv"; "zh"]
 ;
 
 value linenum = ref 0;
@@ -55,8 +55,8 @@ value check first lang =
   in
   do {
     linenum.val := 0;
-    let ic_lex = open_in "../hd/lang/lexicon.txt" in
-    let ic_i18n = open_in "i18n" in
+    let ic_lex = open_in "hd/lang/lexicon.txt" in
+    let ic_i18n = open_in "src/i18n" in
     printf "<h3><a name=%s>%s</h3>\n" lang lang;
     printf "<pre>\n";
     let has_missing = ref False in
