@@ -1,5 +1,5 @@
-(* $Id: mostdesc.ml,v 4.3 2002/03/09 16:48:49 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: mostdesc.ml,v 4.6 2004/12/14 09:30:14 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Gutil;
 open Def;
@@ -63,7 +63,9 @@ value most_desc base p =
          value leq x y = id.(Adef.int_of_iper x) > id.(Adef.int_of_iper y);
        end)
   in
+(*
   let _ = base.data.persons.array () in
+*)
   let _ = base.data.descends.array () in
   let _ = base.data.unions.array () in
   let tab = Array.create base.data.persons.len Num.zero in
@@ -125,6 +127,7 @@ value main () =
       exit 2
     }
     else ();
+    Secure.set_base_dir (Filename.dirname bname.val);
     let base = Iobase.input bname.val in
     let ip =
       Gutil.person_ht_find_unique base p_fname.val p_sname.val p_num.val
