@@ -1,6 +1,6 @@
 (* camlp4r *)
-(* $Id: perso.ml,v 4.45 2002/12/26 14:26:22 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: perso.ml,v 4.60 2004/12/14 09:30:15 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -182,13 +182,13 @@ value find_sosa_aux conf base a p =
             gene_find zil
           else
             let asc = aget conf base ip in
-            match asc.parents with
+            match parents asc with
             [ Some ifam ->
                 let cpl = coi base ifam in
                 let z = Num.twice z in
                 match gene_find zil with
                 [ Left zil ->
-                    Left [(z, cpl.father); (Num.inc z 1, cpl.mother) :: zil]
+                    Left [(z, father cpl); (Num.inc z 1, (mother cpl)) :: zil]
                 | Right z -> Right z ]
             | None -> gene_find zil ]
         } ]
@@ -214,9 +214,9 @@ value find_sosa_aux conf base a p =
       [ Some ifam ->
           let cpl = coi base ifam in
           let z = Num.twice z in
-          match find z cpl.father with
+          match find z (father cpl) with
           [ Some z -> Some z
-          | None -> find (Num.inc z 1) cpl.mother ]
+          | None -> find (Num.inc z 1) (mother cpl) ]
       | None -> None ]
     }
   in
@@ -225,7 +225,7 @@ value find_sosa_aux conf base a p =
 *)
 
 value find_sosa conf base a =
-  match Util.find_person_in_env conf base "z" with
+  match Util.find_sosa_ref conf base with
   [ Some p ->
       if a.cle_index = p.cle_index then Some (Num.one, p)
       else
@@ -292,23 +292,28 @@ value rec eval_variable conf base env sl =
               [ Vbool True -> authorized_age conf base p
               | _ -> False ]
             in
-            let ep = (p, a, u, auth) in loop ep efam sl
+            let ep = (p, a, u, auth) in loop ep Vnone sl
+        | _ -> VVnone ]
+    | ["parent" :: sl] ->
+        match get_env "parent" env with
+        [ Vind p a u ->
+            let ep = (p, a, u, authorized_age conf base p ) in loop ep efam sl
         | _ -> VVnone ]
     | ["father" :: sl] ->
-        match a.parents with
+        match parents a with
         [ Some ifam ->
             let cpl = coi base ifam in
-            let ep = make_ep cpl.father in
-            let cpl = (cpl.father, cpl.mother) in
+            let ep = make_ep (father cpl) in
+            let cpl = ((father cpl), (mother cpl)) in
             let efam = Vfam (foi base ifam) cpl (doi base ifam) in
             loop ep efam sl
         | None -> VVnone ]
     | ["mother" :: sl] ->
-        match a.parents with
+        match parents a with
         [ Some ifam ->
             let cpl = coi base ifam in
-            let ep = make_ep cpl.mother in
-            let cpl = (cpl.mother, cpl.father) in
+            let ep = make_ep (mother cpl) in
+            let cpl = ((mother cpl), (father cpl)) in
             let efam = Vfam (foi base ifam) cpl (doi base ifam) in
             loop ep efam sl
         | None -> VVnone ]
@@ -408,7 +413,7 @@ value string_of_comment conf base env p p_auth =
 value string_of_consanguinity conf base env a p_auth =
   if p_auth then
     string_of_decimal_num conf
-      (round_2_dec (Adef.float_of_fix a.consang *. 100.0)) ^
+      (round_2_dec (Adef.float_of_fix (consang a) *. 100.0)) ^
     "%"
   else ""
 ;
@@ -492,7 +497,7 @@ value string_of_image_size conf base env p p_auth =
   else ""
 ;
 
-value string_of_image_url conf base env p p_auth =
+value string_of_image_url conf base env p p_auth html =
   if p_auth then
     match get_env "image" env with
     [ Vimage x ->
@@ -501,7 +506,8 @@ value string_of_image_url conf base env p p_auth =
             let s = Unix.stat fname in
             let b = acces conf base p in
             let k = default_image_name base p in
-            Format.sprintf "%sm=IM;d=%d;%s;k=/%s" (commd conf)
+            Format.sprintf "%sm=IM%s;d=%d;%s;k=/%s" (commd conf)
+              (if html then "H" else "")
               (int_of_float
                  (mod_float s.Unix.st_mtime (float_of_int max_int)))
               b k
@@ -637,7 +643,7 @@ value string_of_origin_file conf base env =
 ;
 
 value string_of_parent_age conf base p a p_auth parent =
-  match a.parents with
+  match parents a with
   [ Some ifam ->
       let cpl = coi base ifam in
       let pp = pget conf base (parent cpl) in
@@ -656,7 +662,9 @@ value string_of_prefix_no_templ conf base env =
       (fun (k, v) henv -> if k = "templ" then henv else [(k, v) :: henv])
       conf.henv []
   in
-  do { conf.henv := henv; commd conf }
+  let c = conf.command ^ "?" in
+  List.fold_left (fun c (k, v) -> c ^ k ^ "=" ^ v ^ ";") c
+    (henv @ conf.senv)
 ;
 
 value string_of_qualifier conf base env p p_auth =
@@ -771,10 +779,10 @@ value try_eval_gen_variable conf base env (p, a, u, p_auth) efam =
   | "child_cnt" -> eval_int_env "child_cnt" env
   | "child_name" ->
       let force_surname =
-        match a.parents with
+        match parents a with
         [ None -> False
         | Some ifam ->
-            p_surname base (pget conf base (coi base ifam).father) <>
+            p_surname base (pget conf base (father (coi base ifam))) <>
               p_surname base p ]
       in
       if not p_auth && conf.hide_names then "x x"
@@ -800,16 +808,17 @@ value try_eval_gen_variable conf base env (p, a, u, p_auth) efam =
       | _ -> "" ]
   | "family_cnt" -> eval_int_env "family_cnt" env
   | "father_age_at_birth" ->
-      string_of_parent_age conf base p a p_auth (fun cpl -> cpl.father)
+      string_of_parent_age conf base p a p_auth (fun cpl -> (father cpl))
   | "first_name" ->
       if not p_auth && conf.hide_names then "x" else p_first_name base p
   | "first_name_alias" -> string_of_first_name_alias conf base env
   | "first_name_key" ->
       if conf.hide_names && not p_auth then ""
       else code_varenv (Name.lower (p_first_name base p))
+  | "image_html_url" -> string_of_image_url conf base env p p_auth True
   | "image_txt" -> default_image_name base p
   | "image_size" -> string_of_image_size conf base env p p_auth
-  | "image_url" -> string_of_image_url conf base env p p_auth
+  | "image_url" -> string_of_image_url conf base env p p_auth False
   | "ind_access" -> "i=" ^ string_of_int (Adef.int_of_iper p.cle_index)
   | "key" ->
       if not p_auth && conf.hide_names then "x"
@@ -830,7 +839,7 @@ value try_eval_gen_variable conf base env (p, a, u, p_auth) efam =
       string_of_married_to conf base env p p_auth efam
   | "misc_names" -> string_of_misc_names conf base env p p_auth
   | "mother_age_at_birth" ->
-      string_of_parent_age conf base p a p_auth (fun cpl -> cpl.mother)
+      string_of_parent_age conf base p a p_auth (fun cpl -> (mother cpl))
   | "nb_children" ->
       match get_env "fam" env with
       [ Vfam _ _ des -> string_of_int (Array.length des.children)
@@ -923,29 +932,28 @@ value eval_simple_bool_variable conf base env (p, a, u, p_auth) efam =
       | _ -> False ]
   | "are_engaged" ->
       match efam with
-      [ Vfam fam _ _ ->
-          match fam.relation with
-          [ Engaged -> True
-          | _ -> False ]
+      [ Vfam fam _ _ -> fam.relation = Engaged
       | _ -> False ]
   | "are_married" ->
       match efam with
-      [ Vfam fam _ _ ->
-          match fam.relation with
-          [ Married | NoSexesCheck -> True
-          | _ -> False ]
+      [ Vfam fam _ _ -> fam.relation = Married
       | _ -> False ]
   | "are_not_married" ->
       match efam with
-      [ Vfam fam _ _ ->
-          match fam.relation with
-          [ NotMarried -> True
-          | _ -> False ]
+      [ Vfam fam _ _ -> fam.relation = NotMarried
       | _ -> False ]
   | "are_separated" ->
       match efam with
       [ Vfam fam cpl _ -> fam.divorce = Separated
       | _ -> False ]
+  | "is_no_sexes_check" ->
+      match efam with
+      [ Vfam fam _ _ -> fam.relation = NoSexesCheck
+      | _ -> False ]
+  | "is_no_mention" ->
+      match efam with
+      [ Vfam fam _ _ -> fam.relation = NoMention
+      | _ -> False ]
   | "birthday" ->
       if p_auth then
         match Adef.od_of_codate p.birth with
@@ -1008,7 +1016,7 @@ value eval_simple_bool_variable conf base env (p, a, u, p_auth) efam =
       [ Vfam fam _ _ -> p_auth && sou base fam.comment <> ""
       | _ -> False ]
   | "has_consanguinity" ->
-      p_auth && a.consang != Adef.fix (-1) && a.consang != Adef.fix 0
+      p_auth && consang a != Adef.fix (-1) && consang a != Adef.fix 0
   | "has_cremation_place" -> p_auth && sou base p.burial_place <> ""
   | "has_death_date" ->
       match p.death with
@@ -1022,7 +1030,7 @@ value eval_simple_bool_variable conf base env (p, a, u, p_auth) efam =
   | "has_nobility_titles" -> p_auth && p.titles <> []
   | "has_notes" -> p_auth && sou base p.notes <> ""
   | "has_occupation" -> p_auth && sou base p.occupation <> ""
-  | "has_parents" -> a.parents <> None
+  | "has_parents" -> parents a <> None
   | "has_public_name" -> sou base p.public_name <> ""
   | "has_qualifiers" -> p.qualifiers <> []
   | "has_referer" -> Wserver.extract_param "referer: " '\n' conf.request <> ""
@@ -1046,7 +1054,7 @@ value eval_simple_bool_variable conf base env (p, a, u, p_auth) efam =
         (p.rparents <> [] || related <> [])
       else p_auth && (p.rparents <> [] || p.related <> [])
   | "has_siblings" ->
-      match a.parents with
+      match parents a with
       [ Some ifam -> Array.length (doi base ifam).children > 1
       | None -> False ]
   | "has_sosa" ->
@@ -1098,6 +1106,7 @@ value eval_simple_bool_variable conf base env (p, a, u, p_auth) efam =
   | "is_restricted" -> is_hidden p
   | "is_self" -> get_env "pos" env = Vstring "self"
   | "wizard" -> conf.wizard
+  | "friend" -> conf.friend
   | v ->
       let v = extract_var "file_exists_" v in
       if v <> "" then
@@ -1205,18 +1214,18 @@ value eval_expr conf base env p =
   | _ -> ">parse_error" ]
 ;
 
-value rec eval_ast conf base env =
+value rec print_ast conf base env =
   fun
   [ Atext s -> Wserver.wprint "%s" s
   | Atransl upp s n -> print_transl conf base env upp s n
   | Avar s sl -> print_variable conf base env [s :: sl]
   | Awid_hei s -> print_wid_hei conf base env s
-  | Aif e alt ale -> eval_if conf base env e alt ale
-  | Aforeach s sl al -> eval_foreach conf base env s sl al
+  | Aif e alt ale -> print_if conf base env e alt ale
+  | Aforeach s sl al -> print_foreach conf base env s sl al
   | Adefine f xl al alk -> print_define conf base env f xl al alk
   | Aapply f el -> print_apply conf base env f el ]
 and print_define conf base env f xl al alk =
-  List.iter (eval_ast conf base [(f, Vfun xl al) :: env]) alk
+  List.iter (print_ast conf base [(f, Vfun xl al) :: env]) alk
 and print_apply conf base env f el =
   match get_env f env with
   [ Vfun xl al ->
@@ -1236,19 +1245,19 @@ and print_apply conf base env f el =
                | ([], []) -> a
                | _ -> Atext "parse_error" ]
            in
-           eval_ast conf base env a)
+           print_ast conf base env a)
         al
   | _ -> Wserver.wprint ">%%%s???" f ]
-and eval_if conf base env e alt ale =
+and print_if conf base env e alt ale =
   let al = if eval_bool_value conf base env e then alt else ale in
-  List.iter (eval_ast conf base env) al
-and eval_foreach conf base env s sl al =
+  List.iter (print_ast conf base env) al
+and print_foreach conf base env s sl al =
   let (sl, s) =
     let sl = List.rev [s :: sl] in (List.rev (List.tl sl), List.hd sl)
   in
   match eval_variable conf base env sl with
   [ VVsome (env, ep, efam, "") ->
-      eval_simple_foreach conf base env al ep efam s
+      print_simple_foreach conf base env al ep efam s
   | VVsome (env, ep, efam, _) ->
       do {
         Wserver.wprint "foreach ";
@@ -1257,28 +1266,29 @@ and eval_foreach conf base env s sl al =
       }
  | VVcvar s -> Wserver.wprint ">%%%s???" s
  | VVnone -> () ]
-and eval_simple_foreach conf base env al ep efam =
+and print_simple_foreach conf base env al ep efam =
   fun
-  [ "alias" -> eval_foreach_alias conf base env al ep
-  | "child" -> eval_foreach_child conf base env al efam
-  | "family" -> eval_foreach_family conf base env al ep
-  | "first_name_alias" -> eval_foreach_first_name_alias conf base env al ep
-  | "nobility_title" -> eval_foreach_nobility_title conf base env al ep
-  | "qualifier" -> eval_foreach_qualifier conf base env al ep
-  | "related" -> eval_foreach_related conf base env al ep
-  | "relation" -> eval_foreach_relation conf base env al ep
-  | "source" -> eval_foreach_source conf base env al ep
-  | "surname_alias" -> eval_foreach_surname_alias conf base env al ep
-  | "witness" -> eval_foreach_witness conf base env al efam
-  | "witness_relation" -> eval_foreach_witness_relation conf base env al ep
+  [ "alias" -> print_foreach_alias conf base env al ep
+  | "child" -> print_foreach_child conf base env al efam
+  | "family" -> print_foreach_family conf base env al ep
+  | "first_name_alias" -> print_foreach_first_name_alias conf base env al ep
+  | "nobility_title" -> print_foreach_nobility_title conf base env al ep
+  | "parent" -> print_foreach_parent conf base env al ep
+  | "qualifier" -> print_foreach_qualifier conf base env al ep
+  | "related" -> print_foreach_related conf base env al ep
+  | "relation" -> print_foreach_relation conf base env al ep
+  | "source" -> print_foreach_source conf base env al ep
+  | "surname_alias" -> print_foreach_surname_alias conf base env al ep
+  | "witness" -> print_foreach_witness conf base env al efam
+  | "witness_relation" -> print_foreach_witness_relation conf base env al ep
   | s -> Wserver.wprint "foreach %s???" s ]
-and eval_foreach_alias conf base env al (p, _, _, p_auth) =
+and print_foreach_alias conf base env al (p, _, _, p_auth) =
   List.iter
     (fun a ->
        let env = [("alias", Vstring (sou base a)) :: env] in
-       List.iter (eval_ast conf base env) al)
+       List.iter (print_ast conf base env) al)
     p.aliases
-and eval_foreach_child conf base env al =
+and print_foreach_child conf base env al =
   fun
   [ Vfam _ _ des ->
       let auth =
@@ -1315,10 +1325,10 @@ and eval_foreach_child conf base env al =
                [("pos", Vstring "next") :: env]
              else env
            in
-           List.iter (eval_ast conf base env) al)
+           List.iter (print_ast conf base env) al)
         des.children
   | _ -> () ]
-and eval_foreach_family conf base env al (p, _, u, _) =
+and print_foreach_family conf base env al (p, _, u, _) =
   Array.iteri
     (fun i ifam ->
        let fam = foi base ifam in
@@ -1328,42 +1338,55 @@ and eval_foreach_family conf base env al (p, _, u, _) =
        let env = [("#loop", Vint 0) :: env] in
        let env = [("fam", Vfam fam cpl des) :: env] in
        let env = [("family_cnt", Vint (i + 1)) :: env] in
-       List.iter (eval_ast conf base env) al)
+       List.iter (print_ast conf base env) al)
     u.family
-and eval_foreach_first_name_alias conf base env al (p, _, _, p_auth) =
+and print_foreach_first_name_alias conf base env al (p, _, _, p_auth) =
   if p_auth then
     List.iter
       (fun s ->
          let env = [("first_name_alias", Vstring (sou base s)) :: env] in
-         List.iter (eval_ast conf base env) al)
+         List.iter (print_ast conf base env) al)
       p.first_names_aliases
   else ()
-and eval_foreach_nobility_title conf base env al (p, _, _, p_auth) =
+and print_foreach_nobility_title conf base env al (p, _, _, p_auth) =
   if p_auth then
     let titles = nobility_titles_list conf p in
     list_iter_first
       (fun first x ->
          let env = [("nobility_title", Vtitle x) :: env] in
          let env = [("first", Vbool first) :: env] in
-         List.iter (eval_ast conf base env) al)
+         List.iter (print_ast conf base env) al)
       titles
   else ()
-and eval_foreach_qualifier conf base env al (p, _, _, _) =
+and print_foreach_parent conf base env al (_, a, _, _) =
+  match parents a with
+  [ Some ifam ->
+      let cpl = coi base ifam in
+      Array.iter
+        (fun iper ->
+	   let p = pget conf base iper in
+	   let a = aget conf base iper in
+	   let u = uget conf base iper in
+	   let env = [("parent", Vind p a u) :: env] in
+           List.iter (print_ast conf base env) al)
+        (parent_array cpl)
+  | None -> () ]
+and print_foreach_qualifier conf base env al (p, _, _, _) =
   list_iter_first
     (fun first nn ->
        let env = [("qualifier", Vstring (sou base nn)) :: env] in
        let env = [("first", Vbool first) :: env] in
-       List.iter (eval_ast conf base env) al)
+       List.iter (print_ast conf base env) al)
     p.qualifiers
-and eval_foreach_relation conf base env al (p, _, _, p_auth) =
+and print_foreach_relation conf base env al (p, _, _, p_auth) =
   if p_auth then
     List.iter
       (fun r ->
          let env = [("rel", Vrel r) :: env] in
-         List.iter (eval_ast conf base env) al)
+         List.iter (print_ast conf base env) al)
       p.rparents
   else ()
-and eval_foreach_related conf base env al (p, _, _, p_auth) =
+and print_foreach_related conf base env al (p, _, _, p_auth) =
   if p_auth then
     let list =
       List.fold_left
@@ -1405,10 +1428,10 @@ and eval_foreach_related conf base env al (p, _, _, p_auth) =
          let a = aget conf base c.cle_index in
          let u = uget conf base c.cle_index in
          let env = [("c", Vind c a u); ("rel", Vrel r) :: env] in
-         List.iter (eval_ast conf base env) al)
+         List.iter (print_ast conf base env) al)
       list
   else ()
-and eval_foreach_source conf base env al (p, _, u, p_auth) =
+and print_foreach_source conf base env al (p, _, u, p_auth) =
   let rec insert_loop typ src =
     fun
     [ [(typ1, src1) :: srcl] ->
@@ -1453,16 +1476,16 @@ and eval_foreach_source conf base env al (p, _, u, p_auth) =
     if s = "" then ()
     else
       let env = [("src_typ", Vstring src_typ); ("src", Vstring s) :: env] in
-      List.iter (eval_ast conf base env) al
+      List.iter (print_ast conf base env) al
   in
   List.iter print_src srcl
-and eval_foreach_surname_alias conf base env al (p, _, _, _) =
+and print_foreach_surname_alias conf base env al (p, _, _, _) =
   List.iter
     (fun s ->
        let env = [("surname_alias", Vstring (sou base s)) :: env] in
-       List.iter (eval_ast conf base env) al)
+       List.iter (print_ast conf base env) al)
     p.surnames_aliases
-and eval_foreach_witness conf base env al =
+and print_foreach_witness conf base env al =
   fun
   [ Vfam fam _ _ ->
       list_iter_first
@@ -1472,27 +1495,54 @@ and eval_foreach_witness conf base env al =
            let u = uget conf base ip in
            let env = [("witness", Vind p a u) :: env] in
            let env = [("first", Vbool first) :: env] in
-           List.iter (eval_ast conf base env) al)
+           List.iter (print_ast conf base env) al)
         (Array.to_list fam.witnesses)
   | _ -> () ]
-and eval_foreach_witness_relation conf base env al (p, _, _, _) =
+and print_foreach_witness_relation conf base env al (p, _, _, _) =
+  let list =
+    let list = ref [] in
+    do {
+      make_list p.related where rec make_list =
+        fun
+        [ [ic :: icl] ->
+            do {
+              let c = pget conf base ic in
+              if c.sex = Male then
+                Array.iter
+                  (fun ifam ->
+                     let fam = foi base ifam in
+                     if array_memq p.cle_index fam.witnesses then
+                       list.val := [(ifam, fam) :: list.val]
+                     else ())
+                  (uget conf base ic).family
+              else ();
+              make_list icl
+            }
+        | [] -> () ];
+      list.val
+    }
+  in
+  let list =
+    List.sort
+      (fun (_, fam1) (_, fam2) ->
+	 match
+	   (Adef.od_of_codate fam1.marriage, Adef.od_of_codate fam2.marriage)
+	 with
+	 [ (Some d1, Some d2) ->
+	     if strictly_before d1 d2 then -1
+	     else if strictly_before d2 d1 then 1
+	     else 0
+	 | _ -> 0 ])
+      list
+  in
   List.iter
-    (fun ic ->
-       let c = pget conf base ic in
-       if c.sex = Male then
-         Array.iter
-           (fun ifam ->
-              let fam = foi base ifam in
-              if array_memq p.cle_index fam.witnesses then
-                let cpl = coi base ifam in
-                let des = doi base ifam in
-                let cpl = (cpl.father, cpl.mother) in
-                let env = [("fam", Vfam fam cpl des) :: env] in
-                List.iter (eval_ast conf base env) al
-              else ())
-           (uget conf base ic).family
-       else ())
-    p.related
+    (fun (ifam, fam) ->
+       let cpl = coi base ifam in
+       let des = doi base ifam in
+       let cpl = ((father cpl), (mother cpl)) in
+       let env = [("fam", Vfam fam cpl des) :: env] in
+       List.iter (print_ast conf base env) al)
+    list
 ;
 
 value interp_templ conf base p astl =
@@ -1512,7 +1562,7 @@ value interp_templ conf base p astl =
     let env = [("p_auth", Vbool (authorized_age conf base p)) :: env] in
     let env = [("p", Vind p a u) :: env] in env
   in
-  List.iter (eval_ast conf base env) astl
+  List.iter (print_ast conf base env) astl
 ;
 
 (* Main *)
@@ -1531,7 +1581,7 @@ value print conf base p =
     if conf.wizard || conf.friend then None
     else
       let src =
-        match (aget conf base p.cle_index).parents with
+        match parents (aget conf base p.cle_index) with
         [ Some ifam -> sou base (foi base ifam).origin_file
         | None -> "" ]
       in
