@@ -1,6 +1,6 @@
 (* camlp4r q_MLast.cmo *)
-(* $Id: pr_transl.ml,v 4.6 2002/12/01 12:28:14 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: pr_transl.ml,v 4.10 2004/12/14 09:30:16 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open MLast;
 
@@ -21,12 +21,21 @@ value trace =
 value rec expr e =
   match e with
   [ <:expr< $lid:f$ $_$ $str:s$ >> when List.mem f trace ->
-      Printf.printf "%s\n" (Token.eval_string s)
+(*
+      let loc = Token.dummy_loc in
+*)
+      Printf.printf "%s\n" (Token.eval_string (*loc*) s)
   | <:expr< Util.$lid:f$ $_$ $str:s$ >> when List.mem f trace ->
-      Printf.printf "%s\n" (Token.eval_string s)
+(*
+      let loc = Token.dummy_loc in
+*)
+      Printf.printf "%s\n" (Token.eval_string (*loc*) s)
   | <:expr< $lid:f$ $_$ ($lid:g$ $_$ $str:s$ $_$) >>
     when List.mem f trace && List.mem g trace ->
-      Printf.printf "%s\n" (Token.eval_string s)
+(*
+      let loc = Token.dummy_loc in
+*)
+      Printf.printf "%s\n" (Token.eval_string (*loc*) s)
   | <:expr< $lid:x$ >> when List.mem x trace ->
       Stdpp.raise_with_loc (MLast.loc_of_expr e) (Failure "Bad source")
   | <:expr< let $opt:_$ $list:pel$ in $e$ >> ->
@@ -59,6 +68,7 @@ value rec expr e =
   | <:expr< $flo:_$ >> -> ()
   | <:expr< $chr:_$ >> -> ()
   | <:expr< $x$.$y$ >> -> ()
+  | <:expr< assert False >> -> ()
   | x -> not_impl "expr" x ]
 and binding_list pel = List.iter binding pel
 and binding (p, e) =
