@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: relation.ml,v 4.35 2002/11/28 14:10:54 ddr Exp $ *)
-(* Copyright (c) 2002 INRIA *)
+(* $Id: relation.ml,v 4.50 2004/12/14 09:30:17 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -94,7 +94,7 @@ value print_menu conf base p =
             (transl_nth conf "surname/surnames" 0) (transl conf "or")
             (transl conf "public name") (transl conf "or")
             (nominative (transl conf "alias"));
-          match Util.find_person_in_env conf base "z" with
+          match Util.find_sosa_ref conf base with
           [ Some p ->
               do {
                 Wserver.wprint "%s " (transl conf "or");
@@ -145,20 +145,52 @@ value print_menu conf base p =
       end;
       html_p conf;
       tag "table" "border=%d width=\"90%%\"" conf.border begin
-        tag "tr" "align=left" begin
+        tag "tr" begin
           tag "td" "align=right" begin
             Wserver.wprint "%s\n" (capitale (transl conf "long display"));
-            Wserver.wprint "<input type=checkbox name=long value=on><br>\n";
-            Wserver.wprint "%s\n" (capitale (transl conf "include spouses"));
-            Wserver.wprint "<input type=checkbox name=spouse value=on><br>\n";
+            Wserver.wprint "<input type=checkbox name=long value=on>\n";
           end;
           tag "td" "align=right" begin
             Wserver.wprint "%s\n"
               (capitale (transl_nth conf "image/images" 1));
-            Wserver.wprint "<input type=checkbox name=image value=on><br>\n";
+            Wserver.wprint "<input type=checkbox name=image value=on>\n";
+          end;
+        end;
+        tag "tr" begin
+          tag "td" "align=right" begin
+            Wserver.wprint "%s\n" (capitale (transl conf "include spouses"));
+            Wserver.wprint "<input type=checkbox name=spouse value=on>\n";
+          end;
+          tag "td" "align=right" begin
             Wserver.wprint "%s\n"
               (capitale (transl conf "cancel GeneWeb links"));
-            Wserver.wprint "<input type=checkbox name=cgl value=on><br>\n";
+            Wserver.wprint "<input type=checkbox name=cgl value=on>\n";
+          end;
+        end;
+        tag "tr" begin
+          tag "td" "align=right" begin
+            Wserver.wprint "%s\n" (capitale (transl conf "border"));
+            Wserver.wprint "<input name=bd size=1 maxlength=2 value=0>\n";
+          end;
+          tag "td" "align=right" begin
+            Wserver.wprint "\
+<table><tr><td>%s</td>
+<td><input type=radio name=color value=\"\" checked></td>\n"
+              (capitale (transl conf "color"));
+            List.iter
+              (fun c ->
+                 Wserver.wprint "\
+<td bgcolor=%s><input type=radio name=color value=%s></td>\n" c c)
+              ["FFC0C0"; "FFFFC0"; "C0FFC0"; "C0FFFF"; "C0C0FF"; "FFC0FF"];
+            Wserver.wprint "</tr></table>\n";
+          end;
+        end;
+        tag "tr" begin
+          tag "td" "align=right" begin
+            Wserver.wprint "&nbsp;";
+          end;
+          tag "td" "align=right" begin
+            Wserver.wprint "&nbsp;";
           end;
         end;
         tag "tr" "align=left" begin
@@ -262,8 +294,8 @@ value add_missing_parents_of_siblings conf base indl =
                     match ind.di_val with
                     [ Some ip ->
                         let ip =
-                          match (aget conf base ip).parents with
-                          [ Some ifam -> (coi base ifam).father
+                          match parents (aget conf base ip) with
+                          [ Some ifam -> father (coi base ifam)
                           | None -> assert False ]
                         in
                         if List.mem ip ipl then ipl else [ip :: ipl]
@@ -368,19 +400,31 @@ value html_table_of_relation_path_dag conf base elem_txt vbar_txt path =
 ;
 
 value next_relation_link_txt conf ip1 ip2 excl_faml =
-  "href=\"" ^ commd conf ^ "em=R;ei=" ^
-    string_of_int (Adef.int_of_iper ip1) ^ ";i=" ^
-    string_of_int (Adef.int_of_iper ip2) ^
-    (if p_getenv conf.env "spouse" = Some "on" then ";spouse=on" else "") ^
-    (if conf.cancel_links then ";cgl=on" else "") ^ ";et=S" ^
-    fst
-      (List.fold_left
-         (fun (txt, i) ifam ->
-            (txt ^ ";ef" ^ string_of_int i ^ "=" ^
-               string_of_int (Adef.int_of_ifam ifam),
-             i + 1))
-         ("", 0) (List.rev excl_faml)) ^
-    "\""
+  let bd =
+    match p_getenv conf.env "bd" with
+    [ None | Some ("0" | "") -> ""
+    | Some x -> ";bd=" ^ x ]
+  in
+  let color =
+    match p_getenv conf.env "color" with
+    [ None -> ""
+    | Some x -> ";color=" ^ x ]
+  in
+  let (sl, _) =
+    List.fold_left
+      (fun (sl, i) ifam ->
+         ([";ef"; string_of_int i; "=";
+           string_of_int (Adef.int_of_ifam ifam) :: sl], i - 1))
+      (["\""], List.length excl_faml - 1) excl_faml
+  in
+  let sl =
+    ["href=\""; commd conf; "em=R;ei=";
+     string_of_int (Adef.int_of_iper ip1); ";i=";
+     string_of_int (Adef.int_of_iper ip2);
+     if p_getenv conf.env "spouse" = Some "on" then ";spouse=on" else "";
+     if conf.cancel_links then ";cgl=on" else ""; bd; color; ";et=S" :: sl]
+  in
+  String.concat "" sl
 ;
 
 value print_relation_path conf base ip1 ip2 path ifam excl_faml =
@@ -426,7 +470,7 @@ value get_shortest_path_relation conf base ip1 ip2 excl_faml =
         let cpl = coi base ifam in
         mark_fam.(Adef.int_of_ifam ifam) := True;
         let result =
-          [(cpl.father, Parent, ifam); (cpl.mother, Parent, ifam)]
+          [((father cpl), Parent, ifam); ((mother cpl), Parent, ifam)]
         in
         let result =
           result @
@@ -445,7 +489,7 @@ value get_shortest_path_relation conf base ip1 ip2 excl_faml =
                      (fun child children ->
                         [(child, HalfSibling, fam) :: children])
                      (Array.to_list (doi base fam).children) children)
-              (Array.to_list (uget conf base cpl.father).family) []
+              (Array.to_list (uget conf base (father cpl)).family) []
         in
         let result =
           result @
@@ -458,7 +502,7 @@ value get_shortest_path_relation conf base ip1 ip2 excl_faml =
                      (fun child children ->
                         [(child, HalfSibling, fam) :: children])
                      (Array.to_list (doi base fam).children) children)
-              (Array.to_list (uget conf base cpl.mother).family) []
+              (Array.to_list (uget conf base (mother cpl)).family) []
         in
         result
       }
@@ -474,14 +518,14 @@ value get_shortest_path_relation conf base ip1 ip2 excl_faml =
                List.fold_right
                  (fun child children -> [(child, Child, ifam) :: children])
                  (Array.to_list (doi base ifam).children)
-                 [(cpl.father, Mate, ifam); (cpl.mother, Mate, ifam)] @
+                 [((father cpl), Mate, ifam); ((mother cpl), Mate, ifam)] @
                  nb
              })
           (Array.to_list (uget conf base iper).family) []
       in
       let result =
         result @
-          (match (aget conf base iper).parents with
+          (match parents (aget conf base iper) with
            [ Some ifam -> parse_fam ifam
            | _ -> [] ])
       in
@@ -564,10 +608,15 @@ value print_shortest_path conf base p1 p2 =
     trailer conf
   }
   else
+    (* optimization to be used 1/ if database not too big or 2/ running
+    on machines with much memory *)
+(*
     let _ = base.data.ascends.array () in
     let _ = base.data.unions.array () in
     let _ = base.data.couples.array () in
     let _ = base.data.descends.array () in
+*)
+    (**)
     let excl_faml =
       loop [] 0 where rec loop list i =
         match p_getint conf.env ("ef" ^ string_of_int i) with
@@ -851,7 +900,7 @@ value nephew_label conf x p =
 ;
 
 value same_parents conf base p1 p2 =
-  (aget conf base p1.cle_index).parents = (aget conf base p2.cle_index).parents
+  parents (aget conf base p1.cle_index) = parents (aget conf base p2.cle_index)
 ;
 
 value print_link_name conf base n p1 p2 sol =
@@ -1197,7 +1246,12 @@ value print_dag_links conf base p1 p2 rl =
              in
              ()
            };
-           Wserver.wprint ";dag=on\">";
+           let image_opt =
+             match p_getenv conf.env "image" with
+             [ Some "on" -> ";image=on"
+             | _ -> "" ]
+           in
+           Wserver.wprint ";dag=on%s\">" image_opt;
            if is_anc then Wserver.wprint "%s" (transl conf "tree")
            else
              Wserver.wprint "%d %s" nn
@@ -1347,8 +1401,13 @@ value no_sp p = None;
 value compute_relationship conf base by_marr p1 p2 =
   if p1.cle_index == p2.cle_index then None
   else
+    (* optimization to be used 1/ if database not too big or 2/ running
+    on machines with much memory *)
+(*
     let _ = base.data.ascends.array () in
     let _ = base.data.couples.array () in
+*)
+    (**)
     let tstab = Util.create_topological_sort conf base in
     let sol = compute_simple_relationship conf base tstab p1 p2 in
     let sol_by_marr =
@@ -1423,10 +1482,19 @@ value print_one_path conf base found a p1 p2 pp1 pp2 l1 l2 =
   let b2 = find_first_branch conf base dist ip l2 ip2 Neuter in
   match (b1, b2) with
   [ (Some b1, Some b2) ->
+      let bd = match p_getint conf.env "bd" with [ Some x -> x | None -> 0 ] in
+      let td_prop =
+        match Util.p_getenv conf.env "td" with
+        [ Some x -> " " ^ x
+        | _ ->
+            match Util.p_getenv conf.env "color" with
+	    [ None | Some "" -> ""
+            | Some x -> " bgcolor=" ^ x ] ]
+      in
       let info =
         {ip = ip; sp = a.sex; ip1 = ip1; ip2 = ip2; b1 = b1; b2 = b2; c1 = 1;
          c2 = 1; pb1 = None; pb2 = None; nb1 = None; nb2 = None; sp1 = sp1;
-         sp2 = sp2}
+         sp2 = sp2; bd = bd; td_prop = td_prop}
       in
       if List.mem (b1, b2) found.val then ()
       else do {
@@ -1466,6 +1534,12 @@ value print_main_relationship conf base long p1 p2 rel =
     match p_getenv conf.env "cgl" with
     [ Some "on" -> conf.senv := conf.senv @ [("cgl", "on")]
     | _ -> () ];
+    match p_getenv conf.env "bd" with
+    [ None | Some ("0" | "") -> ()
+    | Some x -> conf.senv := conf.senv @ [("bd", x)] ];
+    match p_getenv conf.env "color" with
+    [ None | Some "" -> ()
+    | Some x -> conf.senv := conf.senv @ [("color", x)] ];
     match rel with
     [ None ->
         if p1.cle_index == p2.cle_index then
@@ -1508,8 +1582,8 @@ value print_main_relationship conf base long p1 p2 rel =
                (if Num.eq total Num.one then 0 else 1));
           if long then () else print_dag_links conf base p1 p2 rl;
           if not all_by_marr && authorized_age conf base p1 &&
-             authorized_age conf base p2 && a1.consang != Adef.fix (-1) &&
-             a2.consang != Adef.fix (-1)
+             authorized_age conf base p2 && (consang a1) != Adef.fix (-1) &&
+             consang a2 != Adef.fix (-1)
           then do {
             html_p conf;
             Wserver.wprint "<em>%s: " (capitale (transl conf "relationship"));
