@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: relationLink.ml,v 4.6 2002/05/17 10:06:46 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: relationLink.ml,v 4.12 2004/12/14 09:30:17 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
@@ -23,7 +23,9 @@ type info =
     nb1 : option (list (iper * sex));
     nb2 : option (list (iper * sex));
     sp1 : option person;
-    sp2 : option person }
+    sp2 : option person;
+    bd : int;
+    td_prop : string }
 ;
 
 type dist = { dmin : mutable int; dmax : mutable int; mark : bool };
@@ -41,10 +43,13 @@ value tsort_leq tstab x y =
 value make_dist_tab conf base ia maxlev =
   if maxlev <= threshold.val then phony_dist_tab
   else
-    let _ = base.data.ascends.array () in
-    let _ = base.data.couples.array () in
+    (* optimization to be used 1/ if database not too big or 2/ running
+    on machines with much memory *)
+(*
     let _ = base.data.unions.array () in
     let _ = base.data.descends.array () in
+*)
+    (**)
     let tstab = Util.create_topological_sort conf base in
     let module Pq =
       Pqueue.Make
@@ -74,11 +79,11 @@ value make_dist_tab conf base ia maxlev =
       while not (Pq.is_empty q.val) do {
         let (k, nq) = Pq.take q.val in
         q.val := nq;
-        match (aget conf base (Adef.iper_of_int k)).parents with
+        match parents (aget conf base (Adef.iper_of_int k)) with
         [ Some ifam ->
             let cpl = coi base ifam in
-            let dfath = dist.(Adef.int_of_iper cpl.father) in
-            let dmoth = dist.(Adef.int_of_iper cpl.mother) in
+            let dfath = dist.(Adef.int_of_iper (father cpl)) in
+            let dmoth = dist.(Adef.int_of_iper (mother cpl)) in
             do {
               dist.(k).dmin := min dfath.dmin dmoth.dmin + 1;
               dist.(k).dmax := max dfath.dmax dmoth.dmax + 1;
@@ -98,12 +103,12 @@ value find_first_branch conf base (dmin, dmax) ia =
     else if len == 0 then None
     else if len < dmin ip || len > dmax ip then None
     else
-      match (aget conf base ip).parents with
+      match parents (aget conf base ip) with
       [ Some ifam ->
           let cpl = coi base ifam in
-          match find [(ip, sp) :: br] (len - 1) cpl.father Male with
+          match find [(ip, sp) :: br] (len - 1) (father cpl) Male with
           [ Some _ as r -> r
-          | None -> find [(ip, sp) :: br] (len - 1) cpl.mother Female ]
+          | None -> find [(ip, sp) :: br] (len - 1) (mother cpl) Female ]
       | None -> None ]
 ;
 
@@ -116,10 +121,10 @@ value rec next_branch_same_len conf base dist backward missing ia sa ipl =
         [ Female ->
             next_branch_same_len conf base dist True (missing + 1) ip sp ipl1
         | Male ->
-            match (aget conf base ip).parents with
+            match parents (aget conf base ip) with
             [ Some ifam ->
                 let cpl = coi base ifam in
-                next_branch_same_len conf base dist False missing cpl.mother
+                next_branch_same_len conf base dist False missing (mother cpl)
                   Female ipl
             | _ -> failwith "next_branch_same_len" ]
         | Neuter -> assert False ] ]
@@ -127,10 +132,10 @@ value rec next_branch_same_len conf base dist backward missing ia sa ipl =
   else if missing < fst dist ia || missing > snd dist ia then
     next_branch_same_len conf base dist True missing ia sa ipl
   else
-    match (aget conf base ia).parents with
+    match parents (aget conf base ia) with
     [ Some ifam ->
         let cpl = coi base ifam in
-        next_branch_same_len conf base dist False (missing - 1) cpl.father Male
+        next_branch_same_len conf base dist False (missing - 1) (father cpl) Male
           [(ia, sa) :: ipl]
     | None -> next_branch_same_len conf base dist True missing ia sa ipl ]
 ;
@@ -151,10 +156,10 @@ value rec prev_branch_same_len conf base dist backward missing ia sa ipl =
         [ Male ->
             prev_branch_same_len conf base dist True (missing + 1) ip sp ipl1
         | Female ->
-            match (aget conf base ip).parents with
+            match parents (aget conf base ip) with
             [ Some ifam ->
                 let cpl = coi base ifam in
-                prev_branch_same_len conf base dist False missing cpl.father
+                prev_branch_same_len conf base dist False missing (father cpl)
                   Male ipl
             | _ -> failwith "prev_branch_same_len" ]
         | Neuter -> assert False ] ]
@@ -162,10 +167,10 @@ value rec prev_branch_same_len conf base dist backward missing ia sa ipl =
   else if missing < fst dist ia || missing > snd dist ia then
     prev_branch_same_len conf base dist True missing ia sa ipl
   else
-    match (aget conf base ia).parents with
+    match parents (aget conf base ia) with
     [ Some ifam ->
         let cpl = coi base ifam in
-        prev_branch_same_len conf base dist False (missing - 1) cpl.mother
+        prev_branch_same_len conf base dist False (missing - 1) (mother cpl)
           Female [(ia, sa) :: ipl]
     | None -> prev_branch_same_len conf base dist True missing ia sa ipl ]
 ;
@@ -188,14 +193,14 @@ value spouse_text conf base end_sp ip ipl =
   match (ipl, (p_getenv conf.env "spouse", p_getenv conf.env "opt")) with
   [ ([(ips, _) :: _], (Some "on", _) | (_, Some "spouse")) ->
       let a = aget conf base ips in
-      match a.parents with
+      match parents a with
       [ Some ifam ->
           let c = coi base ifam in
           let fam = foi base ifam in
-          let sp = if ip = c.father then c.mother else c.father in
+          let sp = if ip = (father c) then (mother c) else (father c) in
           let d =
             Date.short_marriage_date_text conf base fam
-              (pget conf base c.father) (pget conf base c.mother)
+              (pget conf base (father c)) (pget conf base (mother c))
           in
           (someone_text conf base sp, d, Some sp)
       | _ -> ("", "", None) ]
@@ -206,25 +211,29 @@ value spouse_text conf base end_sp ip ipl =
   | _ -> ("", "", None) ]
 ;
 
-value print_someone conf base ip =
-  do {
-    Wserver.wprint "%s\n" (someone_text conf base ip);
-    Wserver.wprint "%s" (Dag.image_txt conf base (pget conf base ip))
-  }
-;
-
-value print_spouse conf base n ip ipl =
+value print_someone_and_spouse conf base info in_tab ip n ipl =
   let (s, d, spo) = spouse_text conf base n ip ipl in
-  if s <> "" then do {
-    Wserver.wprint "&amp;%s" d;
-    html_br conf;
-    Wserver.wprint "%s\n" s;
-    match spo with
-    [ Some ip ->
-        Wserver.wprint "%s" (Dag.image_txt conf base (pget conf base ip))
-    | _ -> () ]
+  do {
+    if in_tab && (info.bd > 0 || info.td_prop <> "") then
+      Wserver.wprint "<table border=%d><tr><td align=center%s>" info.bd
+        info.td_prop
+    else ();
+    Wserver.wprint "%s\n" (someone_text conf base ip);
+    Wserver.wprint "%s" (Dag.image_txt conf base (pget conf base ip));
+    if s <> "" then do {
+      Wserver.wprint "&amp;%s" d;
+      html_br conf;
+      Wserver.wprint "%s\n" s;
+      match spo with
+      [ Some ip ->
+          Wserver.wprint "%s" (Dag.image_txt conf base (pget conf base ip))
+      | _ -> () ]
+    }
+    else ();
+    if in_tab && (info.bd > 0 || info.td_prop <> "") then
+      Wserver.wprint "</td></tr></table>"
+    else ();
   }
-  else ()
 ;
 
 value rec print_both_branches conf base info pl1 pl2 =
@@ -258,20 +267,14 @@ value rec print_both_branches conf base info pl1 pl2 =
       tag "td" "valign=top align=center width=\"50%%\"" begin
         match p1 with
         [ Some p1 ->
-            do {
-              print_someone conf base p1;
-              print_spouse conf base info.sp1 p1 pl1
-            }
+            print_someone_and_spouse conf base info True p1 info.sp1 pl1
         | None -> Wserver.wprint "&nbsp;" ];
       end;
       tag "td" begin Wserver.wprint "&nbsp;"; end;
       tag "td" "valign=top align=center width=\"50%%\"" begin
         match p2 with
         [ Some p2 ->
-            do {
-              print_someone conf base p2;
-              print_spouse conf base info.sp2 p2 pl2
-            }
+            print_someone_and_spouse conf base info True p2 info.sp2 pl2
         | None -> Wserver.wprint "&nbsp;" ];
       end;
     end;
@@ -323,29 +326,6 @@ value rec print_both_branches_pre conf base info sz pl1 pl2 =
   }
 ;
 
-value rec print_one_branch conf base ipl1 sp =
-  if ipl1 = [] then ()
-  else do {
-    let (ip1, ipl1) =
-      match ipl1 with
-      [ [(ip1, _) :: ipl1] -> (Some ip1, ipl1)
-      | [] -> (None, []) ]
-    in
-    match ip1 with
-    [ Some ip1 -> do { Wserver.wprint "|"; html_br conf }
-    | None -> () ];
-    match ip1 with
-    [ Some ip1 ->
-        do {
-          print_someone conf base ip1;
-          print_spouse conf base sp ip1 ipl1;
-          html_br conf
-        }
-    | None -> () ];
-    print_one_branch conf base ipl1 sp
-  }
-;
-
 value include_marr conf base n =
   match find_person_in_env conf base n with
   [ Some p -> ";" ^ acces_n conf base n p
@@ -365,6 +345,15 @@ value sign_text conf base sign info b1 b2 c1 c2 =
     (match p_getenv conf.env "image" with
      [ Some "on" -> ";image=on"
      | _ -> "" ]) ^
+    (match p_getenv conf.env "bd" with
+     [ None | Some ("0" | "") -> ""
+     | Some x -> ";bd=" ^ x ]) ^
+    (match p_getenv conf.env "td" with
+     [ None | Some "" -> ""
+     | Some x -> ";td=" ^ x ]) ^
+    (match p_getenv conf.env "color" with
+     [ None | Some "" -> ""
+     | Some x -> ";color=" ^ x ]) ^
     include_marr conf base "3" ^ include_marr conf base "4" ^ "\">" ^ sign ^
     "</a>"
 ;
@@ -420,21 +409,21 @@ value print_prev_next_2 conf base info pb nb =
 value other_parent_text_if_same conf base info =
   match (info.b1, info.b2) with
   [ ([(sib1, _) :: _], [(sib2, _) :: _]) ->
-      match ((aget conf base sib1).parents, (aget conf base sib2).parents) with
+      match (parents (aget conf base sib1), parents (aget conf base sib2)) with
       [ (Some ifam1, Some ifam2) ->
           let cpl1 = coi base ifam1 in
           let cpl2 = coi base ifam2 in
           let other_parent =
-            if cpl1.father = info.ip then
-              if cpl1.mother = cpl2.mother then Some cpl1.mother else None
-            else if cpl1.father = cpl2.father then Some cpl1.father
+            if (father cpl1) = info.ip then
+              if (mother cpl1) = (mother cpl2) then Some (mother cpl1) else None
+            else if (father cpl1) = (father cpl2) then Some (father cpl1)
             else None
           in
           match other_parent with
           [ Some ip ->
               let d =
                 Date.short_marriage_date_text conf base (foi base ifam1)
-                  (pget conf base cpl1.father) (pget conf base cpl1.mother)
+                  (pget conf base (father cpl1)) (pget conf base (mother cpl1))
               in
               Some ("&amp;" ^ d ^ " " ^ someone_text conf base ip, ip)
           | _ -> None ]
@@ -442,17 +431,81 @@ value other_parent_text_if_same conf base info =
   | _ -> None ]
 ;
 
-value print_other_parent_if_same conf base info =
-  match other_parent_text_if_same conf base info with
-  [ Some (s, ip) ->
-      do {
-        Wserver.wprint "%s" s;
-        Wserver.wprint "%s" (Dag.image_txt conf base (pget conf base ip))
-      }
-  | None -> () ]
+value print_someone_and_other_parent_if_same conf base info =
+  do {
+    if info.bd > 0 || info.td_prop <> "" then
+      Wserver.wprint "<table border=%d><tr><td align=center%s>"
+        info.bd info.td_prop
+    else ();
+    Wserver.wprint "%s\n" (someone_text conf base info.ip);
+    Wserver.wprint "%s" (Dag.image_txt conf base (pget conf base info.ip));
+    match other_parent_text_if_same conf base info with
+    [ Some (s, ip) ->
+        do {
+          Wserver.wprint "%s" s;
+          Wserver.wprint "%s" (Dag.image_txt conf base (pget conf base ip))
+        }
+    | None -> () ];
+    if info.bd > 0 || info.td_prop <> "" then
+      Wserver.wprint "</td></tr></table>"
+    else ();
+  }
 ;
 
-value print_with_pre conf base info =
+value rec list_iter_hd_tl f =
+  fun
+  [ [x :: l] -> do { f x l; list_iter_hd_tl f l }
+  | [] -> () ]
+;
+
+value print_one_branch_no_table conf base info =
+  let b = if info.b1 = [] then info.b2 else info.b1 in
+  let sp = if info.b1 = [] then info.sp2 else info.sp1 in
+  tag "center" begin
+    print_someone_and_spouse conf base info False info.ip sp b;
+    html_br conf;
+    list_iter_hd_tl
+      (fun (ip1, _) ipl1 ->
+         do {
+           Wserver.wprint "|";
+           html_br conf;
+           print_someone_and_spouse conf base info False ip1 sp ipl1;
+           html_br conf;
+         })
+      b;
+  end
+;
+
+value print_one_branch_with_table conf base info =
+  let b = if info.b1 = [] then info.b2 else info.b1 in
+  let sp = if info.b1 = [] then info.sp2 else info.sp1 in
+  tag "table" "border=%d cellspacing=0 cellpadding=0 width=\"100%%\""
+    conf.border
+  begin
+    tag "tr" begin
+      tag "td" "align=center" begin
+        print_someone_and_spouse conf base info True info.ip sp b;
+      end;
+      list_iter_hd_tl
+        (fun (ip1, _) ipl1 ->
+           do {
+             tag "tr" begin
+               tag "td" "align=center" begin
+                 Wserver.wprint "|";
+               end;
+             end;
+             tag "tr" begin
+               tag "td" "align=center" begin
+                 print_someone_and_spouse conf base info True ip1 sp ipl1;
+               end;
+             end;
+           })
+        b;
+    end;
+  end
+;
+
+value print_two_branches_with_pre conf base info =
   let sz = 79 in
   tag "pre" begin
     print_pre_center sz (someone_text conf base info.ip);
@@ -479,14 +532,13 @@ value print_with_pre conf base info =
   end
 ;
 
-value print_with_table conf base info =
+value print_two_branches_with_table conf base info =
   tag "table" "border=%d cellspacing=0 cellpadding=0 width=\"100%%\""
     conf.border
   begin
     tag "tr" "align=left" begin
       stag "td" "colspan=3 align=center" begin
-        print_someone conf base info.ip;
-        print_other_parent_if_same conf base info;
+        print_someone_and_other_parent_if_same conf base info;
       end;
     end;
     tag "tr" "align=left" begin
@@ -497,7 +549,9 @@ value print_with_table conf base info =
         Wserver.wprint "<hr size=1 noshade width=\"50%%\" align=right>";
       end;
       Wserver.wprint "\n";
-      stag "td" begin Wserver.wprint "<hr size=1 noshade>"; end;
+      stag "td" begin
+        Wserver.wprint "<hr size=1 noshade width=\"100%%\">";
+      end;
       Wserver.wprint "\n";
       stag "td" "align=left" begin
         Wserver.wprint "<hr size=1 noshade width=\"50%%\" align=left>";
@@ -528,15 +582,16 @@ value print_with_table conf base info =
 ;
 
 value print_relation_path conf base info =
+  let with_table =
+    match p_getenv conf.env "tab" with
+    [ Some "on" -> True
+    | Some "off" -> False
+    | _ -> not (browser_doesnt_have_tables conf) ]
+  in
   if info.b1 = [] || info.b2 = [] then do {
-    let b = if info.b1 = [] then info.b2 else info.b1 in
-    let sp = if info.b1 = [] then info.sp2 else info.sp1 in
-    tag "center" begin
-      print_someone conf base info.ip;
-      print_spouse conf base sp info.ip b;
-      html_br conf;
-      print_one_branch conf base b sp;
-    end;
+    if (info.bd > 0 || info.td_prop <> "") && with_table then
+      print_one_branch_with_table conf base info
+    else print_one_branch_no_table conf base info;
     if not conf.cancel_links &&
        (info.pb1 <> None || info.nb1 <> None || info.pb2 <> None ||
         info.nb2 <> None) then
@@ -551,15 +606,8 @@ value print_relation_path conf base info =
     }
     else ()
   }
-  else
-    let with_table =
-      match p_getenv conf.env "tab" with
-      [ Some "on" -> True
-      | Some "off" -> False
-      | _ -> not (browser_doesnt_have_tables conf) ]
-    in
-    if with_table then print_with_table conf base info
-    else print_with_pre conf base info
+  else if with_table then print_two_branches_with_table conf base info
+  else print_two_branches_with_pre conf base info
 ;
 
 value print_relation_ok conf base info =
@@ -641,10 +689,19 @@ value print_relation_no_dag conf base po ip1 ip2 =
       in
       let sp1 = find_person_in_env conf base "3" in
       let sp2 = find_person_in_env conf base "4" in
+      let bd = match p_getint conf.env "bd" with [ Some x -> x | None -> 0 ] in
+      let td_prop =
+        match Util.p_getenv conf.env "td" with
+        [ Some x -> " " ^ x
+        | _ ->
+            match Util.p_getenv conf.env "color" with
+	    [ None | Some "" -> ""
+            | Some x -> " bgcolor=" ^ x ] ]
+      in
       let info =
         {ip = ip; sp = sp; ip1 = ip1; ip2 = ip2; b1 = b1; b2 = b2; c1 = c1;
          c2 = c2; pb1 = pb1; pb2 = pb2; nb1 = nb1; nb2 = nb2; sp1 = sp1;
-         sp2 = sp2}
+         sp2 = sp2; bd = bd; td_prop = td_prop}
       in
       print_relation_ok conf base info
   | _ -> incorrect_request conf ]
