@@ -1,5 +1,5 @@
-(* camlp4r ./pa_lock.cmo ./pa_html.cmo *)
-(* $Id: sendImage.ml,v 4.8 2002/03/11 17:50:45 ddr Exp $ *)
+(* camlp4r ./pa_html.cmo *)
+(* $Id: sendImage.ml,v 4.13 2004/10/04 11:18:04 ddr Exp $ *)
 
 open Gutil;
 open Util;
@@ -170,7 +170,7 @@ value print_sent conf base p =
 ;
 
 value write_file fname content =
-  let oc = open_out_bin fname in
+  let oc = Secure.open_out_bin fname in
   do { output_string oc content; flush oc; close_out oc }
 ;
 
@@ -192,7 +192,8 @@ value move_file_to_old conf typ fname bfname =
 
 value normal_image_type s =
   if String.length s > 10 && Char.code s.[0] = 0xff &&
-     Char.code s.[1] = 0xd8 && String.sub s 6 4 = "JFIF"
+     Char.code s.[1] = 0xd8 &&
+     (let m = String.sub s 6 4 in m = "JFIF" || m = "Exif")
   then
     ".jpg"
   else if String.length s > 4 && String.sub s 0 4 = "\137PNG" then ".png"
@@ -235,7 +236,7 @@ value dump_bad_image conf s =
   match p_getenv conf.base_env "dump_bad_images" with
   [ Some "yes" ->
       try
-        let oc = open_out_bin "bad-image" in
+        let oc = Secure.open_out_bin "bad-image" in
         do { output_string oc s; flush oc; close_out oc }
       with
       [ Sys_error _ -> () ]
@@ -272,7 +273,7 @@ value effective_send_ok conf base p file =
     let bfdir = Util.base_path ["images"] conf.bname in
     if Sys.file_exists bfdir then bfdir
     else do {
-      let d = Filename.concat Util.base_dir.val "images" in
+      let d = Filename.concat (Secure.base_dir ()) "images" in
       let d1 = Filename.concat d conf.bname in
       try Unix.mkdir d 0o777 with [ Unix.Unix_error _ _ _ -> () ];
       try Unix.mkdir d1 0o777 with [ Unix.Unix_error _ _ _ -> () ];
@@ -296,24 +297,20 @@ value effective_send_ok conf base p file =
 ;
 
 value print_send_ok conf base =
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock Iobase.lock_file bfile with
-  [ Accept ->
-      try
-        let ip =
-          let s = raw_get conf "i" in
-          try int_of_string s with [ Failure _ -> incorrect conf ]
-        in
-        let p = base.data.persons.get ip in
-        let digest = Update.digest_person p in
-        if digest = raw_get conf "digest" then
-          let file = raw_get conf "file" in
+  try
+    let ip =
+      let s = raw_get conf "i" in
+      try int_of_string s with [ Failure _ -> incorrect conf ]
+    in
+    let p = base.data.persons.get ip in
+    let digest = Update.digest_person p in
+    if digest = raw_get conf "digest" then
+      let file = raw_get conf "file" in
 let _ = do { Printf.eprintf "file size %d\n" (String.length file); flush stderr; } in
-          effective_send_ok conf base p file
-        else Update.error_digest conf base
-      with
-      [ Update.ModErr -> () ]
-  | Refuse -> Update.error_locked conf base ]
+      effective_send_ok conf base p file
+    else Update.error_digest conf base
+  with
+  [ Update.ModErr -> () ]
 ;
 
 (* Delete image form validated *)
@@ -351,16 +348,12 @@ value effective_delete_ok conf base p =
 ;
 
 value print_del_ok conf base =
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock Iobase.lock_file bfile with
-  [ Accept ->
-      try
-        match p_getint conf.env "i" with
-        [ Some ip ->
-            let p = base.data.persons.get ip in
-            effective_delete_ok conf base p
-        | None -> incorrect conf ]
-      with
-      [ Update.ModErr -> () ]
-  | Refuse -> Update.error_locked conf base ]
+  try
+    match p_getint conf.env "i" with
+    [ Some ip ->
+        let p = base.data.persons.get ip in
+        effective_delete_ok conf base p
+    | None -> incorrect conf ]
+  with
+  [ Update.ModErr -> () ]
 ;
