@@ -1,6 +1,6 @@
 (* camlp4r ./def.syn.cmo ./pa_html.cmo *)
-(* $Id: some.ml,v 4.21 2002/12/09 12:16:19 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: some.ml,v 4.25 2004/12/14 09:30:17 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -325,8 +325,11 @@ value print_by_branch x conf base not_found_fun (pl, homonymes) =
     let psn =
       match homonymes with
       [ [_] ->
-          try List.assoc "always_surname" conf.base_env = "yes" with
-          [ Not_found -> False ]
+          match p_getenv conf.env "alwsurn" with
+          [ Some x -> x = "yes"
+          | None ->
+              try List.assoc "always_surname" conf.base_env = "yes" with
+              [ Not_found -> False ] ]
       | _ -> True ]
     in
     let title h =
@@ -335,6 +338,7 @@ value print_by_branch x conf base not_found_fun (pl, homonymes) =
         else geneweb_link conf ("m=N;v=" ^ code_varenv (Name.lower x)) x
       in
       do {
+        let homonymes = List.sort compare homonymes in
         Wserver.wprint "%s" (access (List.hd homonymes));
         List.iter (fun x -> Wserver.wprint ", %s" (access x))
           (List.tl homonymes);
@@ -376,18 +380,18 @@ value print_by_branch x conf base not_found_fun (pl, homonymes) =
              }
              else ();
              if br = None || br = Some n then
-               match (aget conf base p.cle_index).parents with
+               match parents (aget conf base p.cle_index) with
                [ Some ifam ->
                    let cpl = coi base ifam in
                    do {
-                     let pp = pget conf base cpl.father in
+                     let pp = pget conf base (father cpl) in
                      if is_hidden pp then
                        Wserver.wprint "&lt;&lt;"
                      else
                        let href = Util.acces conf base pp in
                        wprint_geneweb_link conf href "&lt;&lt;";
                      Wserver.wprint "\n&amp;\n";
-                     let pp = pget conf base cpl.mother in
+                     let pp = pget conf base (mother cpl) in
                      if is_hidden pp then
                        Wserver.wprint "&lt;&lt;"
                      else
@@ -482,19 +486,19 @@ value select_ancestors conf base name_inj ipl =
     (fun ipl ip ->
        let p = pget conf base ip in
        let a = aget conf base ip in
-       match a.parents with
+       match parents a with
        [ Some ifam ->
            let cpl = coi base ifam in
-           let fath = pget conf base cpl.father in
-           let moth = pget conf base cpl.mother in
+           let fath = pget conf base (father cpl) in
+           let moth = pget conf base (mother cpl) in
            let s = str_inj p.surname in
            if str_inj fath.surname <> s && str_inj moth.surname <> s &&
               not (List.memq ip ipl) then
-             if List.memq cpl.father ipl then ipl
+             if List.memq (father cpl) ipl then ipl
              else if not (is_hidden fath) &&
                has_at_least_2_children_with_surname conf base (doi base ifam)
                  p.surname then
-               [cpl.father :: ipl]
+               [(father cpl) :: ipl]
              else [ip :: ipl]
            else ipl
        | _ -> [ip :: ipl] ])
