@@ -1,5 +1,5 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: templ.ml,v 4.15 2002/12/09 05:09:54 ddr Exp $ *)
+(* $Id: templ.ml,v 4.22 2004/12/14 09:53:21 ddr Exp $ *)
 
 open Config;
 open Util;
@@ -28,6 +28,12 @@ value rec get_ident len =
   | [: :] -> Buff.get len ]
 ;
 
+value rec get_value len =
+  parser
+  [ [: `(' ' | '>' | ';' | '\n' | '\r' | '\t') :] -> Buff.get len
+  | [: `c; s :] -> get_value (Buff.store len c) s ]
+;
+
 value rec get_string len =
   parser
   [ [: `'"' :] -> Buff.get len
@@ -49,6 +55,8 @@ value get_variable =
   in
   parser
   [ [: `'%' :] -> ("%", [])
+  | [: `'[' :] -> ("[", [])
+  | [: `']' :] -> ("]", [])
   | [: v = get_ident 0; vl = var_kont :] -> (v, vl) ]
 ;
 
@@ -223,7 +231,8 @@ value parse_templ conf strm =
     [ [: `'%' :] ->
         let astl = if len = 0 then astl else [Atext (buff_get len) :: astl] in
         match get_variable strm with
-        [ ("%", []) -> parse_astl [Atext "%" :: astl] False 0 end_list strm
+        [ (("%" | "[" | "]" as c), []) ->
+            parse_astl [Atext c :: astl] False 0 end_list strm
         | (v, []) when List.mem v end_list -> (List.rev astl, v)
         | ("define", []) -> parse_define astl end_list strm
         | x ->
@@ -232,7 +241,7 @@ value parse_templ conf strm =
               [ ("if", []) -> parse_if strm
               | ("foreach", []) -> parse_foreach strm
               | ("apply", []) -> parse_apply strm
-              | ("wid_hei", []) -> Awid_hei (get_ident 0 strm)
+              | ("wid_hei", []) -> Awid_hei (get_value 0 strm)
               | (v, vl) -> Avar v vl ]
             in
             parse_astl [ast :: astl] False 0 end_list strm ]
@@ -307,16 +316,17 @@ value open_templ conf dir name =
   let std_fname =
     search_in_lang_path (Filename.concat "etc" (name ^ ".txt"))
   in
-  if dir = "" then try Some (open_in std_fname) with [ Sys_error _ -> None ]
+  if dir = "" then
+    try Some (Secure.open_in std_fname) with [ Sys_error _ -> None ]
   else
     let dir = Filename.basename dir in
     let fname =
       Filename.concat (Util.base_path ["etc"] dir) (name ^ ".txt")
     in
-    try Some (open_in fname) with
+    try Some (Secure.open_in fname) with
     [ Sys_error _ ->
         if dir = conf.bname then
-          try Some (open_in std_fname) with [ Sys_error _ -> None ]
+          try Some (Secure.open_in std_fname) with [ Sys_error _ -> None ]
         else None ]
 ;
 
@@ -445,36 +455,57 @@ value rec skip_spaces_and_newlines s i =
 
 value eval_transl_lexicon conf upp s c =
   let r =
-    match try Some (int_of_string c) with [ Failure _ -> None ] with
-    [ Some n ->
-        match split_at_coloncolon s with
-        [ None -> Gutil.nominative (Util.transl_nth conf s n)
-        | Some (s1, s2) ->
-            try
-              if String.length s2 > 0 && s2.[0] = '|' then
-                let i = 1 in
-                let j = String.rindex s2 '|' in
-                let k = skip_spaces_and_newlines s2 (j + 1) in
-                let s3 = String.sub s2 i (j - i) in
-                let s4 = String.sub s2 k (String.length s2 - k) in
-                let s2 = s3 ^ Util.transl_nth conf s4 n in
-                Util.transl_decline conf s1 s2
-              else if String.length s2 > 0 && s2.[0] = ':' then
-                let s2 = String.sub s2 1 (String.length s2 - 1) in
-                Printf.sprintf
-                  (Util.ftransl_nth conf (Util.valid_format "%t" s1) n)
-                     (fun _ -> s2)
-              else raise Not_found
-            with
-            [ Not_found ->
-                Util.transl_decline conf s1 (Util.transl_nth conf s2 n) ] ]
-    | None -> Gutil.nominative (Util.transl conf s) ^ c ]
+    let nth = try Some (int_of_string c) with [ Failure _ -> None ] in
+    match split_at_coloncolon s with
+    [ None ->
+        let s2 =
+          match nth with
+          [ Some n -> Util.transl_nth conf s n
+          | None -> Util.transl conf s ]
+        in
+        Gutil.nominative s2
+    | Some (s1, s2) ->
+        try
+          if String.length s2 > 0 && s2.[0] = '|' then
+            let i = 1 in
+            let j = String.rindex s2 '|' in
+            let k = skip_spaces_and_newlines s2 (j + 1) in
+            let s3 = String.sub s2 i (j - i) in
+            let s4 = String.sub s2 k (String.length s2 - k) in
+            let s5 =
+              match nth with
+              [ Some n -> Util.transl_nth conf s4 n
+              | None -> Util.transl conf s4 ]
+            in
+            let s2 = s3 ^ s5 in
+            Util.transl_decline conf s1 s2
+          else if String.length s2 > 0 && s2.[0] = ':' then
+            let s2 = String.sub s2 1 (String.length s2 - 1) in
+            let s3 = Util.valid_format "%t" s1 in
+            let s4 =
+              match nth with
+              [ Some n -> Util.ftransl_nth conf s3 n
+              | None -> Util.ftransl conf s3 ]
+            in
+            Printf.sprintf s4 (fun _ -> s2)
+          else raise Not_found
+        with
+        [ Not_found ->
+            let s3 =
+              match nth with
+              [ Some n -> Util.transl_nth conf s2 n
+              | None -> Util.transl conf s2 ]
+            in
+            Util.transl_decline conf s1 s3 ] ]
   in
   if upp then capitale r else r
 ;
 
 value eval_transl_inline conf s =
-  Translate.inline conf.lang '%' (fun c -> "%" ^ String.make 1 c) s
+  let (s, alt) =
+    Translate.inline conf.lang '%' (fun c -> "%" ^ String.make 1 c) s
+  in
+  if alt then "[" ^ s ^ "]" else s
 ;
 
 value eval_transl conf upp s c =
@@ -510,7 +541,12 @@ value eval_date_variable od =
       | None -> "" ]
   | "month" ->
       match eval_date_field od with
-      [ Some d -> if d.month = 0 then "" else string_of_int d.month
+      [ Some d ->
+          if d.month = 0 then ""
+          else
+            match od with
+            [ Some (Dgreg _ Dfrench) -> short_f_month d.month
+            | _ -> string_of_int d.month ]
       | None -> "" ]
   | "text" -> eval_date_text od
   | "year" ->
@@ -567,7 +603,7 @@ value print_copyright conf base =
       do {
         html_p conf;
         Wserver.wprint "
-<hr><font size=-1><em>(c) Copyright 2002 INRIA -
+<hr><font size=-1><em>Copyright (c) 1998-2005 INRIA -
 GeneWeb %s</em></font>" Version.txt;
         html_br conf;
       } ]
