@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: update.ml,v 4.25 2002/10/26 01:22:43 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: update.ml,v 4.34 2004/12/14 09:30:17 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
@@ -12,14 +12,6 @@ type create_info = (option date * string * death * option date * string);
 type create = [ Create of sex and option create_info | Link ];
 type key = (string * string * int * create * string);
 
-value rec find_free_occ base f s i =
-  match
-    try Some (person_ht_find_unique base f s i) with [ Not_found -> None ]
-  with
-  [ Some _ -> find_free_occ base f s (i + 1)
-  | None -> i ]
-;
-
 value has_children base u =
   List.exists
     (fun ifam ->
@@ -113,7 +105,7 @@ value update_misc_names_of_family base p u =
                        person_ht_add base name ip
                      else ())
                   (person_misc_names base (poi base ip)))
-             [cpl.mother :: Array.to_list des.children])
+             [mother cpl :: Array.to_list des.children])
         (Array.to_list u.family)
   | _ -> () ]
 ;
@@ -231,8 +223,8 @@ value print_warning conf base =
         (fun _ -> print_someone_strong conf base p)
   | ChangedOrderOfChildren ifam des before ->
       let cpl = coi base ifam in
-      let fath = poi base cpl.father in
-      let moth = poi base cpl.mother in
+      let fath = poi base (father cpl) in
+      let moth = poi base (mother cpl) in
       do {
         Wserver.wprint "%s\n"
           (capitale (transl conf "changed order of children"));
@@ -283,8 +275,8 @@ value print_warning conf base =
           (fcapitale
              (ftransl conf
                 "the following children of %t and %t are not in order"))
-          (fun _ -> print_someone_strong conf base (poi base cpl.father))
-          (fun _ -> print_someone_strong conf base (poi base cpl.mother));
+          (fun _ -> print_someone_strong conf base (poi base (father cpl)))
+          (fun _ -> print_someone_strong conf base (poi base (mother cpl)));
         Wserver.wprint ":\n";
         Wserver.wprint "<ul>\n";
         html_li conf;
@@ -474,6 +466,7 @@ value reconstitute_date_dmy conf var =
     if len > 1 then
       match (y.[0], y.[len-1]) with
       [ ('?', _) -> (Some "maybe", String.sub y 1 (len - 1))
+      | ('~', _) -> (Some "about", String.sub y 1 (len - 1))
       | ('/', '/') -> (Some "about", String.sub y 1 (len - 2))
       | ('<', _) | ('/', _) -> (Some "before", String.sub y 1 (len - 1))
       | ('>', _) -> (Some "after", String.sub y 1 (len - 1))
@@ -481,43 +474,64 @@ value reconstitute_date_dmy conf var =
       | _ -> (prec, y) ]
     else (prec, y)
   in
-  match int_of_field y with
-  [ Some y ->
-      let prec =
-        match prec with
-        [ Some "about" -> About
-        | Some "maybe" -> Maybe
-        | Some "before" -> Before
-        | Some "after" -> After
-        | Some "oryear" ->
-            match get_number var "oryear" conf.env with
-            [ Some y -> OrYear y
-            | None -> Sure ]
-        | Some "yearint" ->
-            match get_number var "oryear" conf.env with
-            [ Some y -> YearInt y
-            | None -> Sure ]
-        | _ -> Sure ]
-      in
-      match get_number var "mm" conf.env with
-      [ Some m ->
-          match get_number var "dd" conf.env with
-          [ Some d ->
-              let d =
-                {day = d; month = m; year = y; prec = prec; delta = 0}
-              in
-              if d.day >= 1 && d.day <= 31 && d.month >= 1 &&
-                 d.month <= 13 then
-                Some d
-              else bad_date conf d
-          | None ->
-              let d =
-                {day = 0; month = m; year = y; prec = prec; delta = 0}
-              in
-              if d.month >= 1 && d.month <= 13 then Some d
-              else bad_date conf d ]
-      | None -> Some {day = 0; month = 0; year = y; prec = prec; delta = 0} ]
-  | None -> None ]
+  let (force_f_cal, m) =
+    let m = get var "mm" conf.env in
+    match String.uppercase m with
+    [ "VD" -> (True, Some 1)
+    | "BR" -> (True, Some 2)
+    | "FM" -> (True, Some 3)
+    | "NI" -> (True, Some 4)
+    | "PL" -> (True, Some 5)
+    | "VT" -> (True, Some 6)
+    | "GE" -> (True, Some 7)
+    | "FL" -> (True, Some 8)
+    | "PR" -> (True, Some 9)
+    | "ME" -> (True, Some 10)
+    | "TH" -> (True, Some 11)
+    | "FT" -> (True, Some 12)
+    | "JC" -> (True, Some 13)
+    | _ -> (False, int_of_field m) ]
+  in
+  let d =
+    match int_of_field y with
+    [ Some y ->
+        let prec =
+          match prec with
+          [ Some "about" -> About
+          | Some "maybe" -> Maybe
+          | Some "before" -> Before
+          | Some "after" -> After
+          | Some "oryear" ->
+              match get_number var "oryear" conf.env with
+              [ Some y -> OrYear y
+              | None -> Sure ]
+          | Some "yearint" ->
+              match get_number var "oryear" conf.env with
+              [ Some y -> YearInt y
+              | None -> Sure ]
+          | _ -> Sure ]
+        in
+        match m with
+        [ Some m ->
+            match get_number var "dd" conf.env with
+            [ Some d ->
+                let d =
+                  {day = d; month = m; year = y; prec = prec; delta = 0}
+                in
+                if d.day >= 1 && d.day <= 31 && d.month >= 1 &&
+                   d.month <= 13 then
+                  Some d
+                else bad_date conf d
+            | None ->
+                let d =
+                  {day = 0; month = m; year = y; prec = prec; delta = 0}
+                in
+                if d.month >= 1 && d.month <= 13 then Some d
+                else bad_date conf d ]
+        | None -> Some {day = 0; month = 0; year = y; prec = prec; delta = 0} ]
+    | None -> None ]
+  in
+  (d, force_f_cal)
 ;
 
 value check_greg_day conf d =
@@ -526,7 +540,7 @@ value check_greg_day conf d =
 
 value reconstitute_date conf var =
   match reconstitute_date_dmy conf var with
-  [ Some d ->
+  [ (Some d, False) ->
       let (d, cal) =
         match p_getenv conf.env (var ^ "_cal") with
         [ Some "G" | None -> do { check_greg_day conf d; (d, Dgregorian) }
@@ -536,7 +550,8 @@ value reconstitute_date conf var =
         | _ -> (d, Dgregorian) ]
       in
       Some (Dgreg d cal)
-  | None ->
+  | (Some d, True) -> Some (Dgreg (Calendar.gregorian_of_french d) Dfrench)
+  | (None, _) ->
       match p_getenv conf.env (var ^ "_text") with
       [ Some txt ->
           let txt = strip_spaces (get var "text" conf.env) in
@@ -678,8 +693,8 @@ value parse_r_parent = parser [ [: `'f' :] -> 0 | [: `'m' :] -> 1 ];
 
 value text_of_var conf =
   fun
-  [ "him" -> transl_nth conf "him/her" 0
-  | "her" -> transl_nth conf "him/her" 1
+  [ "pa1" -> transl_nth conf "him/her" 0
+  | "pa2" -> transl_nth conf "him/her" 1
   | var ->
       match Stream.of_string var with parser
       [ [: `'r'; pos = parse_int 0; `'_'; pn = parse_r_parent :] ->
@@ -776,10 +791,13 @@ value insert_person conf base src new_persons (f, s, o, create, var) =
           let o = if f = "?" || s = "?" then 0 else o in
           let ip = Adef.iper_of_int base.data.persons.len in
           let empty_string = insert_string base "" in
-          let (birth, birth_place) =
+          let (birth, birth_place, baptism, baptism_place) =
             match info with
-            [ Some (b, bpl, _, _, _) -> (b, bpl)
-            | None -> (None, "") ]
+            [ Some (b, bpl, _, _, _) ->
+                if String.length bpl >= 2 && String.sub bpl 0 2 = "b/" then
+                  (None, "", b, String.sub bpl 2 (String.length bpl - 2))
+                else (b, bpl, None, "")
+            | None -> (None, "", None, "") ]
           in
           let (death, death_place) =
             match info with
@@ -800,8 +818,9 @@ value insert_person conf base src new_persons (f, s, o, create, var) =
              occupation = empty_string; sex = sex; access = IfTitles;
              birth = Adef.codate_of_od birth;
              birth_place = insert_string base birth_place;
-             birth_src = empty_string; baptism = Adef.codate_None;
-             baptism_place = empty_string; baptism_src = empty_string;
+             birth_src = empty_string; baptism = Adef.codate_of_od baptism;
+             baptism_place = insert_string base baptism_place;
+             baptism_src = empty_string;
              death = death; death_place = insert_string base death_place;
              death_src = empty_string; burial = UnknownBurial;
              burial_place = empty_string; burial_src = empty_string;
@@ -810,7 +829,7 @@ value insert_person conf base src new_persons (f, s, o, create, var) =
                if f = "?" || s = "?" then empty_string
                else insert_string base src;
              cle_index = ip}
-          and a = {parents = None; consang = Adef.fix (-1)}
+          and a = no_ascend ()
           and u = {family = [| |]} in
           do {
             base.func.patch_person p.cle_index p;
@@ -857,8 +876,8 @@ value print_family_stuff conf base p a u =
                    (Adef.int_of_ifam fi);
                  Wserver.wprint "%s</a><br>\n"
                    (capitale (transl_decline conf "invert" ""));
-                 if cpl1.father = cpl2.father &&
-                    cpl1.mother = cpl2.mother then
+                 if (father cpl1) = (father cpl2) &&
+                    (mother cpl1) = (mother cpl2) then
                     do {
                    stag "a" "href=\"%sm=MRG_FAM;i=%d;i2=%d;ip=%d\""
                      (commd conf) (Adef.int_of_ifam prev_fi)
@@ -898,7 +917,7 @@ value print_family_stuff conf base p a u =
     Wserver.wprint "<br>\n";
     let s = transl_nth conf "marriage/marriages" 0 in
     if (p_first_name base p = "?" || p_surname base p = "?") &&
-       (Array.length u.family <> 0 || a.parents <> None) then
+       (Array.length u.family <> 0 || parents a <> None) then
       ()
     else if p.sex = Neuter then do {
       Wserver.wprint "<a href=\"%sm=ADD_FAM;ip=%d;sex=M\">%s (%s)</a><br>\n"
@@ -983,7 +1002,7 @@ value print conf base p =
             Wserver.wprint "%s" (capitale (transl_decline conf "merge" ""));
           end;
           Wserver.wprint "<br>\n";
-          match a.parents with
+          match parents a with
           [ Some _ -> ()
           | None ->
               if p_first_name base p = "?" || p_surname base p = "?" then ()
