@@ -1,6 +1,6 @@
 (* camlp4r ./pa_html.cmo *)
-(* $Id: updateFam.ml,v 4.34 2002/10/23 15:20:15 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: updateFam.ml,v 4.43 2004/12/14 09:30:18 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Gutil;
@@ -26,9 +26,9 @@ value person_key base ip =
   (first_name, surname, occ, Update.Link, "")
 ;
 
-value string_family_of base fam cpl des =
+value string_family_of conf base fam cpl des =
   let sfam = Gutil.map_family_ps (person_key base) (sou base) fam in
-  let scpl = Gutil.map_couple_p (person_key base) cpl in
+  let scpl = Gutil.map_couple_p conf.multi_parents (person_key base) cpl in
   let sdes = Gutil.map_descend_p (person_key base) des in
   (sfam, scpl, sdes)
 ;
@@ -79,10 +79,10 @@ value get_create (fn, sn, oc, create, var) = create;
 
 value rec eval_variable conf base env ((fam, cpl, des) as fcd) =
   fun
-  [ ["father"; s] -> VVind cpl.father s
-  | ["father"; "create"; s] -> VVcreate (get_create cpl.father) s
-  | ["mother"; s] -> VVind cpl.mother s
-  | ["mother"; "create"; s] -> VVcreate (get_create cpl.mother) s
+  [ ["father"; s] -> VVind (father cpl) s
+  | ["father"; "create"; s] -> VVcreate (get_create (father cpl)) s
+  | ["mother"; s] -> VVind (mother cpl) s
+  | ["mother"; "create"; s] -> VVcreate (get_create (mother cpl)) s
   | ["marriage"; s] -> VVdate (Adef.od_of_codate fam.marriage) s
   | ["divorce"; s] ->
       let d =
@@ -117,6 +117,23 @@ value rec eval_variable conf base env ((fam, cpl, des) as fcd) =
         [ (Some ind, [s]) -> VVind ind s
         | (Some ind, ["create"; s]) -> VVcreate (get_create ind) s
         | _ -> VVnone ]
+  | ["parent" :: sl] ->
+        let r =
+          match get_env "cnt" env with
+          [ Vint i ->
+              let arr = parent_array cpl in
+              let i = i - 1 in
+              if i >= 0 && i < Array.length arr  then
+                Some arr.(i)
+              else if i >= 0 && i < 1 && Array.length arr = 0 then
+                Some ("", "", 0, Update.Create Neuter None, "")
+              else None
+          | _ -> None ]
+        in
+        match (r, sl) with
+        [ (Some ind, [s]) -> VVind ind s
+        | (Some ind, ["create"; s]) -> VVcreate (get_create ind) s
+        | _ -> VVnone ]
   | [] -> VVgen ""
   | [s] ->
       let v = extract_var "cvar_" s in if v <> "" then VVcvar v else VVgen s
@@ -128,7 +145,8 @@ value eval_relation_kind =
   [ Married -> "marr"
   | NotMarried -> "not_marr"
   | Engaged -> "engaged"
-  | NoSexesCheck -> "nsck" ]
+  | NoSexesCheck -> "nsck"
+  | NoMention -> "no_ment" ]
 ;
 
 value eval_divorce =
@@ -170,7 +188,7 @@ value try_eval_gen_variable conf base env ((fam, cpl, des) as fcd) =
       else raise Not_found ]
 ;
 
-value eval_key_variable (fn, sn, oc, create, var) =
+value eval_key_variable conf env (fn, sn, oc, create, var) =
   fun
   [ "first_name" -> quote_escaped fn
   | "occ" -> if oc = 0 then "" else string_of_int oc
@@ -182,32 +200,53 @@ value eval_key_variable (fn, sn, oc, create, var) =
       | Update.Create Female _ -> "female"
       | Update.Create Neuter _ -> "neuter"
       | _ -> "" ]
+  | "himher" ->
+      match get_env "cnt" env with
+      [ Vint 1 -> capitale (transl_nth conf "him/her" 0)
+      | Vint 2 -> capitale (transl_nth conf "him/her" 1)
+      | Vint n -> transl conf "him/her"
+      | _ -> "???" ]
   | s -> ">%" ^ s ^ "???" ]
 ;
 
+value add_precision s p =
+  match p with
+  [ Maybe -> "?" ^ s
+  | Before -> "<" ^ s
+  | After -> ">" ^ s
+  | About -> "/" ^ s ^ "/"
+  | _ -> s ]
+;
+
 value eval_create_variable c =
   fun
   [ "birth_year" ->
       match c with
       [ Update.Create _
+        (Some (Some (Dgreg dmy Dfrench), _, _, _, _)) ->
+          let dmy = Calendar.french_of_gregorian dmy in
+          add_precision (string_of_int dmy.year) dmy.prec
+      | Update.Create _
         (Some (Some (Dgreg {year = y; prec = p} _), _, _, _, _)) ->
-          let s = string_of_int y in
-          match p with
-          [ Maybe -> "?" ^ s
-          | Before -> "<" ^ s
-          | After -> ">" ^ s
-          | About -> "/" ^ s ^ "/"
-          | _ -> s ]
+          add_precision (string_of_int y) p
       | _ -> "" ]
   | "birth_month" ->
       match c with
-      [ Update.Create _ (Some (Some (Dgreg {month = m} _), _, _, _, _))
+      [ Update.Create _ 
+        (Some (Some (Dgreg dmy Dfrench), _, _, _, _)) ->
+          let dmy = Calendar.french_of_gregorian dmy in
+          short_f_month dmy.month
+      | Update.Create _ (Some (Some (Dgreg {month = m} _), _, _, _, _))
         when m <> 0 ->
           string_of_int m
       | _ -> "" ]
   | "birth_day" ->
       match c with
-      [ Update.Create _ (Some (Some (Dgreg {day = d} _), _, _, _, _))
+      [ Update.Create _ 
+        (Some (Some (Dgreg dmy Dfrench), _, _, _, _)) ->
+          let dmy = Calendar.french_of_gregorian dmy in
+          if dmy.day <> 0 then string_of_int dmy.day else ""
+      | Update.Create _ (Some (Some (Dgreg {day = d} _), _, _, _, _))
         when d <> 0 ->
           string_of_int d
       | _ -> "" ]
@@ -218,14 +257,12 @@ value eval_create_variable c =
   | "death_year" ->
       match c with
       [ Update.Create _
+        (Some (_, _, _, Some (Dgreg dmy Dfrench), _)) ->
+          let dmy = Calendar.french_of_gregorian dmy in
+          add_precision (string_of_int dmy.year) dmy.prec
+      | Update.Create _
         (Some (_, _, _, Some (Dgreg {year = y; prec = p} _), _)) ->
-          let s = string_of_int y in
-          match p with
-          [ Maybe -> "?" ^ s
-          | Before -> "<" ^ s
-          | After -> ">" ^ s
-          | About -> "/" ^ s ^ "/"
-          | _ -> s ]
+          add_precision (string_of_int y) p
       | Update.Create _ (Some (_, _, death, None, _)) ->
           match death with
           [ DeadDontKnowWhen -> "+"
@@ -234,13 +271,21 @@ value eval_create_variable c =
       | _ -> "" ]
   | "death_month" ->
       match c with
-      [ Update.Create _ (Some (_, _, _, Some (Dgreg {month = m} _), _))
+      [ Update.Create _ 
+        (Some (_, _, _, Some (Dgreg dmy Dfrench), _)) ->
+          let dmy = Calendar.french_of_gregorian dmy in
+          short_f_month dmy.month
+      | Update.Create _ (Some (_, _, _, Some (Dgreg {month = m} _), _))
         when m <> 0 ->
           string_of_int m
       | _ -> "" ]
   | "death_day" ->
       match c with
-      [ Update.Create _ (Some (_, _, _, Some (Dgreg {day = d} _), _))
+      [ Update.Create _ 
+        (Some (_, _, _, Some (Dgreg dmy Dfrench), _)) ->
+          let dmy = Calendar.french_of_gregorian dmy in
+          if dmy.day <> 0 then string_of_int dmy.day else ""
+      | Update.Create _ (Some (_, _, _, Some (Dgreg {day = d} _), _))
         when d <> 0 ->
           string_of_int d
       | _ -> "" ]
@@ -282,12 +327,23 @@ value eval_gen_bool_variable conf base env fcd =
       else do { Wserver.wprint ">%%%s???" s; False } ]
 ;
 
+value eval_base_bool_variable conf base env fcd =
+  fun
+  [ s ->
+      let v = extract_var "cvar_" s in
+      if v <> "" then
+        match p_getenv conf.base_env v with
+        [ Some "yes" -> True
+        | _ -> False ]
+      else do { Wserver.wprint ">%%%s???" s; False } ]
+;
+
 value eval_bool_variable conf base env fcd s sl =
   match eval_variable conf base env fcd [s :: sl] with
   [ VVgen s -> eval_gen_bool_variable conf base env fcd s
   | VVcreate _ _ -> do { Wserver.wprint ">%%%s???" s; False }
   | VVdate _ _ -> do { Wserver.wprint ">%%%s???" s; False }
-  | VVcvar _ -> do { Wserver.wprint ">%%%s???" s; False }
+  | VVcvar _ -> eval_base_bool_variable conf base env fcd s
   | VVind _ _ -> do { Wserver.wprint ">%%VVind???"; False }
   | VVnone -> do { Wserver.wprint ">%%%s???" s; False } ]
 ;
@@ -317,7 +373,7 @@ value eval_bool_value conf base env fcd =
           | VVcreate c s -> do { Wserver.wprint ">%%%s???" s; "" }
           | VVdate od s -> Templ.eval_date_variable od s
           | VVcvar s -> eval_base_env_variable conf s
-          | VVind pk s -> eval_key_variable pk s
+          | VVind pk s -> eval_key_variable conf env pk s
           | VVnone -> do { Wserver.wprint ">%%%s???" s; "" } ]
         with
         [ Not_found -> do { Wserver.wprint ">%%%s???" s; "" } ]
@@ -339,7 +395,7 @@ value print_variable conf base env fcd sl =
   | VVcvar s ->
       try Wserver.wprint "%s" (List.assoc s conf.base_env) with
       [ Not_found -> () ]
-  | VVind pk s -> Wserver.wprint "%s" (eval_key_variable pk s)
+  | VVind pk s -> Wserver.wprint "%s" (eval_key_variable conf env pk s)
   | VVnone ->
       do {
         Wserver.wprint ">%%";
@@ -399,6 +455,7 @@ and print_simple_foreach conf base env ((fam, cpl, des) as fcd) al s =
   match s with
   [ "child" -> print_foreach_child conf base env fcd al des.children s
   | "witness" -> print_foreach_witness conf base env fcd al fam.witnesses s
+  | "parent" -> print_foreach_parent conf base env fcd al (parent_array cpl) s
   | _ -> Wserver.wprint "foreach %s???" s ]
 and print_foreach_child conf base env fcd al arr lab =
   for i = 0 to max 1 (Array.length arr) - 1 do {
@@ -410,6 +467,11 @@ and print_foreach_witness conf base env fcd al arr lab =
     let env = [("cnt", Vint (i + 1)) :: env] in
     List.iter (print_ast conf base env fcd) al
   }
+and print_foreach_parent conf base env fcd al arr lab =
+  for i = 0 to Array.length arr - 1 do {
+    let env = [("cnt", Vint (i + 1)) :: env] in
+    List.iter (print_ast conf base env fcd) al
+  }
 ;
 
 value interp_templ conf base fcd digest astl =
@@ -436,7 +498,10 @@ value print_mod1 conf base fam cpl des digest =
 ;
 
 value print_merge1 conf base fam des fam2 digest =
-  let cpl = Gutil.map_couple_p (person_key base) (coi base fam.fam_index) in
+  let cpl =
+    Gutil.map_couple_p conf.multi_parents (person_key base)
+      (coi base fam.fam_index)
+  in
   print_update_fam conf base (fam, cpl, des) digest
 ;
 
@@ -478,13 +543,13 @@ value print_inv1 conf base p fam1 fam2 =
       (capitale (transl conf "invert the order of the following families"));
     tag "ul" begin
       html_li conf;
-      Update.print_someone conf base (poi base cpl1.father);
+      Update.print_someone conf base (poi base (father cpl1));
       Wserver.wprint " %s " (transl_nth conf "and" 0);
-      Update.print_someone conf base (poi base cpl1.mother);
+      Update.print_someone conf base (poi base (mother cpl1));
       html_li conf;
-      Update.print_someone conf base (poi base cpl2.father);
+      Update.print_someone conf base (poi base (father cpl2));
       Wserver.wprint " %s " (transl_nth conf "and" 0);
-      Update.print_someone conf base (poi base cpl2.mother);
+      Update.print_someone conf base (poi base (mother cpl2));
     end;
     Wserver.wprint "\n";
     tag "form" "method=POST action=\"%s\"" conf.command begin
@@ -533,7 +598,7 @@ value print_add conf base =
      witnesses = [| |]; relation = Married; divorce = NotDivorced;
      comment = ""; origin_file = ""; fsources = default_source conf;
      fam_index = bogus_family_index}
-  and cpl = {father = fath; mother = moth}
+  and cpl = couple conf.multi_parents fath moth
   and des = {children = [| |]} in
   print_add1 conf base fam cpl des digest False
 ;
@@ -548,8 +613,9 @@ value print_add_parents conf base =
          comment = ""; origin_file = ""; fsources = default_source conf;
          fam_index = bogus_family_index}
       and cpl =
-        {father = ("", sou base p.surname, 0, Update.Create Neuter None, "");
-         mother = ("", "", 0, Update.Create Neuter None, "")}
+        couple conf.multi_parents
+          ("", sou base p.surname, 0, Update.Create Neuter None, "")
+          ("", "", 0, Update.Create Neuter None, "")
       and des =
         {children =
            [| (sou base p.first_name, sou base p.surname, p.occ, Update.Link,
@@ -565,7 +631,7 @@ value print_mod conf base =
       let fam = foi base (Adef.ifam_of_int i) in
       let cpl = coi base (Adef.ifam_of_int i) in
       let des = doi base (Adef.ifam_of_int i) in
-      let (sfam, scpl, sdes) = string_family_of base fam cpl des in
+      let (sfam, scpl, sdes) = string_family_of conf base fam cpl des in
       let digest = Update.digest_family fam cpl des in
       print_mod1 conf base sfam scpl sdes digest
   | _ -> incorrect_request conf ]
