@@ -1,6 +1,6 @@
-(* camlp4r ./pa_lock.cmo ./pa_html.cmo *)
-(* $Id: updateFamOk.ml,v 4.21 2002/11/06 12:16:38 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* camlp4r ./pa_html.cmo *)
+(* $Id: updateFamOk.ml,v 4.38 2004/12/14 09:30:18 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
@@ -95,15 +95,32 @@ value insert_child conf (children, ext) i =
   | _ -> (children, ext) ]
 ;
 
+value insert_parent conf (parents, ext) i =
+  let var = "ins_pa" ^ string_of_int i in
+  match (p_getenv conf.env var, p_getint conf.env (var ^ "_n")) with
+  [ (_, Some n) when n > 1 ->
+      let parents =
+        loop parents n where rec loop parents n =
+          if n > 0 then
+            let new_parent = ("", "", 0, Update.Create Neuter None, "") in
+            loop [new_parent :: parents] (n - 1)
+          else parents
+      in
+      (parents, True)
+  | (Some "on", _) ->
+      let new_parent = ("", "", 0, Update.Create Neuter None, "") in
+      ([new_parent :: parents], True)
+  | _ -> (parents, ext) ]
+;
+
 value reconstitute_family conf =
   let ext = False in
-  let father = reconstitute_parent_or_child conf "him" "" in
-  let mother = reconstitute_parent_or_child conf "her" "" in
   let relation =
     match p_getenv conf.env "mrel" with
     [ Some "not_marr" -> NotMarried
     | Some "engaged" -> Engaged
     | Some "nsck" -> NoSexesCheck
+    | Some "no_ment" -> NoMention
     | _ -> Married ]
   in
   let marriage = Update.reconstitute_date conf "marr" in
@@ -137,7 +154,7 @@ value reconstitute_family conf =
     | _ ->
         Divorced (Adef.codate_of_od (Update.reconstitute_date conf "div")) ]
   in
-  let surname = getn conf "him" "sn" in
+  let surname = getn conf "pa1" "sn" in
   let (children, ext) =
     loop 1 ext where rec loop i ext =
       match
@@ -155,6 +172,20 @@ value reconstitute_family conf =
       | None -> ([], ext) ]
   in
   let (children, ext) = insert_child conf (children, ext) 0 in
+  let (parents, ext) =
+    loop 1 ext where rec loop i ext =
+      match
+        try
+          Some (reconstitute_parent_or_child conf ("pa" ^ string_of_int i) "")
+        with
+        [ Failure _ -> None ]
+      with
+      [ Some c ->
+          let (parents, ext) = loop (i + 1) ext in
+          let (parents, ext) = insert_parent conf (parents, ext) i in
+          ([c :: parents], ext)
+      | None -> ([], ext) ]
+  in
   let comment = strip_spaces (get conf "comment") in
   let fsources = strip_spaces (get conf "src") in
   let origin_file =
@@ -173,7 +204,7 @@ value reconstitute_family conf =
      witnesses = Array.of_list witnesses; relation = relation;
      divorce = divorce; comment = comment; origin_file = origin_file;
      fsources = fsources; fam_index = Adef.ifam_of_int fam_index}
-  and cpl = {father = father; mother = mother}
+  and cpl = parent conf.multi_parents (Array.of_list parents)
   and des = {children = Array.of_list children} in
   (fam, cpl, des, ext)
 ;
@@ -206,7 +237,7 @@ value print_err_parents conf base p =
     tag "ul" begin
       html_li conf;
       Wserver.wprint "%s: %d" (capitale (transl conf "first free number"))
-        (Update.find_free_occ base (p_first_name base p) (p_surname base p)
+        (Gutil.find_free_occ base (p_first_name base p) (p_surname base p)
            0);
     end;
     Update.print_return conf;
@@ -289,19 +320,21 @@ value infer_origin_file_from_other_marriages conf base ifam ip =
 ;
 
 value infer_origin_file conf base ifam ncpl ndes =
-  let r = infer_origin_file_from_other_marriages conf base ifam ncpl.father in
+  let r =
+    infer_origin_file_from_other_marriages conf base ifam (father ncpl)
+  in
   let r =
     if r = None then
-      infer_origin_file_from_other_marriages conf base ifam ncpl.mother
+      infer_origin_file_from_other_marriages conf base ifam (mother ncpl)
     else r
   in
   let r =
     match r with
     [ Some r -> r
     | None ->
-        let afath = aoi base ncpl.father in
-        let amoth = aoi base ncpl.mother in
-        match (afath.parents, amoth.parents) with
+        let afath = aoi base (father ncpl) in
+        let amoth = aoi base (mother ncpl) in
+        match (parents afath, parents amoth) with
         [ (Some if1, _) when sou base (foi base if1).origin_file <> "" ->
             (foi base if1).origin_file
         | (_, Some if2) when sou base (foi base if2).origin_file <> "" ->
@@ -335,8 +368,8 @@ value update_related_witnesses base ofam_witn nfam_witn ncpl =
          if List.memq ip ofam_witn then ippl
          else
            let p = poi base ip in
-           if not (List.mem ncpl.father p.related) then do {
-             p.related := [ncpl.father :: p.related];
+           if not (List.mem (father ncpl) p.related) then do {
+             p.related := [(father ncpl) :: p.related];
              if List.mem_assoc ip ippl then ippl else [(ip, p) :: ippl]
            }
            else ippl)
@@ -348,8 +381,8 @@ value update_related_witnesses base ofam_witn nfam_witn ncpl =
          if List.memq ip nfam_witn then ippl
          else
            let p = poi base ip in
-           if List.mem ncpl.father p.related then do {
-             p.related := List.filter ( \<> ncpl.father) p.related;
+           if List.mem (father ncpl) p.related then do {
+             p.related := List.filter ( \<> (father ncpl)) p.related;
              if List.mem_assoc ip ippl then ippl else [(ip, p) :: ippl]
            }
            else ippl)
@@ -370,7 +403,8 @@ value effective_mod conf base sfam scpl sdes =
     | None -> "" ]
   in
   let ncpl =
-    map_couple_p (Update.insert_person conf base psrc created_p) scpl
+    map_couple_p conf.multi_parents
+      (Update.insert_person conf base psrc created_p) scpl
   in
   let nfam =
     map_family_ps (Update.insert_person conf base psrc created_p)
@@ -379,10 +413,10 @@ value effective_mod conf base sfam scpl sdes =
   let ndes =
     map_descend_p (Update.insert_person conf base psrc created_p) sdes
   in
-  let nfath = poi base ncpl.father in
-  let nmoth = poi base ncpl.mother in
-  let nfath_u = uoi base ncpl.father in
-  let nmoth_u = uoi base ncpl.mother in
+  let nfath = poi base (father ncpl) in
+  let nmoth = poi base (mother ncpl) in
+  let nfath_u = uoi base (father ncpl) in
+  let nmoth_u = uoi base (mother ncpl) in
   do {
     if sfam.relation <> NoSexesCheck then do {
       match nfath.sex with
@@ -392,8 +426,8 @@ value effective_mod conf base sfam scpl sdes =
       [ Male -> print_err_mother_sex conf base nmoth
       | _ -> nmoth.sex := Female ]
     }
-    else if ncpl.father == ncpl.mother then print_err conf base
     else ();
+    if (father ncpl) == (mother ncpl) then print_err conf base else ();
     if sfam.origin_file = "" then 
       nfam.origin_file :=
         if sou base ofam.origin_file <> "" then ofam.origin_file
@@ -403,22 +437,24 @@ value effective_mod conf base sfam scpl sdes =
     base.func.patch_family fi nfam;
     base.func.patch_couple fi ncpl;
     base.func.patch_descend fi ndes;
-    if ncpl.father != ocpl.father && ncpl.father != ocpl.mother then do {
-      let ofath_u = uoi base ocpl.father in
-      ofath_u.family := family_exclude ofath_u.family ofam.fam_index;
-      nfath_u.family := Array.append nfath_u.family [| fi |];
-      base.func.patch_union ocpl.father ofath_u;
-      base.func.patch_union ncpl.father nfath_u
-    }
-    else ();
-    if ncpl.mother != ocpl.mother && ncpl.mother != ocpl.father then do {
-      let omoth_u = uoi base ocpl.mother in
-      omoth_u.family := family_exclude omoth_u.family ofam.fam_index;
-      nmoth_u.family := Array.append nmoth_u.family [| fi |];
-      base.func.patch_union ocpl.mother omoth_u;
-      base.func.patch_union ncpl.mother nmoth_u
-    }
-    else ();
+    let oarr = parent_array ocpl in
+    let narr = parent_array ncpl in
+    for i = 0 to Array.length oarr - 1 do {
+      if not (array_memq oarr.(i) narr) then do {
+        let ou = uoi base oarr.(i) in
+        ou.family := family_exclude ou.family ofam.fam_index;
+        base.func.patch_union oarr.(i) ou
+      }
+      else ()
+    };
+    for i = 0 to Array.length narr - 1 do {
+      if not (array_memq narr.(i) oarr) then do {
+        let nu = uoi base narr.(i) in
+        nu.family := Array.append nu.family [| fi |];
+        base.func.patch_union narr.(i) nu;
+      }
+      else ()
+    };
     let find_asc =
       let cache = Hashtbl.create 101 in
       fun ip ->
@@ -428,28 +464,28 @@ value effective_mod conf base sfam scpl sdes =
             do { Hashtbl.add cache ip a; a } ]
     in
     let same_parents =
-      ncpl.father = ocpl.father && ncpl.mother = ocpl.mother
+      (father ncpl) = (father ocpl) && (mother ncpl) = (mother ocpl)
     in
     Array.iter
       (fun ip ->
          let a = find_asc ip in
          do {
-           a.parents := None;
+           set_parents a None;
            if not (array_memq ip ndes.children) then
-             a.consang := Adef.fix (-1)
+             set_consang a (Adef.fix (-1))
            else ()
          })
       odes.children;
     Array.iter
       (fun ip ->
          let a = find_asc ip in
-         match a.parents with
+         match parents a with
          [ Some _ -> print_err_parents conf base (poi base ip)
          | None ->
              do {
-               a.parents := Some fi;
+               set_parents a (Some fi);
                if not (array_memq ip odes.children) || not same_parents then
-                 a.consang := Adef.fix (-1)
+                 set_consang a (Adef.fix (-1))
                else ()
              } ])
       ndes.children;
@@ -482,7 +518,8 @@ value effective_add conf base sfam scpl sdes =
     | None -> "" ]
   in
   let ncpl =
-    map_couple_p (Update.insert_person conf base psrc created_p) scpl
+    map_couple_p conf.multi_parents
+      (Update.insert_person conf base psrc created_p) scpl
   in
   let nfam =
     map_family_ps (Update.insert_person conf base psrc created_p)
@@ -492,10 +529,10 @@ value effective_add conf base sfam scpl sdes =
     map_descend_p (Update.insert_person conf base psrc created_p) sdes
   in
   let origin_file = infer_origin_file conf base fi ncpl ndes in
-  let nfath_p = poi base ncpl.father in
-  let nmoth_p = poi base ncpl.mother in
-  let nfath_u = uoi base ncpl.father in
-  let nmoth_u = uoi base ncpl.mother in
+  let nfath_p = poi base (father ncpl) in
+  let nmoth_p = poi base (mother ncpl) in
+  let nfath_u = uoi base (father ncpl) in
+  let nmoth_u = uoi base (mother ncpl) in
   do {
     if sfam.relation <> NoSexesCheck then do {
       match nfath_p.sex with
@@ -503,17 +540,17 @@ value effective_add conf base sfam scpl sdes =
       | Male -> ()
       | _ ->
           do {
-            nfath_p.sex := Male; base.func.patch_person ncpl.father nfath_p
+            nfath_p.sex := Male; base.func.patch_person (father ncpl) nfath_p
           } ];
       match nmoth_p.sex with
       [ Male -> print_err_mother_sex conf base nmoth_p
       | Female -> ()
       | _ ->
           do {
-            nmoth_p.sex := Female; base.func.patch_person ncpl.mother nmoth_p
+            nmoth_p.sex := Female; base.func.patch_person (mother ncpl) nmoth_p
           } ]
     }
-    else if ncpl.father == ncpl.mother then print_err conf base
+    else if (father ncpl) == (mother ncpl) then print_err conf base
     else ();
     nfam.fam_index := fi;
     nfam.origin_file := origin_file;
@@ -522,18 +559,18 @@ value effective_add conf base sfam scpl sdes =
     base.func.patch_descend fi ndes;
     nfath_u.family := Array.append nfath_u.family [| fi |];
     nmoth_u.family := Array.append nmoth_u.family [| fi |];
-    base.func.patch_union ncpl.father nfath_u;
-    base.func.patch_union ncpl.mother nmoth_u;
+    base.func.patch_union (father ncpl) nfath_u;
+    base.func.patch_union (mother ncpl) nmoth_u;
     Array.iter
       (fun ip ->
          let a = aoi base ip in
          let p = poi base ip in
-         match a.parents with
+         match parents a with
          [ Some _ -> print_err_parents conf base p
          | None ->
              do {
-               a.parents := Some fi;
-               a.consang := Adef.fix (-1);
+               set_parents a (Some fi);
+               set_consang a (Adef.fix (-1));
                base.func.patch_ascend p.cle_index a
              } ])
       ndes.children;
@@ -572,7 +609,9 @@ value kill_family base fam ip =
 value kill_parents base ip =
   let a = aoi base ip in
   do {
-    a.parents := None; a.consang := Adef.fix (-1); base.func.patch_ascend ip a
+    set_parents a None;
+    set_consang a (Adef.fix (-1));
+    base.func.patch_ascend ip a
   }
 ;
 
@@ -581,11 +620,11 @@ value effective_del conf base fam =
   let cpl = coi base ifam in
   let des = doi base ifam in
   do {
-    kill_family base fam cpl.father;
-    kill_family base fam cpl.mother;
+    kill_family base fam (father cpl);
+    kill_family base fam (mother cpl);
     Array.iter (kill_parents base) des.children;
-    cpl.father := Adef.iper_of_int (-1);
-    cpl.mother := Adef.iper_of_int (-1);
+    set_father cpl (Adef.iper_of_int (-1));
+    set_mother cpl (Adef.iper_of_int (-1));
     fam.witnesses := [| |];
     des.children := [| |];
     fam.comment := Update.insert_string base "";
@@ -596,12 +635,67 @@ value effective_del conf base fam =
   }
 ;
 
-value all_checks_family conf base fam cpl des =
+value array_forall2 f a1 a2 =
+  if Array.length a1 <> Array.length a2 then invalid_arg "array_forall2"
+  else
+    loop 0 where rec loop i =
+      if i == Array.length a1 then True
+      else if f a1.(i) a2.(i) then loop (i + 1)
+      else False
+;
+
+value array_exists f a =
+  loop 0 where rec loop i =
+    if i = Array.length a then False
+    else if f a.(i) then True
+    else loop (i + 1)
+;
+
+value is_a_link =
+  fun
+  [ (_, _, _, Update.Link, _) -> True
+  | _ -> False ]
+;
+
+value is_created_or_already_there ochil_arr nchil schil =
+  not (is_a_link schil) || array_memq nchil ochil_arr
+;
+
+(* need_check_noloop: optimization
+     The no-loop check being a big work on large databases, this
+   optimization tests if this is really necessary or not. It is not
+   necessary if:
+   1/ either all parents are created,
+   2/ or all children are created,
+   3/ or the new family have the same parents than the old one *and*
+      all linked (not created) new children were already children.
+*)
+
+value need_check_noloop (scpl, sdes, onfs) =
+  if array_exists is_a_link (parent_array scpl) &&
+     array_exists is_a_link sdes.children
+  then
+    match onfs with
+    [ Some ((opar, ochil), (npar, nchil)) ->
+        not
+          (array_forall2 (is_created_or_already_there opar) npar
+	     (parent_array scpl)) ||
+        not
+          (array_forall2 (is_created_or_already_there ochil) nchil
+	     sdes.children)
+    | None -> True ]
+  else False
+;
+
+value all_checks_family conf base fam cpl des scdo =
   let wl = ref [] in
   let error = Update.error conf base in
   let warning w = wl.val := [w :: wl.val] in
   do {
-    Gutil.check_noloop_for_person_list base error [cpl.father; cpl.mother];
+    if need_check_noloop scdo then
+      Gutil.check_noloop_for_person_list base error
+        (Array.to_list (parent_array cpl))
+    else ();
     Gutil.check_family base error warning fam cpl des;
     List.rev wl.val
   }
@@ -625,11 +719,11 @@ value print_family conf base wl cpl des =
     Wserver.wprint "<ul>\n";
     html_li conf;
     Wserver.wprint "\n%s"
-      (referenced_person_text conf base (poi base cpl.father));
+      (referenced_person_text conf base (poi base (father cpl)));
     Wserver.wprint "\n";
     html_li conf;
     Wserver.wprint "\n%s"
-      (referenced_person_text conf base (poi base cpl.mother));
+      (referenced_person_text conf base (poi base (mother cpl)));
     Wserver.wprint "</ul>\n";
     if des.children <> [| |] then do {
       html_p conf;
@@ -714,8 +808,8 @@ value forbidden_disconnected conf sfam scpl sdes =
     [ Not_found -> False ]
   in
   if no_dec then
-    if get_create scpl.father = Update.Link ||
-       get_create scpl.mother = Update.Link then
+    if get_create (father scpl) = Update.Link ||
+       get_create (mother scpl) = Update.Link then
       False
     else
       List.for_all (fun p -> get_create p <> Update.Link)
@@ -725,125 +819,122 @@ value forbidden_disconnected conf sfam scpl sdes =
 
 value print_add o_conf base =
   let conf = Update.update_conf o_conf in
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock Iobase.lock_file bfile with
-  [ Accept ->
-      try
-        let (sfam, scpl, sdes, ext) = reconstitute_family conf in
-        let redisp =
-          match p_getenv conf.env "return" with
-          [ Some _ -> True
-          | _ -> False ]
-        in
-        let digest =
-          match p_getint conf.env "ip" with
-          [ Some ip ->
-              string_of_int (Array.length (base.data.unions.get ip).family)
-          | None -> "" ]
-        in
-        let sdigest = raw_get conf "digest" in
-        if digest <> "" && sdigest <> "" && digest <> sdigest then
-          Update.error_digest conf base
-        else if ext || redisp then
-          UpdateFam.print_update_fam conf base (sfam, scpl, sdes) ""
-        else if forbidden_disconnected conf sfam scpl sdes then
-          print_error_disconnected conf
-        else do {
-          strip_family sfam sdes;
-          let (fam, cpl, des) = effective_add conf base sfam scpl sdes in
-          let wl = all_checks_family conf base fam cpl des in
-          let ((fn, sn, occ, _, _), act) =
-            match p_getint conf.env "ip" with
-            [ Some i ->
-                if Adef.int_of_iper cpl.mother = i then (scpl.mother, "af")
-                else
-                  let a = base.data.ascends.get i in
-                  match a.parents with
-                  [ Some x when x = fam.fam_index ->
-                      let p = base.data.persons.get i in
-                      let key =
-                        (sou base p.first_name, sou base p.surname, p.occ,
-                         Update.Link, "")
-                      in
-                      (key, "aa")
-                  | _ -> (scpl.father, "af") ]
-            | _ -> (scpl.father, "af") ]
-          in
-          Util.commit_patches conf base;
-          History.record conf base (fn, sn, occ) act;
-          Update.delete_topological_sort conf base;
-          print_add_ok conf base wl cpl des
-        }
-      with
-      [ Update.ModErr -> () ]
-  | Refuse -> Update.error_locked conf base ]
+  try
+    let (sfam, scpl, sdes, ext) = reconstitute_family conf in
+    let redisp =
+      match p_getenv conf.env "return" with
+      [ Some _ -> True
+      | _ -> False ]
+    in
+    let digest =
+      match p_getint conf.env "ip" with
+      [ Some ip ->
+          string_of_int (Array.length (base.data.unions.get ip).family)
+      | None -> "" ]
+    in
+    let sdigest = raw_get conf "digest" in
+    if digest <> "" && sdigest <> "" && digest <> sdigest then
+      Update.error_digest conf base
+    else if ext || redisp then
+      UpdateFam.print_update_fam conf base (sfam, scpl, sdes) ""
+    else if forbidden_disconnected conf sfam scpl sdes then
+      print_error_disconnected conf
+    else do {
+      strip_family sfam sdes;
+      let (fam, cpl, des) = effective_add conf base sfam scpl sdes in
+      let wl = all_checks_family conf base fam cpl des (scpl, sdes, None) in
+      let ((fn, sn, occ, _, _), act) =
+        match p_getint conf.env "ip" with
+        [ Some i ->
+            if Adef.int_of_iper (mother cpl) = i then ((mother scpl), "af")
+            else
+              let a = base.data.ascends.get i in
+              match parents a with
+              [ Some x when x = fam.fam_index ->
+                  let p = base.data.persons.get i in
+                  let key =
+                    (sou base p.first_name, sou base p.surname, p.occ,
+                     Update.Link, "")
+                  in
+                  (key, "aa")
+              | _ -> ((father scpl), "af") ]
+        | _ -> ((father scpl), "af") ]
+      in
+      Util.commit_patches conf base;
+      History.record conf base (fn, sn, occ) act;
+      Update.delete_topological_sort conf base;
+      print_add_ok conf base wl cpl des
+    }
+  with
+  [ Update.ModErr -> () ]
 ;
 
 value print_del conf base =
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock Iobase.lock_file bfile with
-  [ Accept ->
-      match p_getint conf.env "i" with
-      [ Some i ->
-          let fam = foi base (Adef.ifam_of_int i) in
-          let k =
-            let cpl = coi base (Adef.ifam_of_int i) in
-            let ip =
-              match p_getint conf.env "ip" with
-              [ Some i when Adef.int_of_iper cpl.mother = i -> cpl.mother
-              | _ -> cpl.father ]
-            in
-            let p = poi base ip in
-            (sou base p.first_name, sou base p.surname, p.occ)
-          in
-          do {
-            if not (is_deleted_family fam) then do {
-              effective_del conf base fam;
-              Util.commit_patches conf base;
-              History.record conf base k "df";
-              Update.delete_topological_sort conf base
-            }
-            else ();
-            print_del_ok conf base []
-          }
-      | _ -> incorrect_request conf ]
-  | Refuse -> Update.error_locked conf base ]
+  match p_getint conf.env "i" with
+  [ Some i ->
+      let fam = foi base (Adef.ifam_of_int i) in
+      let k =
+        let cpl = coi base (Adef.ifam_of_int i) in
+        let ip =
+          match p_getint conf.env "ip" with
+          [ Some i when Adef.int_of_iper (mother cpl) = i -> (mother cpl)
+          | _ -> (father cpl) ]
+        in
+        let p = poi base ip in
+        (sou base p.first_name, sou base p.surname, p.occ)
+      in
+      do {
+        if not (is_deleted_family fam) then do {
+          effective_del conf base fam;
+          Util.commit_patches conf base;
+          History.record conf base k "df";
+          Update.delete_topological_sort conf base
+        }
+        else ();
+        print_del_ok conf base []
+      }
+  | _ -> incorrect_request conf ]
 ;
 
 value print_mod_aux conf base callback =
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock Iobase.lock_file bfile with
-  [ Accept ->
-      try
-        let (sfam, scpl, sdes, ext) = reconstitute_family conf in
-        let redisp =
-          match p_getenv conf.env "return" with
-          [ Some _ -> True
-          | _ -> False ]
-        in
-        let digest =
-          Update.digest_family (foi base sfam.fam_index)
-            (coi base sfam.fam_index) (doi base sfam.fam_index)
-        in
-        if digest = raw_get conf "digest" then
-          if ext || redisp then
-            UpdateFam.print_update_fam conf base (sfam, scpl, sdes) digest
-          else do { strip_family sfam sdes; callback sfam scpl sdes }
-        else Update.error_digest conf base
-      with
-      [ Update.ModErr -> () ]
-  | Refuse -> Update.error_locked conf base ]
+  try
+    let (sfam, scpl, sdes, ext) = reconstitute_family conf in
+    let redisp =
+      match p_getenv conf.env "return" with
+      [ Some _ -> True
+      | _ -> False ]
+    in
+    let digest =
+      Update.digest_family (foi base sfam.fam_index)
+        (coi base sfam.fam_index) (doi base sfam.fam_index)
+    in
+    if digest = raw_get conf "digest" then
+      if ext || redisp then
+        UpdateFam.print_update_fam conf base (sfam, scpl, sdes) digest
+      else do { strip_family sfam sdes; callback sfam scpl sdes }
+    else Update.error_digest conf base
+  with
+  [ Update.ModErr -> () ]
+;
+
+value family_structure conf base ifam =
+  let cpl = coi base ifam in
+  let des = doi base ifam in
+  (parent_array cpl, des.children)
 ;
 
 value print_mod o_conf base =
   let conf = Update.update_conf o_conf in
   let callback sfam scpl sdes =
+    let ofs = family_structure conf base sfam.fam_index in
     let (fam, cpl, des) = effective_mod conf base sfam scpl sdes in
-    let wl = all_checks_family conf base fam cpl des in
+    let nfs = (parent_array cpl, des.children) in
+    let onfs = Some (ofs, nfs) in
+    let wl = all_checks_family conf base fam cpl des (scpl, sdes, onfs) in
     let (fn, sn, occ, _, _) =
       match p_getint conf.env "ip" with
-      [ Some i when Adef.int_of_iper cpl.mother = i -> scpl.mother
-      | _ -> scpl.father ]
+      [ Some i when Adef.int_of_iper (mother cpl) = i -> (mother scpl)
+      | _ -> (father scpl) ]
     in
     do {
       Util.commit_patches conf base;
@@ -856,23 +947,19 @@ value print_mod o_conf base =
 ;
 
 value print_inv conf base =
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock Iobase.lock_file bfile with
-  [ Accept ->
-      match (p_getint conf.env "i", p_getint conf.env "f") with
-      [ (Some ip, Some ifam) ->
-          let p = base.data.persons.get ip in
-          let u = base.data.unions.get ip in
-          let k = (sou base p.first_name, sou base p.surname, p.occ) in
-          try
-            do {
-              effective_inv conf base p.cle_index u (Adef.ifam_of_int ifam);
-              Util.commit_patches conf base;
-              History.record conf base k "if";
-              print_inv_ok conf base p
-            }
-          with
-          [ Update.ModErr -> () ]
-      | _ -> incorrect_request conf ]
-  | Refuse -> Update.error_locked conf base ]
+  match (p_getint conf.env "i", p_getint conf.env "f") with
+  [ (Some ip, Some ifam) ->
+      let p = base.data.persons.get ip in
+      let u = base.data.unions.get ip in
+      let k = (sou base p.first_name, sou base p.surname, p.occ) in
+      try
+        do {
+          effective_inv conf base p.cle_index u (Adef.ifam_of_int ifam);
+          Util.commit_patches conf base;
+          History.record conf base k "if";
+          print_inv_ok conf base p
+        }
+      with
+      [ Update.ModErr -> () ]
+  | _ -> incorrect_request conf ]
 ;
