@@ -1,6 +1,6 @@
-(* camlp4r ./pa_lock.cmo ./pa_html.cmo *)
-(* $Id: updateIndOk.ml,v 4.12 2002/11/14 04:15:34 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* camlp4r ./pa_html.cmo *)
+(* $Id: updateIndOk.ml,v 4.18 2004/12/14 09:30:18 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Config;
 open Def;
@@ -322,7 +322,7 @@ value print_conflict conf base p =
     Update.print_error conf base (AlreadyDefined p);
     html_p conf;
     let free_n =
-      Update.find_free_occ base (p_first_name base p) (p_surname base p) 0
+      Gutil.find_free_occ base (p_first_name base p) (p_surname base p) 0
     in
     tag "ul" begin
       html_li conf;
@@ -511,7 +511,7 @@ value effective_add conf base sp =
       map_person_ps (Update.insert_person conf base sp.psources created_p)
         (Update.insert_string base) sp
     in
-    let na = {parents = None; consang = Adef.fix (-1)} in
+    let na = no_ascend () in
     let nu = {family = [| |]} in
     np.cle_index := pi;
     base.func.patch_person pi np;
@@ -537,13 +537,13 @@ value effective_del conf base p =
   let empty = Update.insert_string base "" in
   let asc = aoi base p.cle_index in
   do {
-    match asc.parents with
+    match parents asc with
     [ Some ifam ->
         let des = doi base ifam in
         do {
           des.children := array_except p.cle_index des.children;
-          asc.parents := None;
-          asc.consang := Adef.fix (-1);
+          set_parents asc None;
+          set_consang asc (Adef.fix (-1));
           base.func.patch_descend ifam des;
           base.func.patch_ascend p.cle_index asc;
           ()
@@ -626,7 +626,7 @@ value all_checks_person conf base p a u =
   do {
     Gutil.check_person base error warning p;
     relation_sex_is_coherent base warning p;
-    match a.parents with
+    match parents a with
     [ Some ifam ->
         Gutil.check_family base error warning (foi base ifam) (coi base ifam)
           (doi base ifam)
@@ -678,80 +678,68 @@ value print_del_ok conf base wl =
 
 value print_add o_conf base =
   let conf = Update.update_conf o_conf in
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock (Iobase.lock_file bfile) with
-  [ Accept ->
-      try
-        let (sp, ext) = reconstitute_person conf in
-        let redisp =
-          match p_getenv conf.env "return" with
-          [ Some _ -> True
-          | _ -> False ]
-        in
-        if ext || redisp then UpdateInd.print_update_ind conf base sp ""
-        else do {
-          strip_person sp;
-          match check_person conf base sp with
-          [ Some err -> error_person conf base sp err
-          | None ->
-              let (p, a) = effective_add conf base sp in
-              let u = uoi base p.cle_index in
-              let wl = all_checks_person conf base p a u in
-              let k = (sp.first_name, sp.surname, sp.occ) in
-              do {
-                Util.commit_patches conf base;
-                History.record conf base k "ap";
-                print_add_ok conf base wl p;
-              } ]
-        }
-      with
-      [ Update.ModErr -> () ]
-  | Refuse -> Update.error_locked conf base ]
+  try
+    let (sp, ext) = reconstitute_person conf in
+    let redisp =
+      match p_getenv conf.env "return" with
+      [ Some _ -> True
+      | _ -> False ]
+    in
+    if ext || redisp then UpdateInd.print_update_ind conf base sp ""
+    else do {
+      strip_person sp;
+      match check_person conf base sp with
+      [ Some err -> error_person conf base sp err
+      | None ->
+          let (p, a) = effective_add conf base sp in
+          let u = uoi base p.cle_index in
+          let wl = all_checks_person conf base p a u in
+          let k = (sp.first_name, sp.surname, sp.occ) in
+          do {
+            Util.commit_patches conf base;
+            History.record conf base k "ap";
+            print_add_ok conf base wl p;
+          } ]
+    }
+  with
+  [ Update.ModErr -> () ]
 ;
 
 value print_del conf base =
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock (Iobase.lock_file bfile) with
-  [ Accept ->
-      match p_getint conf.env "i" with
-      [ Some i ->
-          let p = base.data.persons.get i in
-          let k = (sou base p.first_name, sou base p.surname, p.occ) in
-          do {
-            effective_del conf base p;
-            base.func.patch_person p.cle_index p;
-            Util.commit_patches conf base;
-            History.record conf base k "dp";
-            print_del_ok conf base [];
-          }
-      | _ -> incorrect_request conf ]
-  | Refuse -> Update.error_locked conf base ]
+  match p_getint conf.env "i" with
+  [ Some i ->
+      let p = base.data.persons.get i in
+      let k = (sou base p.first_name, sou base p.surname, p.occ) in
+      do {
+        effective_del conf base p;
+        base.func.patch_person p.cle_index p;
+        Util.commit_patches conf base;
+        History.record conf base k "dp";
+        print_del_ok conf base [];
+      }
+  | _ -> incorrect_request conf ]
 ;
 
 value print_mod_aux conf base callback =
-  let bfile = Util.base_path [] (conf.bname ^ ".gwb") in
-  lock (Iobase.lock_file bfile) with
-  [ Accept ->
-      try
-        let (p, ext) = reconstitute_person conf in
-        let redisp =
-          match p_getenv conf.env "return" with
-          [ Some _ -> True
-          | _ -> False ]
-        in
-        let digest = Update.digest_person (poi base p.cle_index) in
-        if digest = raw_get conf "digest" then
-          if ext || redisp then UpdateInd.print_update_ind conf base p digest
-          else do {
-            strip_person p;
-            match check_person conf base p with
-            [ Some err -> error_person conf base p err
-            | None -> callback p ]
-          }
-        else Update.error_digest conf base
-      with
-      [ Update.ModErr -> () ]
-  | Refuse -> Update.error_locked conf base ]
+  try
+    let (p, ext) = reconstitute_person conf in
+    let redisp =
+      match p_getenv conf.env "return" with
+      [ Some _ -> True
+      | _ -> False ]
+    in
+    let digest = Update.digest_person (poi base p.cle_index) in
+    if digest = raw_get conf "digest" then
+      if ext || redisp then UpdateInd.print_update_ind conf base p digest
+      else do {
+        strip_person p;
+        match check_person conf base p with
+        [ Some err -> error_person conf base p err
+        | None -> callback p ]
+      }
+    else Update.error_digest conf base
+  with
+  [ Update.ModErr -> () ]
 ;
 
 value print_mod o_conf base =
