@@ -1,33 +1,27 @@
 (* camlp4r ./pa_lock.cmo *)
-(* $Id: util.ml,v 4.70 2002/12/30 18:40:05 ddr Exp $ *)
-(* Copyright (c) 2002 INRIA *)
+(* $Id: util.ml,v 4.95 2004/12/14 09:53:21 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Config;
 open Gutil;
 
-value open_in fname =
-  if String.contains fname '\000' then raise (Sys_error "")
-  else Pervasives.open_in fname
-;
-
-value open_in_bin fname =
-  if String.contains fname '\000' then raise (Sys_error "")
-  else Pervasives.open_in_bin fname
-;
-
 value sharelib =
   List.fold_right Filename.concat [Gwlib.prefix; "share"] "geneweb"
 ;
 
-value lang_path = ref [Filename.current_dir_name; sharelib];
-value doc_path = ref [];
-value base_dir = ref Filename.current_dir_name;
+value add_lang_path = Secure.add_lang_path;
+value add_doc_path = Secure.add_doc_path;
+value set_base_dir = Secure.set_base_dir;
+
+add_lang_path sharelib;
+add_lang_path Filename.current_dir_name;
+
 value cnt_dir = ref "";
 value images_url = ref "";
 
 value search_in_path p s =
-  loop p.val where rec loop =
+  loop (p ()) where rec loop =
     fun
     [ [d :: dl] ->
         let f = Filename.concat d s in
@@ -35,8 +29,8 @@ value search_in_path p s =
     | [] -> s ]
 ;
 
-value search_in_lang_path = search_in_path lang_path;
-value search_in_doc_path = search_in_path doc_path;
+value search_in_lang_path = search_in_path Secure.lang_path;
+value search_in_doc_path = search_in_path Secure.doc_path;
 
 (* Internationalization *)
 
@@ -84,10 +78,6 @@ value rec capitale s =
     | _ -> s ]
 ;
 
-ifdef OCAML_307 then
-value fcapitale (a : format 'a 'b 'c 'd) : format 'a 'b 'c 'd =
-  Obj.magic capitale a
-else
 value fcapitale (a : format 'a 'b 'c) : format 'a 'b 'c =
   Obj.magic capitale a
 ;
@@ -148,7 +138,7 @@ value plus_decl s =
   | None -> None ]
 ;
 
-value gen_decline conf wt s =
+value gen_decline wt s =
   let s1 = if s = "" then "" else " " ^ s in
   let len = String.length wt in
   if rindex wt '/' <> None then
@@ -166,7 +156,7 @@ value gen_decline conf wt s =
     | _ -> wt ^ decline 'n' s1 ]
 ;
 
-value transl_decline conf w s = gen_decline conf (transl conf w) s;
+value transl_decline conf w s = gen_decline (transl conf w) s;
 
 value gen_decline2 wt s1 s2 =
   let string_of =
@@ -218,18 +208,10 @@ value transl_a_of_gr_eq_gen_lev conf =
   gen_decline2 (transl_nth conf "%1 of %2" 1)
 ;
 
-ifdef OCAML_307 then
-value failed_format s : format 'a 'b 'c 'd = Obj.magic ("[" ^ s ^ "]")
-else
 value failed_format s : format 'a 'b 'c = Obj.magic ("[" ^ s ^ "]");
 
 value valid_format ini_fmt (r : string) =
-  let s : string =
-    ifdef OCAML_307 then
-      Obj.magic (ini_fmt : format 'a 'b 'c 'd)
-    else
-      Obj.magic (ini_fmt : format 'a 'b 'c)
-  in
+  let s : string = Obj.magic (ini_fmt : format 'a 'b 'c) in
   let rec loop i j =
     if i < String.length s - 1 && j < String.length r - 1 then
       match (s.[i], s.[i + 1], r.[j], r.[j + 1]) with
@@ -243,10 +225,7 @@ value valid_format ini_fmt (r : string) =
     else if j < String.length r - 1 then
       if r.[j] == '%' then failed_format s else loop i (j + 1)
     else
-      ifdef OCAML_307 then
-        (Obj.magic r : format 'a 'b 'c 'd)
-      else
-        (Obj.magic r : format 'a 'b 'c)
+      (Obj.magic r : format 'a 'b 'c)
   in
   loop 0 0
 ;
@@ -278,7 +257,7 @@ value ftransl_nth conf s p =
 ;
 
 value fdecline conf w s =
-  valid_format w (gen_decline conf (Obj.magic w : string) s)
+  valid_format w (gen_decline (Obj.magic w : string) s)
 ;
 
 (* *)
@@ -457,11 +436,11 @@ value p_getint env label =
 
 value parent_has_title base p =
   let a = aoi base p.cle_index in
-  match a.parents with
+  match parents a with
   [ Some ifam ->
       let cpl = coi base ifam in
-      let fath = poi base cpl.father in
-      let moth = poi base cpl.mother in
+      let fath = poi base (father cpl) in
+      let moth = poi base (mother cpl) in
       fath.access <> Private && fath.titles <> [] ||
       moth.access <> Private && moth.titles <> []
   | _ -> False ]
@@ -579,9 +558,7 @@ value pget (conf : config) base ip =
 ;
 
 value aget (conf : config) base ip =
-  if is_restricted conf base ip then
-    { parents = None;
-      consang = Adef.fix_of_float 0.0 }
+  if is_restricted conf base ip then no_ascend ()
   else base.data.ascends.get (Adef.int_of_iper ip)
 ;
 
@@ -627,7 +604,10 @@ value acces_n conf base n x =
     "p" ^ n ^ "=" ^ code_varenv (Name.lower first_name) ^ ";n" ^ n ^ "=" ^
       code_varenv (Name.lower surname) ^
       (if x.occ > 0 then ";oc" ^ n ^ "=" ^ string_of_int x.occ else "")
-  else "i" ^ n ^ "=" ^ string_of_int (Adef.int_of_iper x.cle_index)
+  else
+    "i" ^ n ^ "=" ^ string_of_int (Adef.int_of_iper x.cle_index) ^
+    (if conf.wizard && x.occ > 0 then ";oc" ^ n ^ "=" ^ string_of_int x.occ
+     else "")
 ;
 
 value acces conf base x = acces_n conf base "" x;
@@ -652,7 +632,12 @@ value gen_person_text (p_first_name, p_surname) conf base p =
       | (n, [nn :: _]) -> n ^ " <em>" ^ sou base nn ^ "</em>"
       | (n, []) -> n ]
     in
-    beg ^ " " ^ p_surname base p
+    let ali =
+      match p.aliases with
+      [ [alias :: _] -> " (" ^ sou base alias ^ ")"
+      | _ -> "" ]
+    in
+    beg ^ " " ^ p_surname base p ^ ali
 ;
 
 value gen_person_text_no_html (p_first_name, p_surname) conf base p =
@@ -673,11 +658,19 @@ value gen_person_text_without_surname (p_first_name, p_surname) conf base p =
   if is_hidden p then restricted_txt conf
   else if conf.hide_names && not (fast_auth_age conf p) then "x x"
   else
-    match (sou base p.public_name, p.qualifiers) with
-    [ (n, [nn :: _]) when n <> "" -> n ^ " <em>" ^ sou base nn ^ "</em>"
-    | (n, []) when n <> "" -> n
-    | (_, [nn :: _]) -> p_first_name base p ^ " <em>" ^ sou base nn ^ "</em>"
-    | (_, []) -> p_first_name base p ]
+    let s =
+      match (sou base p.public_name, p.qualifiers) with
+      [ (n, [nn :: _]) when n <> "" -> n ^ " <em>" ^ sou base nn ^ "</em>"
+      | (n, []) when n <> "" -> n
+      | (_, [nn :: _]) -> p_first_name base p ^ " <em>" ^ sou base nn ^ "</em>"
+      | (_, []) -> p_first_name base p ]
+    in
+    let ali =
+      match p.aliases with
+      [ [alias :: _] -> " (" ^ sou base alias ^ ")"
+      | _ -> "" ]
+    in
+    s ^ ali
 ;
 
 value person_text = gen_person_text std_access;
@@ -805,7 +798,7 @@ value surname_begin n =
   else
     let i =
       strip_spaces i where rec strip_spaces i =
-        if n.[i - 1] == ' ' then strip_spaces (pred i) else i
+        if i >= 1 && n.[i - 1] == ' ' then strip_spaces (pred i) else i
     in
     " (" ^ String.sub n 0 i ^ ")"
 ;
@@ -856,7 +849,7 @@ value input_to_semi ic =
 ;
 
 value base_path pref bname =
-  let pref = [base_dir.val :: pref] in
+  let pref = [Secure.base_dir () :: pref] in
   let bfile = List.fold_right Filename.concat pref bname in
   ifdef WIN95 then bfile
   else if Sys.file_exists bfile then bfile
@@ -881,8 +874,9 @@ value open_etc_file fname =
     search_in_lang_path
       (Filename.concat "etc" (Filename.basename fname ^ ".txt"))
   in
-  try Some (open_in fname1) with
-  [ Sys_error _ -> try Some (open_in fname2) with [ Sys_error _ -> None ] ]
+  try Some (Secure.open_in fname1) with
+  [ Sys_error _ ->
+      try Some (Secure.open_in fname2) with [ Sys_error _ -> None ] ]
 ;
 
 value macro_etc env imcom c =
@@ -924,7 +918,8 @@ value rec copy_from_etc env lang imcom ic =
                 if c = ']' then Buff.get len
                 else loop (Buff.store len c) (input_char ic)
             in
-            let s = Translate.inline lang '%' (macro_etc env imcom) s in
+            let (s, alt) = Translate.inline lang '%' (macro_etc env imcom) s in
+            let s = if alt then "[" ^ s ^ "]" else s in
             Wserver.wprint "%s" s
           else
             Wserver.wprint "[%c" c
@@ -1006,13 +1001,13 @@ value url_no_index conf base =
           if is_deleted_family (base.data.families.get i) then None
           else
             let cpl = base.data.couples.get i in
-            let p = pget conf base cpl.father in
+            let p = pget conf base (father cpl) in
             let f = scratch p.first_name in
             let s = scratch p.surname in
             if f = "" || s = "" then None
             else
               let oc = string_of_int p.occ in
-              let u = uget conf base cpl.father in
+              let u = uget conf base (father cpl) in
               let n =
                 loop 0 where rec loop k =
                   if u.family.(k) == Adef.ifam_of_int i then string_of_int k
@@ -1080,7 +1075,7 @@ value include_hed_trl conf base_opt suff =
     if Sys.file_exists fname then fname
     else base_path ["lang"] (conf.bname ^ suff)
   in
-  match try Some (open_in hed_fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_in hed_fname) with [ Sys_error _ -> None ] with
   [ Some ic ->
       let url () =
         match base_opt with
@@ -1099,23 +1094,31 @@ value include_hed_trl conf base_opt suff =
         [ Some i -> String.sub s (i + 1) (String.length s - i - 1)
         | None -> "" ]
       in
-      copy_from_etc [('p', pref); ('s', suff)] conf.lang conf.indep_command
-        ic
+      copy_from_etc [('p', pref); ('s', suff); ('t', fun _ -> commd conf)]
+        conf.lang conf.indep_command ic
   | None -> () ]
 ;
 
 value message_to_wizard conf =
-  if (conf.wizard || conf.just_friend_wizard) && conf.user <> "" then
-    let fname = conf.bname ^ "_" ^ conf.user ^ "_mess.txt" in
-    match try Some (open_in fname) with [ Sys_error _ -> None ] with
-    [ Some ic ->
-        try while True do { Wserver.wprint "%c" (input_char ic); } with
-        [ End_of_file -> close_in ic ]
-    | None -> () ]
+  if conf.wizard || conf.just_friend_wizard then
+    let print_file fname =
+      let fname = base_path ["etc"; conf.bname] (fname ^ ".txt") in
+      match try Some (Secure.open_in fname) with [ Sys_error _ -> None ] with
+      [ Some ic ->
+          try while True do { Wserver.wprint "%c" (input_char ic); } with
+          [ End_of_file -> close_in ic ]
+      | None -> () ]
+    in
+    do {
+      print_file "mess_wizard";
+      if conf.user <> "" then
+        print_file ("mess_wizard_" ^ conf.user)
+      else ();
+    }
   else ()
 ;
 
-value header_no_page_title conf title =
+value header_without_page_title conf title =
   do {
     html1 conf;
     Wserver.wprint "\
@@ -1140,7 +1143,7 @@ value header_no_page_title conf title =
 
 value header conf title =
   do {
-    header_no_page_title conf title;
+    header_without_page_title conf title;
     Wserver.wprint "<h1 align=center><font color=%s>" conf.highlight;
     title False;
     Wserver.wprint "</font></h1>\n";
@@ -1149,13 +1152,27 @@ value header conf title =
 
 value rheader conf title =
   do {
-    header_no_page_title conf title;
+    header_without_page_title conf title;
     Wserver.wprint "<center><h1><font color=%s>" red_color;
     title False;
     Wserver.wprint "</font></h1></center>\n";
   }
 ;
 
+value header_no_page_title conf title =
+  do {
+    header_without_page_title conf title;
+    match p_getenv conf.env "title" with
+    [ None | Some "" -> ()
+    | Some x ->
+        do {
+          Wserver.wprint "<h1 align=center><font color=%s>" conf.highlight;
+          Wserver.wprint "%s" x;
+          Wserver.wprint "</font></h1>\n"
+        } ];
+  }
+;
+
 value start_with s i p =
   i + String.length p <= String.length s &&
   String.lowercase (String.sub s i (String.length p)) = p
@@ -1366,7 +1383,7 @@ value gen_trailer with_logo conf =
     else
       Wserver.wprint "\
 <p>
-<a href=\"%s\"><img src=\"%s/gwlogo.gif\"
+<a href=\"%s\"><img src=\"%s/gwlogo.png\"
 alt=... width=64 height=72 align=right border=0></a>
 <br>
 " (commd conf) (image_prefix conf);
@@ -1376,7 +1393,7 @@ alt=... width=64 height=72 align=right border=0></a>
         do {
           html_p conf;
           Wserver.wprint "
-<hr><font size=-1><em>(c) Copyright 2003 INRIA -
+<hr><font size=-1><em>Copyright (c) 1998-2005 INRIA -
 GeneWeb %s</em></font>" Version.txt;
           html_br conf;
           ()
@@ -1494,15 +1511,15 @@ value specify_homonymous conf base p =
   | (_, []) ->
       let a = aget conf base p.cle_index in
       let ifam =
-        match a.parents with
+        match parents a with
         [ Some ifam ->
             let cpl = coi base ifam in
             let fath =
-              let fath = pget conf base cpl.father in
+              let fath = pget conf base (father cpl) in
               if p_first_name base fath = "?" then None else Some fath
             in
             let moth =
-              let moth = pget conf base cpl.mother in
+              let moth = pget conf base (mother cpl) in
               if p_first_name base moth = "?" then None else Some moth
             in
             Some (fath, moth)
@@ -1578,13 +1595,15 @@ value source_image_file_name bname str =
     List.fold_right Filename.concat [base_path ["src"] bname; "images"] str
   in
   let fname2 =
-    List.fold_right Filename.concat [base_dir.val; "src"; "images"] str
+    List.fold_right Filename.concat [Secure.base_dir (); "src"; "images"] str
   in
   if Sys.file_exists fname1 then fname1 else fname2
 ;
 
 value image_file_name str =
-  let fname1 = List.fold_right Filename.concat [base_dir.val; "images"] str in
+  let fname1 =
+    List.fold_right Filename.concat [Secure.base_dir (); "images"] str
+  in
   if Sys.file_exists fname1 then fname1
   else search_in_lang_path (Filename.concat "images" str)
 ;
@@ -1616,41 +1635,45 @@ value jpeg_image_size ic =
     let str = String.create 10 in do { really_input ic str 0 10; str }
   in
   if Char.code magic.[0] = 0xff && Char.code magic.[1] = 0xd8 &&
-     String.sub magic 6 4 = "JFIF" then
-    let rec loop () =
+     (let m = String.sub magic 6 4 in m = "JFIF" || m = "Exif") then
+    let exif_type = String.sub magic 6 4 = "Exif" in
+    let rec loop found =
       do {
         while Char.code (input_char ic) <> 0xFF do { () };
         let ch =
           loop (input_char ic) where rec loop ch =
             if Char.code ch = 0xFF then loop (input_char ic) else ch
         in
-        if Char.code ch >= 0xC0 && Char.code ch <= 0xC3 then do {
-          for i = 1 to 3 do { let _ = input_char ic in () };
-          let a = input_char ic in
-          let b = input_char ic in
-          let c = input_char ic in
-          let d = input_char ic in
-          let wid = Char.code c lsl 8 lor Char.code d in
-          let hei = Char.code a lsl 8 lor Char.code b in
-          Some (wid, hei)
-        }
+        if Char.code ch = 0xC0 || Char.code ch = 0xC3 then
+          if exif_type && not found then
+            loop True
+          else do {
+            for i = 1 to 3 do { let _ = input_char ic in () };
+            let a = input_char ic in
+            let b = input_char ic in
+            let c = input_char ic in
+            let d = input_char ic in
+            let wid = Char.code c lsl 8 lor Char.code d in
+            let hei = Char.code a lsl 8 lor Char.code b in
+            Some (wid, hei)
+          }
         else
           let a = input_char ic in
           let b = input_char ic in
           let len = Char.code a lsl 8 lor Char.code b in
-          if len < 2 then None
-          else do {
+          let len = if len >= 32768 then 0 else len in
+          do {
             for i = 1 to len - 2 do { let _ = input_char ic in () };
-            if Char.code ch <> 0xDA then loop () else None
+            if Char.code ch <> 0xDA then loop found else None
           }
       }
     in
-    loop ()
+    loop False
   else None
 ;
 
 value image_size fname =
-  match try Some (open_in_bin fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_in_bin fname) with [ Sys_error _ -> None ] with
   [ Some ic ->
       let r =
         try
@@ -1770,12 +1793,30 @@ value find_person_in_env conf base suff =
                    x.occ == occ)
                 xl
             in
-            Some r
+            if not conf.hide_names || authorized_age conf base r then Some r
+            else None
           with
           [ Not_found -> None ]
       | _ -> None ] ]
 ;
 
+value find_sosa_ref conf base =
+  match find_person_in_env conf base "z" with
+  [ Some p -> Some p
+  | None ->
+      match p_getenv conf.base_env "default_sosa_ref" with
+      [ Some n ->
+          if n = "" then None
+          else
+            match person_ht_find_all base n with
+            [ [ip] ->
+                let p = pget conf base ip in
+                if is_hidden p then None
+                else Some p
+            | _ -> None ]
+      | None -> None ] ]
+;
+
 value create_topological_sort conf base =
   match p_getenv conf.env "opt" with
   [ Some "no_tsfile" ->
@@ -1793,7 +1834,9 @@ value create_topological_sort conf base =
           in
           let r =
             match
-              try Some (open_in_bin tstab_file) with [ Sys_error _ -> None ]
+              try
+                Some (Secure.open_in_bin tstab_file) with
+                [ Sys_error _ -> None ]
             with
             [ Some ic ->
                 let r =
@@ -1814,7 +1857,7 @@ value create_topological_sort conf base =
                   base.data.visible.v_write ()
                 else ();
                 match
-                  try Some (open_out_bin tstab_file) with
+                  try Some (Secure.open_out_bin tstab_file) with
                   [ Sys_error _ -> None ]
                 with
                 [ Some oc ->
@@ -1841,11 +1884,11 @@ value branch_of_sosa conf base ip n =
       fun
       [ [] -> Some [(ip, sp) :: ipl]
       | [goto_fath :: nl] ->
-          match (aget conf base ip).parents with
+          match parents (aget conf base ip) with
           [ Some ifam ->
               let cpl = coi base ifam in
-              if goto_fath then loop [(ip, sp) :: ipl] cpl.father Male nl
-              else loop [(ip, sp) :: ipl] cpl.mother Female nl
+              if goto_fath then loop [(ip, sp) :: ipl] (father cpl) Male nl
+              else loop [(ip, sp) :: ipl] (mother cpl) Female nl
           | _ -> None ] ]
     in
     loop [] ip (pget conf base ip).sex (expand [] n)
@@ -2011,7 +2054,7 @@ exception Ok;
 value has_nephews_or_nieces conf base p =
   try
     let a = aget conf base p.cle_index in
-    match a.parents with
+    match parents a with
     [ Some ifam ->
         let des = doi base ifam in
         do {
@@ -2099,7 +2142,10 @@ value relation_txt conf sex fam =
   match fam.relation with
   [ NotMarried -> ftransl_nth conf "relationship%t to" is
   | Married | NoSexesCheck -> ftransl_nth conf "married%t to" is
-  | Engaged -> ftransl_nth conf "engaged%t to" is ]
+  | Engaged -> ftransl_nth conf "engaged%t to" is
+  | NoMention ->
+      let s = "%t " ^ transl conf "with" in
+      valid_format "%t" s ]
 ;
 
 value escache_value conf =
@@ -2125,7 +2171,7 @@ value std_date conf =
 ;
 
 value read_wf_trace fname =
-  match try Some (open_in fname) with [ Sys_error _ -> None ] with
+  match try Some (Secure.open_in fname) with [ Sys_error _ -> None ] with
   [ Some ic ->
       let r = ref [] in
       do {
@@ -2137,7 +2183,7 @@ value read_wf_trace fname =
 ;
 
 value write_wf_trace fname wt =
-  let oc = open_out fname in
+  let oc = Secure.open_out fname in
   do {
     List.iter (fun (dt, u) -> Printf.fprintf oc "%s %s\n" dt u) wt;
     close_out oc;
@@ -2227,3 +2273,21 @@ value unselected_bullets conf =
        [ Failure _ -> sl ])
     [] conf.env
 ;
+
+value short_f_month m =
+  match m with
+  [ 1 -> "VD"
+  | 2 -> "BR"
+  | 3 -> "FM"
+  | 4 -> "NI"
+  | 5 -> "PL"
+  | 6 -> "VT"
+  | 7 -> "GE"
+  | 8 -> "FL"
+  | 9 -> "PR"
+  | 10 -> "ME"
+  | 11 -> "TH"
+  | 12 -> "FT"
+  | 13 -> "JC"
+  | _ -> "" ]
+;
