@@ -1,15 +1,12 @@
-(* $Id: util.mli,v 4.26 2002/12/30 18:40:06 ddr Exp $ *)
-(* Copyright (c) 2001 INRIA *)
+(* $Id: util.mli,v 4.32 2004/12/14 09:30:18 ddr Exp $ *)
+(* Copyright (c) 1998-2005 INRIA *)
 
 open Def;
 open Config;
 
-value open_in : string -> in_channel;
-value open_in_bin : string -> in_channel;
-
-value lang_path : ref (list string);
-value doc_path: ref (list string);
-value base_dir : ref string;
+value add_lang_path : string -> unit;
+value add_doc_path : string -> unit;
+value set_base_dir : string -> unit;
 value cnt_dir : ref string;
 value images_url : ref string;
 value image_prefix : config -> string;
@@ -119,9 +116,6 @@ value surname_end : string -> string;
 
 value specify_homonymous : config -> base -> person -> unit;
 
-ifdef OCAML_307 then
-value valid_format : format 'a 'b 'c 'd -> string -> format 'a 'b 'c 'd
-else
 value valid_format : format 'a 'b 'c -> string -> format 'a 'b 'c;
 
 value transl : config -> string -> string;
@@ -130,20 +124,10 @@ value transl_nth_def : config -> string -> int -> int -> string;
 value transl_decline : config -> string -> string -> string;
 value transl_a_of_b : config -> string -> string -> string;
 value transl_a_of_gr_eq_gen_lev : config -> string -> string -> string;
-ifdef OCAML_307 then
-declare
-value ftransl : config -> format 'a 'b 'c 'd -> format 'a 'b 'c 'd;
-value ftransl_nth : config -> format 'a 'b 'c 'd -> int -> format 'a 'b 'c 'd;
-value fdecline : config -> format 'a 'b 'c 'd -> string -> format 'a 'b 'c 'd;
-value fcapitale : format 'a 'b 'c 'd -> format 'a 'b 'c 'd;
-end
-else
-declare
 value ftransl : config -> format 'a 'b 'c -> format 'a 'b 'c;
 value ftransl_nth : config -> format 'a 'b 'c -> int -> format 'a 'b 'c;
 value fdecline : config -> format 'a 'b 'c -> string -> format 'a 'b 'c;
 value fcapitale : format 'a 'b 'c -> format 'a 'b 'c;
-end;
 value nth_field : string -> int -> string;
 
 value cftransl : config -> string -> list string -> string;
@@ -152,10 +136,6 @@ value std_color : config -> string -> string;
 
 value index_of_sex : sex -> int;
 
-ifdef OCAML_307 then
-value relation_txt :
-  config -> sex -> family -> format (('a -> 'b) -> 'b) 'a 'b 'b
-else
 value relation_txt :
   config -> sex -> family -> format (('a -> 'b) -> 'b) 'a 'b;
 
@@ -164,6 +144,7 @@ value incorrect_request : config -> unit;
 value string_of_decimal_num : config -> float -> string;
 
 value find_person_in_env : config -> base -> string -> option person;
+value find_sosa_ref : config -> base -> option person;
 
 value quote_escaped : string -> string;
 
@@ -220,3 +201,5 @@ value print_pre_right : int -> string -> unit;
 
 value print_selection_bullet : config -> option (string * bool) -> unit;
 value unselected_bullets : config -> list int;
+
+value short_f_month : int -> string;
